Package: qwraps2
Title: Quick Wraps 2
Version: 0.6.3
Authors@R: c(
    person(given = "Peter", family = "DeWitt",  email = "dewittpe@gmail.com",          role = c("aut", "cre"), comment = c(ORCID = "0000-0002-6391-0795")),
    person(given = "Tell",  family = "Bennett", email = "tell.bennett@cuanschutz.edu", role = c("ctb"),        comment = c(ORCID = "0000-0003-1483-4236"))
    )
Description: A collection of (wrapper) functions the creator found useful
    for quickly placing data summaries and formatted regression results into
    '.Rnw' or '.Rmd' files. Functions for generating commonly used graphics,
    such as receiver operating curves or Bland-Altman plots, are also provided
    by 'qwraps2'.  'qwraps2' is an updated version of a package 'qwraps'. The
    original version 'qwraps' was never submitted to CRAN but can be found at
    <https://github.com/dewittpe/qwraps/>. The implementation and limited scope
    of the functions within 'qwraps2' <https://github.com/dewittpe/qwraps2/> is
    fundamentally different from 'qwraps'.
Depends: R (>= 3.5.0)
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/dewittpe/qwraps2/,
        https://www.peteredewitt.com/qwraps2/
BugReports: https://github.com/dewittpe/qwraps2/issues
Language: en-us
LazyData: true
Imports: ggplot2, knitr, Rcpp (>= 0.12.11), utils, xfun
Suggests: dplyr (>= 1.0.0), survival, covr, glmnet, rmarkdown
LinkingTo: Rcpp (>= 0.12.11), RcppArmadillo
VignetteBuilder: knitr, rmarkdown
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-06-17 19:53:21 UTC; dewittp
Author: Peter DeWitt [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6391-0795>),
  Tell Bennett [ctb] (ORCID: <https://orcid.org/0000-0003-1483-4236>)
Maintainer: Peter DeWitt <dewittpe@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-18 05:10:07 UTC

# Generated by roxygen2: do not edit by hand

S3method(cbind,qwraps2_qable)
S3method(confusion_matrix,default)
S3method(confusion_matrix,formula)
S3method(confusion_matrix,glm)
S3method(extract_fpvalue,lm)
S3method(extract_fstat,lm)
S3method(frmtci,data.frame)
S3method(frmtci,default)
S3method(frmtci,matrix)
S3method(frmtci,qwraps2_mean_ci)
S3method(ggplot2_extract_legend,ggplot)
S3method(print,qwraps2_confusion_matrix)
S3method(print,qwraps2_file_check)
S3method(print,qwraps2_ggplot2_extracted_legend)
S3method(print,qwraps2_mean_ci)
S3method(print,qwraps2_pkg_check)
S3method(print,qwraps2_qable)
S3method(print,qwraps2_set_diff)
S3method(print,qwraps2_summary_table)
S3method(qable,data.frame)
S3method(qable,matrix)
S3method(qacf,data.frame)
S3method(qacf,default)
S3method(qblandaltman,data.frame)
S3method(qblandaltman,default)
S3method(qkmplot,default)
S3method(qkmplot,qkmplot_data)
S3method(qkmplot,survfit)
S3method(qkmplot_build_data_frame,survfit)
S3method(qprc,default)
S3method(qprc,glm)
S3method(qprc,qwraps2_confusion_matrix)
S3method(qrmst,qkmplot_data)
S3method(qrmst,survfit)
S3method(qroc,default)
S3method(qroc,glm)
S3method(qroc,qwraps2_confusion_matrix)
S3method(qroc_build_data_frame,default)
S3method(qroc_build_data_frame,glm)
S3method(qsummary,data.frame)
S3method(qsummary,grouped_df)
S3method(rbind,qwraps2_qable)
S3method(summary_table,data.frame)
S3method(summary_table,grouped_df)
export("%s%")
export(CRANpkg)
export(GitHubpkg)
export(Githubpkg)
export(Gitlabpkg)
export(Rpkg)
export(StatStepribbon)
export(backtick)
export(confusion_matrix)
export(extract_fpvalue)
export(extract_fstat)
export(file_check)
export(frmt)
export(frmtci)
export(frmtp)
export(ggplot2_extract_legend)
export(gmean)
export(gmean_sd)
export(gsd)
export(gvar)
export(invlogit)
export(lazyload_cache_dir)
export(lazyload_cache_labels)
export(ll)
export(logit)
export(mean_ci)
export(mean_sd)
export(mean_se)
export(median_iqr)
export(n_perc)
export(n_perc0)
export(perc_n)
export(pkg_check)
export(qable)
export(qacf)
export(qblandaltman)
export(qblandaltman_build_data_frame)
export(qkmplot)
export(qkmplot_build_data_frame)
export(qkmplot_bulid_data_frame)
export(qprc)
export(qprc_build_data_frame)
export(qrmst)
export(qroc)
export(qroc_build_data_frame)
export(qsummary)
export(set_diff)
export(spin_comments)
export(stat_stepribbon)
export(summary_table)
export(traprule)
importFrom(Rcpp,sourceCpp)
useDynLib(qwraps2)

## ----label = "setup", include = FALSE-----------------------------------------
knitr::opts_chunk$set(collapse = TRUE)

## -----------------------------------------------------------------------------
set.seed(42)
library(qwraps2)
options(qwraps2_markup = "markdown")

## ----label = 'build_example_data_for_qable'-----------------------------------
d <- data.frame(
       group = sample(size = 15, paste0("grp", 1:5), replace = TRUE)
     , id = sample(size = 15, x = LETTERS)
     , V2 = rnorm(15)
     , V3 = rep(c(1, 2, NA), times = 5)
     )
d <- d[order(d$group, d$id), ]

## ----label = "kable1", results = "asis"---------------------------------------
knitr::kable(d, row.names = FALSE)

## -----------------------------------------------------------------------------
c(table(d$group))

## ----label = "qable1", results = "asis"---------------------------------------
qable(  x = d[, c("V2", "V3")]
      , rgroup = c(table(d$group)) # row group
      , rnames = d$id              # row names
)

## ----results = "asis"---------------------------------------------------------
model <-
  glm(spam ~
        word_freq_your + word_freq_conference + word_freq_business +
        char_freq_semicolon + char_freq_exclamation_point +
        capital_run_length_total + capital_run_length_longest
    , data = spambase
    , family = binomial()
  )

model_summary <-
  data.frame(
    parameter = names(coef(model))
  , odd_ratio = frmt(exp(coef(model)), digits = 3)
  , lcl       = frmt(exp(coef(model) + qnorm(0.025) * sqrt(diag(vcov(model)))), digits = 3)
  , ucl       = frmt(exp(coef(model) + qnorm(0.975) * sqrt(diag(vcov(model)))), digits = 3)
  , pval      = frmtp(summary(model)$coef[, 4])
  )

qable(model_summary[-1, c('odd_ratio', 'lcl', 'ucl', 'pval')]
      , rtitle = "Parameter"
      , rgroup = c("Word Frequency" = 3, "Character Frequency" = 2, "Capital Run Length" = 2)
      , rnames = c("Your", "Conference", "Business", ";", "!", "Total", "Longest")
      , kable_args = list(align = "lrrrr", caption = "Regression Model Summary")
      , cnames = c("Odds Ratio", "Lower Conf. Limit", "Upper Conf. Limit", "P-value")
      )

## -----------------------------------------------------------------------------
our_summary1 <-
  list("Miles Per Gallon" =
       list("min"       = ~ min(mpg),
            "max"       = ~ max(mpg),
            "mean (sd)" = ~ qwraps2::mean_sd(mpg)),
       "Displacement" =
       list("min"       = ~ min(disp),
            "median"    = ~ median(disp),
            "max"       = ~ max(disp),
            "mean (sd)" = ~ qwraps2::mean_sd(disp)),
       "Weight (1000 lbs)" =
       list("min"       = ~ min(wt),
            "max"       = ~ max(wt),
            "mean (sd)" = ~ qwraps2::mean_sd(wt)),
       "Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5))
       )

## ----label = "mtcars2_whole", results = "asis"--------------------------------
whole <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , qable_args = list(kable_args = list(caption = "mtcars2 data summary"))
  )
whole

## ----label = "mtcars2_by_cylf", results = "asis"------------------------------
by_cylf <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_factor"))
  )
by_cylf


## ----label = "mtcars2_by_cylc", results = "asis"------------------------------
by_cylc <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_character")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_character"))
  )
by_cylc

## ----label = "mtcars2_by_cyl_transmission", results = "asis"------------------
by_cyl_am <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor", "transmission")
  )
by_cyl_am

## ----label = "mtcars2_cbind", results = "asis"--------------------------------
both <- cbind(whole, by_cylf)
both

## ----label = "updated_both", results = "asis"---------------------------------

print(both,
      qable_args = list(
        rtitle = "ROW-TITLE",
        cnames = c("Col 0", "Col 1", "Col 2", "Col 3"),
        kable_args = list(
          align = "lcrcr",
          caption = "mtcars2 data summary - new caption"
        )
      ))

## -----------------------------------------------------------------------------
str(both)

## -----------------------------------------------------------------------------
# difference in means
mpvals <-
  sapply(
         list(mpg  = lm(mpg  ~ cyl_factor, data = mtcars2),
              disp = lm(disp ~ cyl_factor, data = mtcars2),
              wt   = lm(wt   ~ cyl_factor, data = mtcars2)),
         extract_fpvalue)

# Fisher test
fpval <- frmtp(fisher.test(table(mtcars2$gear, mtcars2$cyl_factor))$p.value)

## -----------------------------------------------------------------------------
both <- cbind(both, "P-value" = "")
both[grepl("mean \\(sd\\)", both[, 1]), "P-value"] <- mpvals
both[grepl("Forward Gears", both[, 1]), "P-value"] <- fpval

## ----label = "both_with_pvals", results = "asis"------------------------------
print(both, qable_args = list(kable_args = list(caption = "mtcars2 summary with p-values")))

## ----results = "asis"---------------------------------------------------------
gear_summary <-
  list("Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5)),
       "Transmission" =
       list("Automatic" = ~ qwraps2::n_perc0(am == 0),
            "Manual"    = ~ qwraps2::n_perc0(am == 1))
       )

gear_summary <-
setNames(gear_summary,
         c(
         paste("Forward Gears: ", frmtp(fisher.test(xtabs( ~ gear + cyl_factor, data = mtcars2))$p.value)),
         paste("Transmission: ",  frmtp(fisher.test(xtabs( ~ am + cyl_factor, data = mtcars2))$p.value)))
         )

summary_table(mtcars2, gear_summary, by = "cyl_factor")

## -----------------------------------------------------------------------------
t_mpg  <- summary_table(mtcars2, summaries = our_summary1["Miles Per Gallon"], by = "cyl_factor")
t_disp <- summary_table(mtcars2, summaries = our_summary1["Displacement"], by = "cyl_factor")
t_wt   <- summary_table(mtcars2, summaries = our_summary1["Weight (1000 lbs)"], by = "cyl_factor")

t_mpg  <- cbind(t_mpg,  "pvalue" = "")
t_disp <- cbind(t_disp, "pvalue" = "")
t_wt   <- cbind(t_wt,   "pvalue" = "")

t_mpg[ grepl("mean", t_mpg[, 1]),  "pvalue"] <- "mpg-pvalue"
t_disp[grepl("mean", t_disp[, 1]), "pvalue"] <- "disp-pvalue"
t_wt[  grepl("mean", t_wt[, 1]),   "pvalue"] <- "wt-pvalue"

## -----------------------------------------------------------------------------
rbind(t_mpg, t_disp, t_wt)
rbind(t_wt, t_disp, t_mpg)

## -----------------------------------------------------------------------------
new_data_frame <-
  data.frame(age = c(18, 20, 24, 17, 43),
             edu = c(1, 3, 1, 5, 2),
             rt  = c(0.01, 0.04, 0.02, 0.10, 0.06))

# Set a label for the variables
attr(new_data_frame$age, "label") <- "Age in years"
attr(new_data_frame$rt,  "label") <- "Reaction time"

# mistakenly set the attribute to name instead of label
attr(new_data_frame$edu, "name") <- "Education"

## -----------------------------------------------------------------------------
qsummary(new_data_frame)

## ----results = "asis"---------------------------------------------------------
summary_table(new_data_frame)

## -----------------------------------------------------------------------------
qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")])

## ----label="summary_table_mtcars2_default", results = "asis"------------------
summary_table(mtcars2[, c("mpg", "cyl_factor", "wt")])

## -----------------------------------------------------------------------------
new_summary <-
  qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")],
           numeric_summaries = list("Minimum" = "~ min(%s)",
                                    "Maximum" = "~ max(%s)"),
           n_perc_args = list(digits = 1, show_symbol = TRUE, show_denom = "always"))
str(new_summary)

## ----results = "asis"---------------------------------------------------------
summary_table(mtcars2, new_summary)

## -----------------------------------------------------------------------------
print(sessionInfo(), local = FALSE)

#'
# /*
if (interactive()) {
  devtools::load_all()
} else {
# */
library(qwraps2)
packageVersion("qwraps2")
# /*
}
# */
#'
#' # mtcars2
# /* {{{ */
#'
#' some blah blah blah
#'
# /* }}} */
#'
#'
#+ label = "chunk1"
2 + 2

#'
#'
#' blah blah
#'
#'

## ----label = "setup", include = FALSE-----------------------------------------
knitr::opts_chunk$set(collapse = TRUE)
library(qwraps2)
packageVersion("qwraps2")

## -----------------------------------------------------------------------------
set.seed(42)
n <- 250
x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
x1[1] <- runif(1)
x2[1] <- runif(1)
x3[1] <- runif(1)
x4[1] <- runif(1)

# white noise
Z.1 <- rnorm(n, 0, 1)
Z.2 <- rnorm(n, 0, 2)
Z.3 <- rnorm(n, 0, 5)

for(i in 2:n)
{
  x1[i] <- x1[i-1] + Z.1[i] - Z.1[i-1] + x4[i-1] - x2[i-1]
  x2[i] <- x2[i-1] - 2 * Z.2[i] + Z.2[i-1] - x4[i-1]
  x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z.3[i] + Z.3[i-1]
  x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
}
testdf <- data.frame(x1, x2, x3, x4)

# Base acf plot for one variable
acf(testdf$x1)

# qacf plot for one variable
qacf(testdf$x1)
qacf(testdf$x1, show_sig = TRUE)


## ----fig.width = 5, fig.height = 5--------------------------------------------
# more than one variable
acf(testdf)
qacf(testdf)
qacf(testdf, show_sig = TRUE)

## -----------------------------------------------------------------------------
acf_plot_data <- qacf(testdf)$data
head(acf_plot_data)

## -----------------------------------------------------------------------------
pefr_m1 <-
  cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
        "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

## -----------------------------------------------------------------------------
cor(pefr_m1)

ggplot2::ggplot(data = as.data.frame(pefr_m1)) +
  ggplot2::aes(x = Large, y = Mini) +
  ggplot2::geom_point() +
  ggplot2::xlab("Large Meter") +
  ggplot2::ylab("Mini Meter") +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(0, 800) +
  ggplot2::geom_abline(slope = 1)

## -----------------------------------------------------------------------------
# default plot
qblandaltman(pefr_m1)

# modified plot
ggplot2::last_plot() +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(-100, 100) +
  ggplot2::xlab("Average of two meters") +
  ggplot2::ylab("Difference in the measurements")

## -----------------------------------------------------------------------------
pefr_mini <-
  cbind(m1 = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"],
        m2 = pefr[pefr$measurement == 2 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

qblandaltman(pefr_mini)

## -----------------------------------------------------------------------------
# create a survfit object
require(survival)
leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x, data = survival::aml)

# base R km plot
survival:::plot.survfit(leukemia.surv, conf.int = TRUE, lty = 2:3, col = 1:2)


## ----fig.width = 5------------------------------------------------------------
# qkmplot
qkmplot(leukemia.surv, conf_int = TRUE)

## -----------------------------------------------------------------------------
leukemia_km_data <- qkmplot_bulid_data_frame(leukemia.surv)
head(leukemia_km_data, 3)


## ----fig.width = 5------------------------------------------------------------
qkmplot(leukemia_km_data)

## ----fig.width = 5------------------------------------------------------------
intonly_fit <- survival::survfit(survival::Surv(time, status) ~ 1, data = survival::aml)
survival:::plot.survfit(intonly_fit, conf.int = TRUE)
qkmplot(intonly_fit, conf_int = TRUE)

## -----------------------------------------------------------------------------
set.seed(42)
tidx <- runif(nrow(spambase)) <= 0.80
xidx <- which(names(spambase) != "spam")
yidx <- which(names(spambase) == "spam")
training_set   <- spambase[tidx, ]
validating_set <- spambase[!tidx, ]

## -----------------------------------------------------------------------------
logistic_model <-
  glm(
    spam ~ .
  , data = training_set
  , family = binomial()
  )

ridge_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 0
  )

lasso_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 1
  )

## -----------------------------------------------------------------------------
validating_set$logistic_model_prediction <-
  predict(
    logistic_model
  , newdata = validating_set
  , type = "response"
  )

validating_set$ridge_model_prediction <-
  as.numeric(
    predict(
      ridge_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

validating_set$lasso_model_prediction <-
  as.numeric(
    predict(
      lasso_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

## -----------------------------------------------------------------------------
cm1 <- confusion_matrix(spam ~ logistic_model_prediction, data = validating_set)
cm2 <- confusion_matrix(spam ~ ridge_model_prediction, data = validating_set)
cm3 <- confusion_matrix(spam ~ lasso_model_prediction, data = validating_set)

## -----------------------------------------------------------------------------
qroc(cm1) + ggplot2::ggtitle("Logisitic Model")
qroc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qroc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

## -----------------------------------------------------------------------------
roc_plot_data <-
  rbind(
      cbind(Model = paste("Logisitic; AUROC =", frmt(cm1$auroc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUROC =",     frmt(cm2$auroc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUROC =",     frmt(cm3$auroc, 3)), cm3$cm_stats)
    )

qroc(roc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::theme(legend.position = "bottom")

## -----------------------------------------------------------------------------
qprc(cm1) + ggplot2::ggtitle("Logisitic Model")
qprc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qprc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

prc_plot_data <-
  rbind(
      cbind(Model = paste("Logisitic; AUPRC =", frmt(cm1$auprc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUPRC =",     frmt(cm2$auprc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUPRC =",     frmt(cm3$auprc, 3)), cm3$cm_stats)
    )

qprc(prc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::geom_hline(yintercept = cm1$prevalence) +
  ggplot2::theme(legend.position = "bottom")

## ----label = "sessioninfo"----------------------------------------------------
sessionInfo()

#' # Spinner 1
#'
#' This file is a simple R script used to show the use of the spin comment
#' checking function.
# /* opening C-style comment
2 + 2
# end of the first comment */
#'
#'
exp(- 1i * pi)

#'
#'
#' More text and then an clossing comment, there is a missing openning.
# */

#'
#' What about openning without closing?
#'
# /*

# /* opened
sum(1:10)
# closed */

library(qwraps2)

m <- mean(mtcars$mpg)
s <- sd(mtcars$mpg)

# basic formatting
stopifnot(
  identical(
    mean_sd(mtcars$mpg) # should default to LaTeX markup
    ,
    qwraps2::frmt(m, digits = 2) %s% " $\\pm$ " %s% qwraps2::frmt(s, digits = 2)
  )
)

stopifnot(
  identical(
    mean_sd(mtcars$mpg, denote_sd = "paren")
    ,
    qwraps2::frmt(m, digits = 2) %s% " (" %s% qwraps2::frmt(s, digits = 2) %s% ")"
  )
)

stopifnot(
  identical(
    mean_sd(mtcars$mpg, markup = "markdown")
    ,
    qwraps2::frmt(m, digits = 2) %s% " &plusmn; " %s% qwraps2::frmt(s, digits = 2)
  )
)

stopifnot(
  identical(
    mean_sd(mtcars$mpg, show_n = "always", markup = "markdown")
    ,
    qwraps2::frmt(nrow(mtcars)) %s% "; " %s% qwraps2::frmt(m, digits = 2) %s% " &plusmn; " %s% qwraps2::frmt(s, digits = 2)
  )
)

stopifnot(
  identical(
    mean_sd(c(mtcars$mpg,NA), na_rm = TRUE, show_n = "always", markup = "markdown")
    ,
    qwraps2::frmt(nrow(mtcars)) %s% "; " %s% qwraps2::frmt(m, digits = 2) %s% " &plusmn; " %s% qwraps2::frmt(s, digits = 2)
  )
)

stopifnot(
  identical(
    mean_sd(c(mtcars$mpg, NA), na_rm = TRUE, show_n = "never", markup = "markdown")
    ,
    qwraps2::frmt(m, digits = 2) %s% " &plusmn; " %s% qwraps2::frmt(s, digits = 2)
  )
)


# error if show_n is not as expected
stopifnot(inherits(tryCatch(mean_sd(mtcars$mpg, show_n = TRUE), error = function(e) e), "error"))

# warning if na.rm is used instead of na_rm
stopifnot(inherits(tryCatch(mean_sd(mtcars$mpg, na.rm = TRUE), warning = function(w) w), "warning"))

library(qwraps2)
################################################################################
df <-
  data.frame(
      truth = c(1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0)
    , pred  = c(1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0)
  )

TP <- with(df, sum(truth == 1 & pred == 1))
TN <- with(df, sum(truth == 0 & pred == 0))
FP <- with(df, sum(truth == 0 & pred == 1))
FN <- with(df, sum(truth == 1 & pred == 0))

cmat <- confusion_matrix(df$truth, df$pred, thresholds = 1)$cm_stats
stopifnot(nrow(cmat) == 3L)
stopifnot(cmat[is.finite(cmat$threshold), "TP"] == TP)
stopifnot(cmat[is.finite(cmat$threshold), "TN"] == TN)
stopifnot(cmat[is.finite(cmat$threshold), "FP"] == FP)
stopifnot(cmat[is.finite(cmat$threshold), "FN"] == FN)

################################################################################
set.seed(42)
test  <- c(rep(1, 53), rep(0, 47))
truth <- c(rep(1, 20), rep(0, 33), rep(1, 10), rep(0, 37))

# S3 versions give same results, save for the call element
cm1 <- confusion_matrix(truth, test)
cm2 <- confusion_matrix(truth ~ test)
stopifnot(all.equal(cm1, cm2, check.attributes = FALSE))

con_mat <- confusion_matrix(truth = truth, predicted = test, thresholds = NULL)

# check names, if this fails and is due to a change in the code, make sure the
# documentation for the return of confusion_matrix has been updated.
stopifnot(
  names(con_mat) == c("cm_stats", "auroc", "auroc_ci", "auprc", "auprc_ci", "confint_method", "alpha", "prevalence")
)

# check names of cm_stats
stopifnot(
  names(con_mat$cm_stats) == c("threshold", "TP", "TN", "FP", "FN"
                               , "sensitivity", "sensitivity_lcl", "sensitivity_ucl"
                               , "specificity", "specificity_lcl", "specificity_ucl"
                               , "ppv", "ppv_lcl", "ppv_ucl"
                               , "npv", "npv_lcl", "npv_ucl"
                               , "accuracy", "accuracy_lcl", "accuracy_ucl"
                               , "youden"
                               , "mcc"
                               , "f1")
)

# Counts are as expected
stopifnot(con_mat[["cm_stats"]][["TP"]] == c(30, 30, 20, 0),
          con_mat[["cm_stats"]][["TN"]] == c(0, 0, 37, 70),
          con_mat[["cm_stats"]][["FP"]] == c(70, 70, 33, 0),
          con_mat[["cm_stats"]][["FN"]] == c(0, 0, 10, 30))

con_mat <- confusion_matrix(truth = truth, predicted = test, thresholds = 1)
stopifnot(con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["TP"]] == 20,
          con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["TN"]] == 37,
          con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["FP"]] == 33,
          con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["FN"]] == 10)

# Check Statistics
stopifnot(con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["sensitivity"]] == 20 / 30)
stopifnot(con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["specificity"]] == 37 / 70)
stopifnot(con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["ppv"]] == 20 / 53)
stopifnot(con_mat[["cm_stats"]][is.finite(con_mat$cm_stats$threshold), ][["npv"]] == 37 / 47)

# GERBIL convention for indeterminate sensitivity, PPV, and F1: if TP, FP, and
# FN are all 0 then all three metrics are 1; if TP is 0 and FP + FN > 0 then all
# three metrics are 0.
cm_one_fp <- confusion_matrix(truth = 0, predicted = 1, thresholds = 1)$cm_stats
cm_one_fp <- cm_one_fp[is.finite(cm_one_fp$threshold), ]
stopifnot(cm_one_fp$sensitivity == 0)
stopifnot(cm_one_fp$ppv == 0)
stopifnot(cm_one_fp$f1 == 0)

cm_one_fn <- confusion_matrix(truth = 1, predicted = 0, thresholds = 1)$cm_stats
cm_one_fn <- cm_one_fn[is.finite(cm_one_fn$threshold), ]
stopifnot(cm_one_fn$sensitivity == 0)
stopifnot(cm_one_fn$ppv == 0)
stopifnot(cm_one_fn$f1 == 0)

cm_empty_positive_prediction <- confusion_matrix(truth = 0, predicted = 0, thresholds = Inf)$cm_stats
cm_empty_positive_prediction <- cm_empty_positive_prediction[cm_empty_positive_prediction$threshold == Inf, ]
stopifnot(cm_empty_positive_prediction$sensitivity == 1)
stopifnot(cm_empty_positive_prediction$ppv == 1)
stopifnot(cm_empty_positive_prediction$f1 == 1)

# Boundary confidence intervals should not produce NaNs when proportions are
# exactly zero or one.
cm_perfect <- confusion_matrix(truth = c(0, 1), predicted = c(0.2, 0.8), thresholds = 0.5)
cm_perfect_stats <- cm_perfect$cm_stats
stopifnot(!any(is.nan(unlist(cm_perfect_stats[, grepl("_(lcl|ucl)$", names(cm_perfect_stats))]))))
stopifnot(!any(is.nan(cm_perfect$auroc_ci)))
stopifnot(!any(is.nan(cm_perfect$auprc_ci)))

# errors if non-binomial fit is passed
fit <- glm(mpg > 20 ~ wt, data = mtcars)
test <- tryCatch(confusion_matrix(fit), error = function(e) e)
stopifnot(!is.null(test))
stopifnot(inherits(test, "error"))

#' @title Count and Percentage
#'
#' @description A function for calculating and formatting counts and
#' percentages.
#'
#' @details
#' Default behavior will return the count of successes and the percentage as "N
#' (pp%).  If there are missing values in the input vector the omission of such
#' can be controlled by setting \code{na.rm = TRUE}.  In this case, the number
#' of non-missing values will be reported by default.  Omission of the
#' non-missing values can be controlled by setting \code{show_denom = "never"}.
#'
#' The function \code{n_perc0} uses a set of default arguments which may be
#' advantageous for use in building tables.
#'
#' @param x a 0:1 or boolean vector
#' @param digits digits to the right of the decimal point to return in the
#' percentage estimate.
#' @param na_rm if true, omit NA values
#' @param show_denom defaults to "ifNA".  Other options are "always" or "never".
#' @param show_symbol if TRUE (default) the percent symbol is shown, else it is suppressed.
#' @param markup latex or markdown
#' @param ... pass through
#'
#' @return a character vector of the formatted values
#'
#' @examples
#'
#' n_perc(c(0, 1,1, 1, 0, 0), show_denom = "always")
#' n_perc(c(0, 1,1, 1, 0, 0, NA), na_rm = TRUE)
#'
#' n_perc(mtcars$cyl == 6)
#'
#' set.seed(42)
#' x <- rbinom(4269, 1, 0.314)
#' n_perc(x)
#' n_perc(x, show_denom = "always")
#' n_perc(x, show_symbol = FALSE)
#'
#' # n_perc0 examples
#' n_perc0(c(0, 1,1, 1, 0, 0))
#' n_perc0(mtcars$cyl == 6)
#'
#' @rdname n_perc
#' @export
n_perc <- function(x,
                   digits = getOption("qwraps2_frmt_digits", 2),
                   na_rm = FALSE,
                   show_denom = "ifNA",
                   show_symbol = TRUE,
                   markup = getOption("qwraps2_markup", "latex"),
                   ...
                   ) {

  cl <- as.list(match.call())
  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning(paste(cl[[1]], "uses the argument `na_rm`, not `na.rm`."))
  }

  stopifnot(inherits(na_rm, "logical"))
  stopifnot(inherits(show_symbol, "logical"))
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_denom %in% c("always", "never", "ifNA"))
  stopifnot(inherits(x, "logical") | all(stats::na.omit(x) %in% c(0.0, 1.0)))

  d <- sum(!is.na(x))
  n <- sum(x, na.rm = na_rm)
  p <- frmt(100 * n/d, digits)

  if (show_denom == "never") {
    rtn <- paste0(frmt(as.integer(n)), " (", p, "%)")
  } else {
    if (show_denom =="always" | any(is.na(x))) {
      rtn <- paste0(frmt(as.integer(n)), "/", frmt(as.integer(d)), " (", p, "%)")
    } else {
      rtn <- paste0(frmt(as.integer(n)), " (", p, "%)")
    }
  }

  if (!show_symbol) {
    rtn <- gsub("%", "", rtn)
  }

  if (markup == "latex") {
    rtn <- gsub("%", "\\\\%", rtn)
  }

  return(rtn)
}

#' @rdname n_perc
#' @export
perc_n <- function(x,
                   digits = getOption("qwraps2_frmt_digits", 2),
                   na_rm = FALSE,
                   show_denom = "ifNA",
                   show_symbol = FALSE,
                   markup = getOption("qwraps2_markup", "latex"),
                   ...) {

  cl <- as.list(match.call())
  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning(paste(cl[[1]], "uses the argument `na_rm`, not `na.rm`."))
  }

  stopifnot(inherits(na_rm, "logical"))
  stopifnot(inherits(show_symbol, "logical"))
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_denom %in% c("always", "never", "ifNA"))
  stopifnot(inherits(x, "logical") | all(stats::na.omit(x) %in% c(0, 1)))

  d <- sum(!is.na(x))
  n <- sum(x, na.rm = na_rm)
  p <- frmt(100 * n/d, digits)

  if (show_denom =="always" | any(is.na(x))) {
    rtn <- paste0(p, "% (n = ", frmt(as.integer(d)), " non-missing)")
  } else {
    rtn <- paste0(p, "% (n = ", frmt(as.integer(d)), ")")
  }

  if (markup == "latex") {
    rtn <- gsub("%", "\\\\%", rtn)
  }

  return(rtn)
}

#' @rdname n_perc
#' @export
n_perc0 <- function(x,
                   digits = 0,
                   na_rm = FALSE,
                   show_denom = "never",
                   show_symbol = FALSE,
                   markup = getOption("qwraps2_markup", "latex"),
                   ...) {

  cl <- as.list(match.call())
  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning(paste(cl[[1]], "uses the argument `na_rm`, not `na.rm`."))
  }

  stopifnot(inherits(na_rm, "logical"))
  stopifnot(inherits(show_symbol, "logical"))
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_denom %in% c("always", "never", "ifNA"))
  stopifnot(inherits(x, "logical") | all(stats::na.omit(x) %in% c(0, 1)))

  n_perc(x, digits, na_rm, show_denom, show_symbol, markup)
}

#' @title Median and Inner Quartile Range
#'
#' @description A function for calculating and formatting the median and
#' inner quartile range of a data vector.
#'
#' @details
#' Given a numeric vector, \code{median_iqr} will return a character string with
#' the median and IQR.  Formatting of the output will be extended in
#' future versions.
#'
#' @param x a numeric vector
#' @param digits digits to the right of the decimal point to return.
#' @param na_rm if true, omit NA values
#' @param show_n defaults to "ifNA".  Other options are "always" or "never".
#' @param markup latex or markdown
#' @param ... pass through
#'
#' @return a character vector of the formatted values
#'
#' @examples
#' set.seed(42)
#' x <- rnorm(1000, 3, 4)
#' median(x)
#' quantile(x, probs = c(1, 3)/4)
#' median_iqr(x)
#' median_iqr(x, show_n = "always")
#'
#' x[187] <- NA
#' # median_iqr(x) ## Will error
#' median_iqr(x, na_rm = TRUE)
#'
#' @export
median_iqr <- function(x,
                       digits = getOption("qwraps2_frmt_digits", 2),
                       na_rm = FALSE,
                       show_n = "ifNA",
                       markup = getOption("qwraps2_markup", "latex"),
                       ...) {

  cl <- as.list(match.call())
  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning("qwraps2::median_iqr uses the argument `na_rm`, not `na.rm`.")
  }

  stopifnot(inherits(na_rm, "logical"))
  stopifnot(length(markup) == 1L)
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_n %in% c("ifNA", "always", "never"))

  n <- sum(!is.na(x))
  m <- stats::median(x, na.rm = na_rm)
  qs <- stats::quantile(x, probs = c(1, 3) / 4, na.rm = na_rm)

  rtn <- paste0(qwraps2::frmt(m, digits), " (",
                qwraps2::frmt(qs[1L], digits), ", ",
                qwraps2::frmt(qs[2L], digits), ")")

  if (show_n == "always" | (show_n == "ifNA" & any(is.na(x)))) {
    rtn <- paste0(qwraps2::frmt(as.integer(n), digits), "; ", rtn)
  }

  return(rtn)
}

#' @title Mean and Standard deviation
#'
#' @description A function for calculating and formatting means and
#' standard deviations.
#'
#' @details
#' Given a numeric vector, \code{mean_sd} will return a character string with
#' the mean and standard deviation.  Formatting of the output will be extended in
#' future versions.
#'
#' @param x a numeric vector
#' @param digits digits to the right of the decimal point to return in the
#' percentage estimate.
#' @param na_rm if true, omit NA values
#' @param show_n defaults to "ifNA".  Other options are "always" or "never".
#' @param denote_sd a character string set to either "pm" or "paren" for reporting 'mean
#' \eqn{\pm} sd' or 'mean (sd)'
#' @param markup character string with value "latex" or "markdown"
#' @param ... pass through
#'
#' @seealso \code{\link{gmean_sd}}, \code{\link{n_perc}},
#' \code{\link{median_iqr}}
#'
#' @return a character vector of the formatted values
#'
#' @examples
#' set.seed(42)
#' x <- rnorm(1000, 3, 4)
#' mean(x)
#' sd(x)
#' mean_sd(x)
#' mean_sd(x, show_n = "always")
#' mean_sd(x, show_n = "always", denote_sd = "paren")
#'
#' x[187] <- NA
#' mean_sd(x, na_rm = TRUE)
#'
#' @export
mean_sd <- function(x,
                    digits = getOption("qwraps2_frmt_digits", 2),
                    na_rm = FALSE,
                    show_n = "ifNA",
                    denote_sd = "pm",
                    markup = getOption("qwraps2_markup", "latex"),
                    ...) {

  cl <- as.list(match.call())
  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning("qwraps2::mean_sd uses the argument `na_rm`, not `na.rm`.")
  }
  stopifnot(inherits(na_rm, "logical"))
  stopifnot(length(markup) == 1L)
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_n %in% c("ifNA", "always", "never"))
  stopifnot(denote_sd %in% c("pm", "paren"))

  n <- sum(!is.na(x))
  m <- mean(x, na.rm = na_rm)
  s <- stats::sd(x, na.rm = na_rm)

  if (show_n == "always" | (show_n == "ifNA" & any(is.na(x)))) {
    rtn <- paste0(frmt(as.integer(n), digits), "; ", frmt(m, digits), " $\\pm$ ", frmt(s, digits))
  } else {
    rtn <- paste0(frmt(m, digits), " $\\pm$ ", frmt(s, digits))
  }

  if (denote_sd == "paren") {
    rtn <- gsub("\\$\\\\pm\\$\\s(.*)", "\\(\\1\\)", rtn)
  }

  if (markup == "markdown") {
    rtn <- gsub("\\$\\\\pm\\$", "&plusmn;", rtn)
  }

  return(rtn)
}

#' @title Means and Confidence Intervals
#'
#' @description A function for calculating and formatting means and
#' confidence interval.
#'
#' @details
#' Given a numeric vector, \code{mean_ci} will return a vector with the mean,
#' LCL, and UCL.  Using \code{frmtci} will be helpful for reporting the results
#' in print.
#'
#' @param x a numeric vector
#' @param na_rm if true, omit NA values
#' @param alpha defaults to \code{getOption('qwraps2_alpha', 0.05)}.  The
#' symmetric 100(1-alpha)\% CI will be determined.
#' @param qdist defaults to \code{qnorm}.  use \code{qt} for a Student t
#' intervals.
#' @param qdist.args list of arguments passed to \code{qdist}
#' @param ... not currently used
#'
#' @return a vector with the mean, lower confidence limit (LCL), and the upper
#' confidence limit (UCL).
#'
#' @seealso \code{\link{frmtci}}
#'
#' @examples
#' # using the standard normal for the CI
#' mean_ci(mtcars$mpg)
#'
#' # print it nicely
#' qwraps2::frmtci(mean_ci(mtcars$mpg))
#' qwraps2::frmtci(mean_ci(mtcars$mpg), show_level = TRUE)
#' qwraps2::frmtci(mean_ci(mtcars$mpg, alpha = 0.01), show_level = TRUE)
#'
#' # Compare to the ci that comes form t.test
#' t.test(mtcars$mpg)
#' t.test(mtcars$mpg)$conf.int
#' mean_ci(mtcars$mpg, qdist = stats::qt, qdist.args = list(df = 31))
#'
#' @export
mean_ci <- function(x,
                    na_rm = FALSE,
                    alpha = getOption("qwraps2_alpha", 0.05),
                    qdist = stats::qnorm,
                    qdist.args = list(),
                    ...
                    ) {
  cl <- as.list(match.call())
  if ("transform" %in% names(cl)) {
    stop("the transform option has been deprecated.  Its existence made it too easy to return incorrect values.  Think about the delta method.")
  }

  if (!("na_rm" %in% names(cl)) & ("na.rm" %in% names(cl))) {
    na_rm <- cl$na.rm
    warning("qwraps2::mean_ci uses the argument `na_rm`, not `na.rm`.")
  }

  stopifnot(inherits(na_rm, "logical"))

  m <- mean(x, na.rm = na_rm)
  s <- stats::sd(x,   na.rm = na_rm)
  n <- if (na_rm) { sum(!is.na(x)) } else { length(x) }

  qd <- match.fun(qdist)

  scores <- do.call(qd, c(list(p = c(alpha/2, 1 - alpha/2)), qdist.args))

  out <- c("mean" = m, "lcl"  = m + scores[1] * s / sqrt(n),  "ucl"  = m + scores[2] * s / sqrt(n))

  attr(out, 'alpha') <- alpha
  class(out) <- c("qwraps2_mean_ci", class(out))
  out
}

#' @export
#' @param ... arguments passed to \code{frmtci}.
#' @rdname mean_ci
print.qwraps2_mean_ci <- function(x, ...) {
  print(frmtci(x, ...))
}

#' @title ggplot2 tools
#'
#' @description A few handy tools for working with ggplot2.
#'
#' @details
#' The \code{ggplot2_extract_legend} function returns a list with the first
#' element being the legend and the second the original plot with the legend
#' omitted.
#'
#' @param x a ggplot object
#' @param ... not currently used
#'
#' @return a list with each elements
#' \describe{
#'   \item{legend}{}
#'   \item{plot}{the x}
#' }
#'
#' @examples
#' # a simple plot
#' my_plot <-
#'   ggplot2::ggplot(mtcars) +
#'   ggplot2::aes(x = wt, y = mpg, color = wt, shape = factor(cyl)) +
#'   ggplot2::geom_point()
#'
#' my_plot
#'
#' # extract the legend.  the return object is a list with two elements, the first
#' # element is the legend, the second is the original plot sans legend.
#' temp <- ggplot2_extract_legend(my_plot)
#'
#' # view just the legend.  This can be done via a call to the object or using
#' # plot or print.
#' temp
#' plot(temp[[1]])
#'
#' # the original plot without the legened
#' plot(temp[[2]])
#'
#' @export
#' @rdname ggplot2_tools
ggplot2_extract_legend <- function(x, ...) {
  UseMethod("ggplot2_extract_legend")
}

#' @export
ggplot2_extract_legend.ggplot <- function(x, ...) {
  ggg <- ggplot2::ggplotGrob(x)$grobs
  ggg <- ggg[[which(sapply(ggg, inherits, "gtable"))]]

  out <- list(legend = ggg,
              plot   = {x + ggplot2::theme(legend.position = "none")})

  class(out) <- c("qwraps2_ggplot2_extracted_legend", class(out))
  out
}

#' @export
print.qwraps2_ggplot2_extracted_legend <- function(x, ...) {
  graphics::plot(x[[1]])
}

#' Package Checks
#'
#' Check if a package is available on the local machine and optionally verify a
#' version.
#'
#' When writing a script that will be shared it is very likely that the multiple
#' authors/users will need to have a certain set of packages available to load.
#' The \code{pkg_check} function will verify that the packages are available to
#' load, this includes an optional version test, and attach the package to the
#' search list if requested.
#'
#' Testing for package versions will is done as \code{packageVersion(x) >=
#' version}.  If you need a specific version of a package you should explicitly
#' use \code{packageVersion(x) == version} in your script.  In general,
#' \code{pkg_check} is a handy tool in interactive sessions.  For a package you
#' should have package version documentation in the DESCRIPTION file.
#' For a script a base R solution of
#' \code{stopifnot(packageVersion("pkg") >= "x.y.z")}
#'
#' @param pkgs a character vector of package names to check for
#' @param versions an optional character vector, of the same length of
#' \code{pkgs} for the minimum version of the packages.
#' @param stop if \code{TRUE} then an error is thrown if any of the checks fail.
#' If \code{FALSE} (default) a logical is returned.
#'
#' @examples
#' # verify that the packages qwraps2, and ggplot2 are available (this should be
#' # TRUE if you have qwraps2 installed since ggplot2 is imported by qwraps2)
#' pkg_check(c("qwraps2", "ggplot2"))
#'
#' # show that the return is FALSE if a package is not available
#' pkg_check(c("qwraps2", "ggplot2", "NOT a PCKG"))
#'
#' # verify the version for just ggplot2
#' pkg_check(c("qwraps2", "ggplot2"), c(NA, "2.2.0"))
#'
#' # verify the version for qwraps2 (this is expected to fail as we are looking for
#' # version 42.3.14 which is far too advanced for the actual package development.
#' pkg_check(c("qwraps2", "ggplot2"), c("42.3.14", "2.2.0"))
#'
#' \dontrun{
#'   # You can have the function throw an error is any of the checks fail
#'   pkg_check(c("qwraps2", "ggplot2"),
#'             c("42.3.14", "2.2.0"),
#'             stop = TRUE)
#' }
#'
#' \dontrun{
#'   # If you have missing packages that can be installed from CRAN you may find
#'   # the following helpful.  If this code, with the needed edits, were placed at
#'   # the top of a script, then if a package is missing then the current version
#'   # from a target repository will be installed.  Use this set up with
#'   # discretion, others may not want the automatic install of packages.
#'   pkgs <- pkg_check("<packages to install>")
#'   if (!pkgs) {
#'     install.packages(attr(pkgs, "checks")[!attr(pkgs, "checks")$available][["package"]])
#'   }
#' }
#'
#'
#' @export
pkg_check <- function(pkgs, versions, stop = FALSE) {
  if (missing(versions)) {
    versions <- rep(NA, length(pkgs))
  } else {
    stopifnot(length(pkgs) == length(versions))
  }

  checks <-
    Map(function(p, v) {
          pkgv <- try(utils::packageVersion(p), silent = TRUE)

          if (inherits(pkgv, "try-error")) {
            out <- list(package = p, version = v, available = FALSE, installed_version = NA)
          } else {
            if (is.na(v)) {
              out <- list(package = p, version = v, available = TRUE, installed_version = as.character(pkgv))
            } else {
              out <- list(package = p, version = v, available = pkgv >= v, installed_version = as.character(pkgv))
            }
          }
          out
  },
           p = pkgs, v = versions)
  checks <- lapply(checks, as.data.frame)
  checks <- do.call(rbind, checks)

  out <- all(checks$available)
  attr(out, "checks") <- checks
  attr(out, "class")  <- "qwraps2_pkg_check"

  if (!out && stop) {
    print(out)
    stop("At least one package is not available.", call. = FALSE)
  }

  out
}

#' @export
print.qwraps2_pkg_check <- function(x, ...) {
  if (x) {
    print(TRUE)
  } else {
    attr(x, 'class') <- NULL
    print.default(x)
  }
  invisible(x)
}

#' Lazyload Cache
#'
#' Lazyload Cached label(s) or a whole directory.
#'
#' These functions are helpful for loading cached chunks into an interactive R
#' session.  Consider the following scenario: you use knitr and have cached
#' chunks for lazyloading.  You've created the document, close up your IDE and
#' move on to the next project.  Later, you revisit the initial project and need
#' to retrieve the objects created in the cached chunks.  One option is to
#' reevaluate all the code, but this could be time consuming.  The other option
#' is to use \code{lazyload_cache_labels} or \code{lazyload_cache_dir} to
#' quickly (lazy)load the chunks into an active R session.
#'
#' Use \code{lazyload_cache_dir} to load a whole directory of cached objects.
#'
#' Use \code{lazyload_cache_labels} to load an explicit set of cached chunks.
#'
#' @param path the path to the cache directory.
#' @param envir the environment to load the objects into
#' @param ask if TRUE ask the user to confirm loading each database found in
#' \code{path}
#' @param verbose if TRUE display the chunk labels being loaded
#' @param ... additional arguments passed to \code{\link[base]{list.files}}.
#'
#' @examples
#' # this example is based on \url{https://stackoverflow.com/a/41439691/1104685}
#'
#' # create a temp directory for a and place a .Rmd file within
#' tmpdir <- normalizePath(paste0(tempdir(), "/llcache_eg"), mustWork = FALSE)
#' tmprmd <- tempfile(pattern = "report", tmpdir = tmpdir, fileext = "Rmd")
#' dir.create(tmpdir)
#' oldwd <- getwd()
#' setwd(tmpdir)
#'
#' # build an example .Rmd file
#' # note that the variable x is created in the first chunk and then overwritten
#' # in the second chunk
#' cat("---",
#'     "title: \"A Report\"",
#'     "output: html_document",
#'     "---",
#'     "",
#'     "```{r first-chunk, cache = TRUE}",
#'     "mpg_by_wt_hp <- lm(mpg ~ wt + hp, data = mtcars)",
#'     "x_is_pi <- pi",
#'     "x <- pi",
#'     "```",
#'     "",
#'     "```{r second-chunk, cache = TRUE}",
#'     "mpg_by_wt_hp_am <- lm(mpg ~ wt + hp + am, data = mtcars)",
#'     "x_is_e <- exp(1)",
#'     "x <- exp(1)",
#'     "```",
#'     sep = "\n",
#'     file = tmprmd)
#'
#' # knit the file.  evaluate the chunks in a new environment so we can compare
#' # the objects after loading the cache.
#' kenv <- new.env()
#' knitr::knit(input = tmprmd, envir = kenv)
#'
#' # The objects defined in the .Rmd file are now in kenv
#' ls(envir = kenv)
#'
#' # view the cache
#' list.files(path = tmpdir, recursive = TRUE)
#'
#' # create three more environment, and load only the first chunk into the
#' # first, and the second chunk into the second, and then load all of the
#' # cache into the third
#' env1 <- new.env()
#' env2 <- new.env()
#' env3 <- new.env()
#'
#' lazyload_cache_labels(labels = "first-chunk",
#'                       path = paste0(tmpdir, "/cache"),
#'                       envir = env1)
#'
#' lazyload_cache_labels(labels = "second-chunk",
#'                       path = paste0(tmpdir, "/cache"),
#'                       envir = env2)
#'
#' lazyload_cache_dir(path = paste0(tmpdir, "/cache"), envir = env3)
#'
#' # Look at the contents of each of these environments
#' ls(envir = kenv)
#' ls(envir = env1)
#' ls(envir = env2)
#' ls(envir = env3)
#'
#' # The regression models are only fitted once and should be the same in all the
#' # environments where they exist, as should the variables x_is_e and x_is_pi
#' all.equal(kenv$mpg_by_wt_hp, env1$mpg_by_wt_hp)
#' all.equal(env1$mpg_by_wt_hp, env3$mpg_by_wt_hp)
#'
#' all.equal(kenv$mpg_by_wt_hp_am, env2$mpg_by_wt_hp_am)
#' all.equal(env2$mpg_by_wt_hp_am, env3$mpg_by_wt_hp_am)
#'
#' # The value of x, however, should be different in the different
#' # environments.  For kenv, env2, and env3 the value should be exp(1) as that
#' # was the last assignment value.  In env1 the value should be pi as that is
#' # the only relevant assignment.
#'
#' all.equal(kenv$x, exp(1))
#' all.equal(env1$x, pi)
#' all.equal(env2$x, exp(1))
#' all.equal(env3$x, exp(1))
#'
#' # cleanup
#' setwd(oldwd)
#' unlink(tmpdir, recursive = TRUE)
#'
#' @export
#' @rdname lazyload_cache
lazyload_cache_dir <- function(path = "./cache", envir = parent.frame(), ask = FALSE, verbose = TRUE, ...) {
  files <- do.call(list.files, list(path = path, pattern = "\\.rdx$", full.names = TRUE, ...))
  files <- gsub("\\.rdx", "", files)

  load_these <- rep(TRUE, length(files))

  if (ask) {
    for (i in seq_along(files)) {
      answer <- readline(prompt = paste("load database:", gsub("_[0-9a-f]{32}", "", files[i]), "(y/n)"))
      load_these[i] <- answer %in% c("Yes", "yes", "Y", "y")
    }
  }

  files <- files[load_these]

  if (!verbose) {
    sapply(files, lazyLoad, envir = envir)
  } else {
    sapply(files,
           function(x, envir) {
             message(paste("Lazyloading:", gsub("_[0-9a-f]{32}", "", x)))
             lazyLoad(x, envir = envir) },
           envir = envir)
  }

  invisible()
}

#' @param labels a character vector of the chunk labels to load.
#' @param filter an optional function passed to \code{\link[base:lazyload]{lazyLoad}}.
#' when called on a character vector of object names returns a logical vector:
#' only objects for which this is true will be loaded.
#' @export
#' @rdname lazyload_cache
lazyload_cache_labels <- function(labels, path = "./cache/", envir = parent.frame(), verbose = TRUE, filter, ...) {
  files <- do.call(list.files, list(path = path, pattern = paste0("^(", paste(labels, collapse = "|"), ")_[0-9a-f]{32}\\.rdx$"), full.names = TRUE, ...))
  files <- gsub("\\.rdx$", "", files)

  lfound <- sapply(lapply(labels, grepl, x = files), any)

  if (!all(lfound)) {
    files <- do.call(list.files, list(path = path, pattern = "_[0-9a-f]{32}\\.rdx$", full.names = FALSE, ...))
    files <- gsub("_[0-9a-f]{32}\\.rdx$", "", files)
    message(paste0("label(s)\n", paste(paste0("  ", labels[!lfound]), collapse = "\n"), "\nnot found in path '", path, "\n\n",
                   "Available labels:\n", paste(paste0("  ", files), collapse = "\n")))
    warning("Nothing loaded", call. = FALSE)
  } else {

    if (!verbose) {
      sapply(files, lazyLoad, envir = envir, filter = filter)
    } else {
      sapply(files,
             function(x, envir, filter) {
               message(paste("Lazyloading", x))
               lazyLoad(x, envir = envir, filter = filter) },
             envir = envir,
             filter = filter)
    }
  }
  invisible()
}

library(qwraps2)
qs <- qsummary(mtcars2[c("cyl", "cyl_factor", "cyl_character")])

stopifnot(identical(qs[["cyl"]],
                    list("minimum"      = ~ qwraps2::frmt(min(cyl)),
                         "median (IQR)" = ~ qwraps2::median_iqr(cyl),
                         "mean (sd)"    = ~ qwraps2::mean_sd(cyl),
                         "maximum"      = ~ qwraps2::frmt(max(cyl)))))

stopifnot(identical(qs[["cyl_factor"]],
                  list("6 cylinders" = ~ qwraps2::n_perc(cyl_factor == "6 cylinders", digits = 0, show_symbol = FALSE),
                       "4 cylinders" = ~ qwraps2::n_perc(cyl_factor == "4 cylinders", digits = 0, show_symbol = FALSE),
                       "8 cylinders" = ~ qwraps2::n_perc(cyl_factor == "8 cylinders", digits = 0, show_symbol = FALSE))))

stopifnot(identical(qs[["cyl_character"]],
                  list("4 cylinders" = ~ qwraps2::n_perc(cyl_character == "4 cylinders", digits = 0, show_symbol = FALSE),
                       "6 cylinders" = ~ qwraps2::n_perc(cyl_character == "6 cylinders", digits = 0, show_symbol = FALSE),
                       "8 cylinders" = ~ qwraps2::n_perc(cyl_character == "8 cylinders", digits = 0, show_symbol = FALSE))))

qs_quoted <- qsummary(data.frame(x = c("A's", 'B "quoted"', "plain")))
stopifnot(identical(names(qs_quoted$x), c("A's", 'B "quoted"', "plain")))

st_quoted <- summary_table(data.frame(x = c("A's", 'B "quoted"', "plain")))
stopifnot(inherits(st_quoted, "qwraps2_summary_table"))

if ( requireNamespace("dplyr", quietly = TRUE) ) {
  # expect warnings
  temp <- dplyr::group_by(mtcars, am, vs)

  test <- tryCatch(summary_table(temp), warning = function(w) w)
  stopifnot(identical(inherits(test, "warning"), TRUE) &
            identical(grepl("grouped_df detected", test$message), TRUE))

  test <- tryCatch(summary_table(temp, by = "am"), warning = function(w) w)
  stopifnot(identical(inherits(test, "warning"), TRUE) &
            identical(grepl("You've passed", test$message), TRUE))

# dplyr::group_by equivalent output
  out1 <- capture.output(suppressWarnings(summary_table(temp)))
  out2 <- capture.output(summary_table(mtcars, by = c("am", "vs")))
  stopifnot(identical(out1, out2))
}

################################################################################
# check the number of columns is as expected for summary tables
stab <- summary_table(mtcars, by = c("am", "vs"))
stopifnot(identical(ncol(stab), 5L))

stab_am <- summary_table(mtcars, by = "am")
stab_vs <- summary_table(mtcars, by = "vs")
stab_cbind <- cbind(stab_am, stab_vs)
stopifnot(identical(ncol(stab_am) + ncol(stab_vs) - 1L, ncol(stab_cbind)))

################################################################################
# check that the print method will update the attributes
stab <- summary_table(mtcars[, c("mpg", "cyl", "am")], by = c("am"), qable_args = list(rtitle = "RTITLE"))
attributes(stab)

################################################################################
##                                End of File                                 ##
################################################################################

#' @title Format Wrappers
#'
#' @description Functions for formatting numeric values for consistent display
#' in reports.
#'
#' @details
#' `frmt` was originally really just a wrapper for the \code{formatC}.  It has
#' extended functionality now as I have found common uses cases.
#'
#' `frmtp` formats P-values per journal
#' requirements.  As I work on papers aimed at different journals, the
#' formatting functions will be extended to match.
#'
#' Default settings are controlled through the function arguments but should be
#' set via \code{options()}.
#'
#' Default settings report the P-value exactly if P >
#' \code{getOption("qwraps2_frmtp_digits", 4)} and reports
#' P < \code{10^-(getOption("qwraps2_frmtp_digits", 2))} otherwise.  By the
#' leading zero is controlled via
#' \code{getOption("qwraps2_frmtp_leading0", TRUE)}
#' and a upper or lower case P is controlled by
#' \code{getOption("qwraps2_frmtp_case", "upper")}.  These options are ignored
#' if \code{style != "default"}.
#'
#' Journals with predefined P-value formatting are noted in the
#' \pkg{\link{qwraps2}} documentation.
#'
#' `frmtci` takes a \code{matrix}, or \code{data.frame}, with a point estimate
#' and the lcl and ucl and formats a string for reporting.  est (lcl, ucl) is
#' the default.  The confidence level can be added to the string, e.g., "est
#' (95% CI: lcl, ucl)" with \code{show_level = TRUE} or by setting your own
#' format.
#'
#' `frmtcip` expects four values, est, lcl, ucl, and p-value.  The resulting
#' sting will be of the form "est (lcl, ucl; p-value)".
#'
#' The `Rpkg`, `CRANpkg`, and `Githubpkg` functions are used to help make
#' documenting packages stylistically consistent and with valid urls.  These
#' functions were inspired by similar ones found in the BioConductor BiocStyle
#' package.
#'
#' @seealso
#' \code{\link{formatC}}
#'
#' @param x a vector of numbers or a numeric matrix to format.
#' @param digits number of digits, including trailing zeros, to the right of the
#' decimal point.  This option is ignored if \code{is.integer(x) == TRUE)}.
#' @param append a character string to append to the formatted number.  This is
#' particularly useful for percentages or adding punctuation to the end of the
#' formatted number.  This should be a vector of length 1, or equal to the
#' length of \code{x}.
#'
#' @return a character vector of the formatted numbers
#'
#' @examples
#'
#' ### Formatting numbers
#' integers <- c(1234L, 9861230L)
#' numbers  <- c(1234,  9861230)
#' frmt(integers)  # no decimal point
#' frmt(numbers)   # decimal point and zeros to the right
#'
#' numbers <- c(0.1234, 0.1, 1234.4321, 0.365, 0.375)
#' frmt(numbers)
#'
#' # reporting a percentage
#' frmt(17/19 * 100, digits = 2, append = "%")   # good for markdown
#' frmt(17/19 * 100, digits = 2, append = "\\%") # good for LaTeX
#'
#' # append one character
#' frmt(c(1, 2, 3)/19 * 100, digits = 2, append = "%")
#'
#' # append different characters
#' frmt(c(1, 2, 3)/19 * 100, digits = 2, append = c("%;", "%!", "%."))
#'
#' ### Formatting p-values
#' ps <- c(0.2, 0.001, 0.00092, 0.047, 0.034781, 0.0000872, 0.787, 0.05, 0.043)
#' # LaTeX is the default markup language
#' cbind("raw"      = ps,
#'       "default"  = frmtp(ps),
#'       "3lower"   = frmtp(ps, digits = 3, case = "lower"),
#'       "PediDent" = frmtp(ps, style = "pediatric_dentistry"))
#'
#' ### Using markdown
#' cbind("raw"      = ps,
#'       "default"  = frmtp(ps, markup = "markdown"),
#'       "3lower"   = frmtp(ps, digits = 3, case = "lower", markup = "markdown"),
#'       "PediDent" = frmtp(ps, style = "pediatric_dentistry", markup = "markdown"))
#'
#' @export
#' @rdname frmt
frmt <- function(x
                 , digits = getOption("qwraps2_frmt_digits", 2)
                 , append = NULL
                 ) {
  rtn <-
    sapply(x,
           function(xx) {
             if (is.integer(xx)) {
               formatC(xx, format = "d", big.mark = ",")
             } else {
               formatC(xx, digits = digits, format = "f", big.mark = ",")
             }
           })

  if (!is.null(append)) {
    if (length(append) == 1L) {
      rtn <- paste0(rtn, append)
    } else if (length(append) == length(x)) {
      rtn <- paste0(rtn, append)
    } else {
      stop("length(append) != length(x)")
    }
  }
  rtn
}


#' @export
#' @rdname frmt
#' @param style a character string indicating a specific journal requirements
#' for p-value formatting.
#' @param markup a character string indicating if the output should be latex or
#' markup.
#' @param case a character string indicating if the output should be upper case
#' or lower case.
#' @param leading0 boolean, whether or not the p-value should be reported as
#' 0.0123 (TRUE, default), or .0123 (FALSE).
frmtp <- function(x,
                  style    = getOption("qwraps2_journal", "default"),
                  digits   = getOption("qwraps2_frmtp_digits", 4),
                  markup   = getOption("qwraps2_markup", "latex"),
                  case     = getOption("qwraps2_frmtp_case", "upper"),
                  leading0 = getOption("qwraps2_frmtp_leading0", TRUE)) {

  rtn <-
    switch(style,
           default               = frmtp_default(x, digits, case, leading0),
           obstetrics_gynecology = frmtp_obstetrics_gynecology(x),
           pediatric_dentistry   = frmtp_pediatric_dentistry(x))

  if (markup == "latex") {
    rtn <- paste0("$", rtn, "$")
  } else if (markup == "markdown") {
    rtn <- gsub("(P|p)", "\\*\\1\\*", rtn)
  }
  return(rtn)
}

frmtp_default <- function(x, digits, case, leading0) {
  p_cutoff <- 10^-digits

  sapply(x,
         function(xx) {
           if (xx < p_cutoff) {
             p_val <- paste("P <", formatC(p_cutoff, format = "g"))
           } else {
             p_val <- paste0("P = ", formatC(xx, digits = digits, format = "f"))
           }

           if (case == "lower") {
             p_val <- tolower(p_val)
           }

           if (!leading0) {
             p_val <- gsub("0\\.", "\\.", p_val)
           }

           p_val
         })
}

frmtp_pediatric_dentistry <- function(x) {
  sapply(x, function(xx) {

         if (xx < 0.001) {
           p_val <- "< .001"
         } else if (xx < 0.01) {
           p_val <- paste0("= ", formatC(xx, digits = 3, format = "f"))
         } else if (xx < 0.05 & round(xx, 2) == 0.05) {
           p_val <- paste0("= ", formatC(xx, digits = 3, format = "f"))
         } else {
           p_val <- paste0("= ", formatC(xx, digits = 2, format = "f"))
                             }
         paste0("P ", gsub("0\\.", "\\.", p_val))
      })
}

frmtp_obstetrics_gynecology <- function(x) {
  sapply(x, function(xx) {

         if (xx < 0.001) {
           p_val <- "< .001"
         } else {
           p_val <- paste0("= ", formatC(xx, digits = 3, format = "f"))
                             }
         paste0("P ", gsub("0\\.", "\\.", p_val))
      })
}


#' @export
#' @rdname frmt
#'
#' @param est the numeric index of the vector element or the matrix column
#' containing the point estimate.
#' @param lcl the numeric index of the vector element or the matrix column
#' containing the lower confidence limit.
#' @param ucl the numeric index of the vector element or the matrix column
#' containing the upper confidence limit.
#' @param format a string with "est" "lcl", and "ucl" to denote the location of
#' the estimate, lower confidence limit, and upper confidence limit for the
#' formatted string.  Defaults to "est (lcl, ucl)".
#' @param show_level defaults to FALSE.  If TRUE and \code{format} is the
#' default, then
#' "100*(1-options()$qwraps2_alpha)% CI:" will be placed between the left
#' parenthesis and the lcl.  If set to a string, then the given string will be
#' placed between the left parenthesis and the lcl.  If the \code{format} is not
#' the default, then this argument is ignored.
#' @param ... args passed to frmt
#' @examples
#' # Formatting the point estimate and confidence interval
#' # for a set of three values
#' temp <- c(a = 1.23, b = .32, CC = 1.78)
#' frmtci(temp)
#'
#' # show level uses getOption("qwraps2_alpha", 0.05)
#' frmtci(temp, show_level = TRUE)
#'
#' # note that the show_level will be ignored in the following
#' frmtci(temp, format = "est ***lcl, ucl***", show_level = TRUE)
#'
#' # show_level as a character
#' frmtci(temp, show_level = "confidence between: ")
#'
#' # For a matrix: the numbers in this example don't mean anything, but the
#' # formatting should.
#' temp2 <- matrix(rnorm(12), nrow = 4,
#'                 dimnames = list(c("A", "B", "C", "D"), c("EST", "LOW", "HIGH")))
#' temp2
#' frmtci(temp2)
#'
#' # similar for a data.frame
#' df2 <- as.data.frame(temp2)
#' frmtci(df2)
#'
#' @export
frmtci <- function(x, est = 1, lcl = 2, ucl = 3, format = "est (lcl, ucl)", show_level = FALSE, ...) {
  UseMethod("frmtci")
}

#' @export
frmtci.default <- function(x, est = 1, lcl = 2, ucl = 3, format = "est (lcl, ucl)", show_level = FALSE, ...) {

  .est <- qwraps2::frmt(x[est], ...)
  .lcl <- qwraps2::frmt(x[lcl], ...)
  .ucl <- qwraps2::frmt(x[ucl], ...)

  if (format == "est (lcl, ucl)") {
    out <- sub("ucl", .ucl, sub("lcl", .lcl, sub("est", .est, format)))
    if (is.character(show_level)) {
      lvl <- paste0("(", show_level)
      out <- sub("\\(", lvl, out)
    } else if(is.logical(show_level)) {
      if (show_level) {
      lvl <- paste0("(", 100 * (1 - getOption("qwraps2_alpha", 0.05)), "% CI: ")
      out <- sub("\\(", lvl, out)
      }
    }
  } else {
    out <- sub("ucl", .ucl, sub("lcl", .lcl, sub("est", .est, format)))
  }
  out
}

#' @export
frmtci.matrix <- function(x, est = 1, lcl = 2, ucl = 3, format = "est (lcl, ucl)", show_level = FALSE, ...) {
  apply(x, 1, frmtci.default, est = est, lcl = lcl, ucl = ucl, format = format, show_level = show_level, ...)
}

#' @export
frmtci.data.frame <- function(x, est = 1, lcl = 2, ucl = 3, format = "est (lcl, ucl)", show_level = FALSE, ...) {
  frmtci.matrix(as.matrix(x[, c(est, lcl, ucl)]), est = 1, lcl = 2, ucl = 3, format = format, show_level = show_level, ...)
}

#' @export
frmtci.qwraps2_mean_ci <- function(x, est = 1, lcl = 2, ucl = 3, format = "est (lcl, ucl)", show_level = FALSE, ...) {

  .est <- qwraps2::frmt(x[est], ...)
  .lcl <- qwraps2::frmt(x[lcl], ...)
  .ucl <- qwraps2::frmt(x[ucl], ...)

  if (format == "est (lcl, ucl)") {
    out <- sub("ucl", .ucl, sub("lcl", .lcl, sub("est", .est, format)))

    if (show_level) {
      lvl <- paste0("(", 100 * (1 - attr(x, "alpha")), "% CI: ")
      out <- sub("\\(", lvl, out)
    }
  } else {
    out <- sub("ucl", .ucl, sub("lcl", .lcl, sub("est", .est, format)))
  }
  out
}

#' Formatting Style on URLs for packages on CRAN, Github, and Gitlab.
#'
#' Functions for controlling the look of package names in markdown created
#' vignettes and easy curating of URLs for the packages.
#'
#' @param pkg The name of the package, will work as a quoted or raw name.
#' @param username username for Github.com or Gitlab.com
#' @examples
#'
#' Rpkg(qwraps2)
#' Rpkg("qwraps2")
#'
#' CRANpkg(qwraps2)
#' CRANpkg("qwraps2")
#'
#' Githubpkg(qwraps2, "dewittpe")
#' Githubpkg("qwraps2", dewittpe)
#'
#' Gitlabpkg(qwraps2, "dewittpe")
#' Gitlabpkg("qwraps2", dewittpe)
#'
#' @name Rpkg
NULL

#' @export
#' @rdname Rpkg
Rpkg <- function(pkg) {
  pkg <- deparse(substitute(pkg))
  pkg <- gsub("\"|\'", "", pkg)
  sprintf("*%s*", pkg)
}

#' @export
#' @rdname Rpkg
CRANpkg <- function(pkg) {
  pkg <- deparse(substitute(pkg))
  pkg <- gsub("\"|\'", "", pkg)
  pkg <- sprintf("[%s](https://cran.r-project.org/package=%s)", pkg, pkg)
  cl <- list(quote(Rpkg), pkg = pkg)
  eval(as.call(cl))
}

#' @export
#' @rdname Rpkg
Githubpkg <- function(pkg, username) {
  pkg <- deparse(substitute(pkg))
  pkg <- gsub("\"|\'", "", pkg)
  usn <- deparse(substitute(username))
  usn <- gsub("\"|\'", "", usn)
  pkg <- sprintf("[%s](https://github.com/%s/package=%s)", pkg, usn, pkg)
  cl <- list(quote(Rpkg), pkg = pkg)
  eval(as.call(cl))
}

#' @export
#' @rdname Rpkg
GitHubpkg <- function(pkg, username) {
  Githubpkg(pkg, username)
}

#' @export
#' @rdname Rpkg
Gitlabpkg <- function(pkg, username) {
  pkg <- deparse(substitute(pkg))
  pkg <- gsub("\"|\'", "", pkg)
  usn <- deparse(substitute(username))
  usn <- gsub("\"|\'", "", usn)
  pkg <- sprintf("[%s](https://gitlab.com/%s/package=%s)", pkg, usn, pkg)
  cl <- list(quote(Rpkg), pkg = pkg)
  eval(as.call(cl))
}

#' Backtick
#'
#' Encapsulate a string in backticks. Very helpful for inline code in
#' \code{\link[knitr]{spin}} scripts.
#'
#' @param x the thing to be deparsed and encapsulated in backticks
#' @param dequote remove the first and last double or signal quote form \code{x}
#'
#' @examples
#' backtick("a quoted string")
#' backtick(no-quote)
#' backtick(noquote)
#'
#' @export
backtick <- function(x, dequote = FALSE) {
  x <- deparse(substitute(x))
  if (dequote) {
    x <- sub("^(\"|\')", "", x)
    x <- sub("(\"|\')$", "", x)
  }
  sprintf("`%s`", x)
}

#' @title List Object Aliases
#'
#' @description Aliases for \code{\link[base]{ls}} providing additional details.
#'
#' @references The basis for this work came from a Stack Overflow posting:
#' \url{https://stackoverflow.com/q/1358003/1104685}
#'
#' @seealso \code{\link[base]{ls}}
#'
#' @param pos specifies the environment as a position in the search list
#' @param pattern an optional regular expression.  Only names matching
#' \code{pattern} are returned.  \code{\link[utils]{glob2rx}} can be used to
#' convert wildcard patterns to regular expressions.
#' @param order_by a character, order the results by \dQuote{object},
#' \dQuote{size} (default), \dQuote{class}, \dQuote{rows}, or \dQuote{columns}.
#' @param decreasing logical, defaults to \code{TRUE}, decreasing order? passed
#' to \code{\link[base]{order}}.
#'
#' @return a data.frame with columns
#' \itemize{
#'  \item object: name of the object
#'  \item class: class, or mode if class is not present, of the object
#'  \item size: approximate size, in bytes, of the object in memory
#'  \item rows: number of rows for data.frames or matrices, or the number of
#'  elements for a list like structure
#'  \item columns: number of columns for data.frames or matrices
#' }
#'
#' @examples
#' # View your current workspace
#' \dontrun{
#' ls()
#' ll()
#' }
#'
#' # View another environment
#' e <- new.env()
#' ll(e)
#'
#' e$fit <- lm(mpg ~ wt, mtcars)
#' e$fit2 <- lm(mpg ~ wt + am + vs, data = mtcars)
#' e$x <- rnorm(1e5)
#' e$y <- runif(1e4)
#' e$z <- with(e, x * y)
#' e$w <- sum(e$z)
#'
#' ls(e)
#' ll(e)
#'
#' @export
ll <- function (pos = 1, pattern, order_by = "size", decreasing = order_by %in% c("size", "rows", "columns")) {
    napply <- function(names, fn) sapply(names, function(x)
                                         fn(get(x, pos = pos)))
    names <- ls(pos = pos, pattern = pattern)
    if (length(names) == 0) {
      return(names)
    }
    obj.class <- napply(names, function(x) as.character(class(x))[1])
    obj.mode <- napply(names, mode)
    obj.type <- ifelse(is.na(obj.class), obj.mode, obj.class)
    obj.size <- napply(names, utils::object.size)
    obj.dim <- t(napply(names, function(x) as.numeric(dim(x))[1:2]))
    vec <- is.na(obj.dim)[, 1] & (obj.type != "function")
    obj.dim[vec, 1] <- napply(names, length)[vec]
    out <- data.frame("object" = names,
                      "class" = obj.type,
                      "size"  = obj.size,
                      "rows"  = obj.dim[, 1],
                      "columns" = obj.dim[, 2])

    out <- out[order(out[[order_by]], decreasing = decreasing), ]

    rownames(out) <- NULL

    out
}

#' @title Kaplan-Meier Plot
#'
#' @description A ggplot2 version of a Kaplan-Meier Plot
#'
#' @details
#' Functions to build data frames explicitly or implicitly and then create a
#' ggplot2 Kaplan-Meier plot. The misspelled
#' \code{qkmplot_bulid_data_frame} is retained as a deprecated
#' backward-compatible wrapper for \code{qkmplot_build_data_frame}.
#'
#' More details and examples for graphics within qwraps2 are in the
#' vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
#'
#' @param x object
#' @param conf_int logical if TRUE show the CI
#' @param ... Other arguments passed to survival::plot.survfit
#'
#' @return a ggplot.
#'
#' @examples
#' if (requireNamespace("survival", quietly = TRUE)) {
#'
#'   leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x,
#'                                      data = survival::aml)
#'
#'   qkmplot(leukemia.surv, conf_int = TRUE)
#'
#'   qkmplot_build_data_frame(leukemia.surv)
#'
#'   qrmst(leukemia.surv) # NaN for rmst.se in Nonmaintained strata as last observation is an event
#'   qrmst(leukemia.surv, 44)
#'
#'   # pbc examples
#'   pbc_fit <-
#'     survival::survfit(
#'         formula = survival::Surv(time, status > 0) ~ trt
#'       , data = survival::pbc
#'       , subset = !is.na(trt)
#'     )
#'
#'   qkmplot(pbc_fit)
#'   qkmplot(pbc_fit, conf_int = TRUE)
#'
#'   qrmst(pbc_fit)
#'   qrmst(pbc_fit)
#' }
#'
#' @export
#' @rdname qkmplot
qkmplot <- function(x, conf_int = FALSE, ...) {
  UseMethod("qkmplot")
}

#' @export
qkmplot.default <- function(x, conf_int = FALSE, ...) {
  qkmplot_ggplot(x, conf_int = conf_int, ...)
}

#' @export
qkmplot.survfit <- function(x, conf_int = FALSE, ...) {
  qkmplot_ggplot(qkmplot_build_data_frame(x), conf_int = conf_int, ...)
}

#' @export
qkmplot.qkmplot_data <- function(x, conf_int = FALSE, ...) {
  qkmplot_ggplot(x, conf_int = conf_int, ...)
}

qkmplot_ggplot <- function(dat, conf_int = FALSE, ...) {
  layers <- list(eval(substitute(ggplot2::aes(x = X, y = Y), list(X = as.name("time"), Y = as.name("surv")))),
                 if (!is.null(dat$strata)) {eval(substitute(ggplot2::aes(colour = X, fill = Y), list(X = as.name("strata"), Y = as.name("strata")))) } else {NULL},
                 ggplot2::geom_step(),
                 ggplot2::ylim(c(0, 1)),
                 ggplot2::ylab("Survival"),
                 ggplot2::geom_point(data = dat[dat$n.censor > 0, ], shape = 3, alpha = 0.9))

  if (conf_int) {
    layers <- append(layers,
                     ggplot2::geom_ribbon(mapping = eval(substitute(ggplot2::aes(ymin = MN, ymax = MX), list(MN = as.name("lower"), MX = as.name("upper"))))
                                          , alpha = 0.2
                                          , stat = "stepribbon")
      )
  }

  ggplot2::ggplot(dat) + layers
}

#' @export
#' @rdname qkmplot
qkmplot_build_data_frame <- function(x) {
  UseMethod("qkmplot_build_data_frame")
}

#' @export
#' @rdname qkmplot
qkmplot_build_data_frame.survfit <- function(x) {

  plot_data <- data.frame(time = x[['time']],
                          n.risk = x[['n.risk']],
                          n.event = x[['n.event']],
                          n.censor = x[['n.censor']],
                          surv = x[['surv']],
                          upper = x[['upper']],
                          lower = x[['lower']],
                          stringsAsFactors = FALSE)

  if (!is.null(x$strata)) {
    plot_data$strata <- rep(attr(x[['strata']], "names"), times = x[['strata']])
    for( s in attr(x[['strata']], "names")) {
      if (min(plot_data$time[plot_data$strata == s]) > 0) {
        plot_data <- rbind(data.frame(time = 0,
                                      n.risk = max(plot_data$n.risk[plot_data$strata == s]),
                                      n.event = 0,
                                      n.censor = 0,
                                      surv = 1,
                                      upper = 1,
                                      lower = 1,
                                      strata = s)
                           , plot_data)
      }
    }
    plot_data <- plot_data[order(plot_data$strata, plot_data$time),]
  } else {
    if (min(plot_data$time > 0)) {
        plot_data <- rbind(data.frame(time = 0,
                                      n.risk = max(plot_data$n.risk),
                                      n.event = 0,
                                      n.censor = 0,
                                      surv = 1,
                                      upper = 1,
                                      lower = 1
                                      )
                           , plot_data)
    }
    plot_data <- plot_data[order(plot_data$time),]
  }

  class(plot_data) <- c("qkmplot_data", class(plot_data))
  plot_data
}

#' @param tau upper bound on time for restricted mean survival time estimate
#' @export
#' @rdname qkmplot
qrmst <- function(x, tau = Inf) {
  UseMethod("qrmst")
}

#' @export
#' @rdname qkmplot
qrmst.survfit <- function(x, tau = Inf) {
  d <- qkmplot_build_data_frame(x)
  # d <- subset(d, d$time > 0)
  qrmst.qkmplot_data(d, tau = tau)
}

#' @export
#' @rdname qkmplot
qrmst.qkmplot_data <- function(x, tau = Inf) {

  if (is.null(x$strata)) {
    x$strata <- "All"
  }

  d <- split(x, x$strata)

  if (is.infinite(tau)) {
    tau <- min(sapply(d, function(x) {max(x$time)}))
  }

  for (i in seq_along(d)) {

    d[[i]] <- subset(d[[i]], d[[i]]$time <= tau)

    if(!isTRUE(all.equal(max(d[[i]][["time"]]), tau))) {
      taildf <- utils::tail(d[[i]], n = 1)
      taildf$time <- tau
      d[[i]] <- rbind(d[[i]], taildf)
    }

    d[[i]]$timeL <- c(0, d[[i]]$time[-length(d[[i]]$time)])
    d[[i]]$rsum  <- d[[i]]$surv * (d[[i]]$time - d[[i]]$timeL)
    d[[i]]$rcsum <- rev(cumsum(rev(d[[i]]$rsum)))
    d[[i]]$rvsum <- (d[[i]]$rcsum^2 * d[[i]]$n.event) / (d[[i]]$n.risk * (d[[i]]$n.risk - d[[i]]$n.event))

    d[[i]] <-
        data.frame(
          strata  = d[[i]]$strata[1]
        , rmst    = sum(d[[i]]$rsum)
        , rmtl    = tau - sum(d[[i]]$rsum)
        , rmst.se = sqrt(sum(d[[i]]$rvsum))
        , tau     = max(d[[i]]$time)
        )
  }

  d <- do.call(rbind, d)
  d
}

library(hexSticker)

s <- sticker(
    subplot = "toolbox.png",
    s_x = 1.0,
    s_y = 0.95,
    s_width = 0.8,
    s_height = 1.0,
    package = '',
    p_x = 1,
    p_y = 1.7,
    p_color = "#111111",
    p_family = "serif",
    p_fontface = "plain",
    p_size = 18,
    h_size = 1.2,
    h_fill = "#512141", #"#904878",
    h_color = "#000000",
    spotlight = FALSE,
    l_x = 0.7,
    l_y = 1.6,
    l_width = 3,
    l_height = 3,
    l_alpha = 0.3,
    url = "",
    u_x = 1,
    u_y = 0.08,
    u_color = "black",
    u_family = "Aller_Rg",
    u_size = 1.5,
    u_angle = 30,
    white_around_sticker = FALSE,
    filename = "qwraps2logo.png",
    asp = 1,
    dpi = 300
  )

plot(s)

library(qwraps2)

# verify the return is TRUE for a set of installed packages
stopifnot(pkg_check(c("qwraps2", "ggplot2")))

# verify a return of FALSE if at least one of the packages is not installed
stopifnot(pkg_check(c("qwraps2", "NOT A PKG")) == FALSE)

# verify FALSE is installed package is < provided version
stopifnot(pkg_check(c("qwraps2"), "999.999.999") == FALSE)

# Expect an error to be thrown when the `stop` argument is used.
test <- tryCatch(pkg_check(c("qwraps2", "NOT A PKG"), stop = TRUE), error = function(e) e)
stopifnot(inherits(test, 'error'))

library(qwraps2)

# verify the exported data sets
stopifnot(
 identical(
   data(package = "qwraps2")$results[, "Item"]
 , c("mtcars2", "pefr", "spambase")
 )
)

# the data sets should be lazyloaded, so the following should just work
mtcars2
pefr
spambase

################################################################################
##                               mtcars2 checks                               ##

# if these dim sizes fail and the dimensions have changed, make sure to update
# the R/datasets.R documentation too
stopifnot(inherits(mtcars2, "data.frame"))
stopifnot(identical(nrow(mtcars2), 32L))
stopifnot(identical(ncol(mtcars2), 19L))

stopifnot(identical(names(mtcars2)[1], "make"))
stopifnot(identical(names(mtcars2)[2], "model"))
stopifnot(identical(names(mtcars2)[3], "mpg"))
stopifnot(identical(names(mtcars2)[4], "disp"))
stopifnot(identical(names(mtcars2)[5], "hp"))
stopifnot(identical(names(mtcars2)[6], "drat"))
stopifnot(identical(names(mtcars2)[7], "wt"))
stopifnot(identical(names(mtcars2)[8], "qsec"))
stopifnot(identical(names(mtcars2)[9], "cyl"))
stopifnot(identical(names(mtcars2)[10], "cyl_character"))
stopifnot(identical(names(mtcars2)[11], "cyl_factor"))
stopifnot(identical(names(mtcars2)[12], "vs"))
stopifnot(identical(names(mtcars2)[13], "engine"))
stopifnot(identical(names(mtcars2)[14], "am"))
stopifnot(identical(names(mtcars2)[15], "transmission"))
stopifnot(identical(names(mtcars2)[16], "gear"))
stopifnot(identical(names(mtcars2)[17], "gear_factor"))
stopifnot(identical(names(mtcars2)[18], "carb"))
stopifnot(identical(names(mtcars2)[19], "test_date"))

# verify values are the same as in the mtcars data set
stopifnot(
  sapply(names(mtcars), function(nm) {identical(mtcars2[[nm]], mtcars[[nm]])})
    )

# verify the levels of the mtcars2$cyl_factor are as expected
stopifnot(identical(class(mtcars2[["cyl_factor"]]), "factor"))
stopifnot(
  identical(
    levels(mtcars2[["cyl_factor"]])
    ,
    c("6 cylinders", "4 cylinders", "8 cylinders")
    )
  )

# character version
stopifnot(identical(class(mtcars2[["cyl_character"]]), "character"))
stopifnot(
  identical(
    sort(unique(mtcars2[["cyl_character"]]))
    ,
    c("4 cylinders", "6 cylinders", "8 cylinders")
    )
  )

# gear_factor
stopifnot(identical(class(mtcars2[["gear_factor"]]), "factor"))
stopifnot(
  identical(
    levels(mtcars2[["gear_factor"]])
    ,
    c("3 forward gears", "4 forward gears", "5 forward gears")
    )
  )

# engine
stopifnot(identical(class(mtcars2[["engine"]]), "factor"))
stopifnot(
  identical(
    levels(mtcars2[["engine"]])
    ,
    c("V-shaped", "straight")
    )
  )

# transmission
stopifnot(identical(class(mtcars2[["transmission"]]), "factor"))
stopifnot(
  identical(
    levels(mtcars2[["transmission"]])
    ,
    c("Automatic", "Manual")
    )
  )

################################################################################
##                              spambase checks                               ##
# verify the names
stopifnot(
  identical(
    names(spambase)
  , c("word_freq_make", "word_freq_address", "word_freq_all", "word_freq_3d",
      "word_freq_our", "word_freq_over", "word_freq_remove",
      "word_freq_internet", "word_freq_order", "word_freq_mail",
      "word_freq_receive", "word_freq_will", "word_freq_people",
      "word_freq_report", "word_freq_addresses", "word_freq_free",
      "word_freq_business", "word_freq_email", "word_freq_you",
      "word_freq_credit", "word_freq_your", "word_freq_font", "word_freq_000",
      "word_freq_money", "word_freq_hp", "word_freq_hpl", "word_freq_george",
      "word_freq_650", "word_freq_lab", "word_freq_labs", "word_freq_telnet",
      "word_freq_857", "word_freq_data", "word_freq_415", "word_freq_85",
      "word_freq_technology", "word_freq_1999", "word_freq_parts",
      "word_freq_pm", "word_freq_direct", "word_freq_cs", "word_freq_meeting",
      "word_freq_original", "word_freq_project", "word_freq_re",
      "word_freq_edu", "word_freq_table", "word_freq_conference",
      "char_freq_semicolon", "char_freq_parenthesis",
      "char_freq_square_bracket", "char_freq_exclamation_point",
      "char_freq_dollar_sign", "char_freq_pound", "capital_run_length_average",
      "capital_run_length_longest", "capital_run_length_total", "spam")
  )
)

################################################################################
##                                End of File                                 ##
################################################################################

#' mtcars2
#'
#' An extended version of \code{\link[datasets]{mtcars}} data set.
#'
#' @format a data.frame with 32 rows and 19 columns
#'
#' \tabular{rlll}{
#' [,  1] \tab make          \tab Manufacturer name                        \tab parted out from \code{rownames(mtcars)} \cr
#' [,  2] \tab model         \tab                                          \tab parted out from \code{rownames(mtcars)} \cr
#' [,  3] \tab mpg           \tab miles per (US) gallon                    \tab identical to mtcars$mpg \cr
#' [,  4] \tab disp          \tab Displacement (cu.in.)                    \tab identical to mtcars$disp \cr
#' [,  5] \tab hp            \tab Gross horsepower                         \tab identical to mtcars$hp \cr
#' [,  6] \tab drat          \tab Rear axle ratio                          \tab identical to mtcars$drat \cr
#' [,  7] \tab wt            \tab weight (1000 lbs)                        \tab identical to mtcars$wt \cr
#' [,  8] \tab qsec          \tab 1/4 mile time                            \tab identical to mtcars$qsec \cr
#' [,  9] \tab cyl           \tab number of cylinders                      \tab identical to mtcars$cyl \cr
#' [, 10] \tab cyl_character \tab                                          \tab \cr
#' [, 11] \tab cyl_factor    \tab                                          \tab \cr
#' [, 12] \tab vs            \tab Engine (0 = V-shaped, 1 = straight)      \tab identical to mtcars$vs \cr
#' [, 13] \tab engine        \tab                                          \tab \cr
#' [, 14] \tab am            \tab Transmission (0 = automatic, 1 = manual) \tab identical to mtcars$am \cr
#' [, 15] \tab transmission  \tab                                          \tab \cr
#' [, 16] \tab gear          \tab Number of forward gears                  \tab identical to mtcars$gear \cr
#' [, 17] \tab gear_factor   \tab                                          \tab \cr
#' [, 18] \tab carb          \tab Number of carburetors                    \tab identical to mtcars$carb \cr
#' [, 19] \tab test_date     \tab fictitious testing date                  \tab \cr
#' }
#'
#' @seealso \code{vignette("qwraps2-data-sets", package = "qwraps2")} for
#' details on the construction of the data set.
"mtcars2"

#' pefr
#'
#' Peak expiratory flow rate data
#'
#' Peak expiratory flow rate (pefr) data is used for examples within the qwraps2
#' package.  The data has been transcribed from Bland (1986).
#'
#' \dQuote{The sample comprised colleagues and family of J.M.B. chosen to give a
#' wide range of PEFR but in no way representative of any defined population.
#' Two measurements were made with a Wright peak flow meter and two with a mini
#' Wright meter, in random order. All measurements were taken by J.M.B., using
#' the same two instruments. (These data were collected to demonstrate the
#' statistical method and provide no evidence on the comparability of these two
#' instruments.) We did not repeat suspect readings and took a single reading as
#' our measurement of PEFR. Only the first measurement by each method is used to
#' illustrate the comparison of methods, the second measurements being used in
#' the study of repeatability.}
#'
#' @format a data frame with four columns
#' \tabular{rll}{
#' [, 1] \tab subject     \tab id number \cr
#' [, 2] \tab measurement \tab first or second \cr
#' [, 3] \tab meter       \tab \dQuote{Wright peak flow meter} or \dQuote{Mini Write peak flow meter} \cr
#' [, 4] \tab pefr        \tab peak expiratory flow rate (liters / min) \cr
#' }
#'
#' @seealso \code{vignette('qwraps2-data-sets', package = 'qwraps2')} for
#' details on the construction of the data set.
#'
#' @references
#'
#' Bland, J. Martin, and Douglas G Altman. "Statistical methods for assessing
#' agreement between two methods of clinical measurement." The lancet 327, no. 8476
#' (1986): 307-310.
#'
"pefr"

#' Spambase
#'
#' Classifying Email as Spam or Non-Spam
#'
#' @format a data.frame with 4601 rows, 58 columns; 57 features and 0/1
#' indicator for spam
#'
#' Used under CC BY 4.0 license.
#'
#' @seealso \code{vignette("qwraps2-data-sets", package = "qwraps2")} for
#' details on the construction of the data set.
#'
#' @references
#' Hopkins,Mark, Reeber,Erik, Forman,George, and Suermondt,Jaap. (1999). Spambase. UCI Machine Learning Repository. https://doi.org/10.24432/C53G6X.
#'
"spambase"

library(qwraps2)

set.seed(42)
p <- runif(100)

stopifnot(all.equal(suppressWarnings(logit(p)), log(p/(1-p))))
stopifnot(all.equal(suppressWarnings(invlogit(log(p/(1-p)))), p))

x <- seq(-3, 3, length = 1001)
stopifnot(all.equal(suppressWarnings(invlogit(x)), plogis(x)))

x <- seq(sqrt(.Machine$double.eps),  1 - sqrt(.Machine$double.eps), length = 1001)
stopifnot(all.equal(suppressWarnings(logit(x)), qlogis(x)))

library(qwraps2)

# test that a warning is thrown when there are missmatches
test <-
  tryCatch(
           spin_comments(hair = system.file("examples/spinner1.R", package = "qwraps2"))
           , error   = function(e) e , warning = function(w) w)
stopifnot(inherits(test, "warning"))

# verify return is FALSE
x <- suppressWarnings(spin_comments(hair = system.file("examples/spinner1.R", package = "qwraps2")))
stopifnot(isFALSE(x))
x_notes <- attr(x, "notes")
stopifnot(typeof(x_notes) == "character")
stopifnot(identical(x_notes[1], "  * started on line 5; ended on line 7"))
stopifnot(identical(x_notes[2], "  * no starting delimiter; ended on line 15"))
stopifnot(identical(x_notes[3], "  * started on line 22; ended on line 24"))
stopifnot(identical(x_notes[4], "  * started on line 20; no end delimiter"))

# verify no warning when there are no missmatches
test <-
  tryCatch(
           spin_comments(hair = system.file("examples/spinner2.R", package = "qwraps2"))
           , error   = function(e) e , warning = function(w) w)
stopifnot(!inherits(test, "warning"))

# verify true
x <- spin_comments(hair = system.file("examples/spinner2.R", package = "qwraps2"))
stopifnot(x)

x_notes <- attr(x, "notes")
stopifnot(typeof(x_notes) == "character")
stopifnot(identical(x_notes[1], "  * started on line 2; ended on line 6"))
stopifnot(identical(x_notes[2], "  * started on line 9; ended on line 11"))
stopifnot(identical(x_notes[3], "  * started on line 14; ended on line 14"))
stopifnot(identical(x_notes[4], "  * started on line 18; ended on line 18"))

#' Spin Comment Check
#'
#' A tool to help identify the opening and closing of comments in a spin
#' document.  This function is designed to help the user resolve the error
#' "comments must be put in pairs of start and end delimiters."
#'
#' @param hair Path to the R script. The script must be encoded in UTF-8 if it
#'   contains multi-byte characters.
#' @param comment A pair of regular expressions for the start and end delimiters
#'   of comments; the lines between a start and an end delimiter will be
#'   ignored. By default, the delimiters are \verb{/*} at the beginning of a
#'   line, and \verb{*/} at the end, following the convention of C-style
#'   comments.
#' @param text A character vector of code, as an alternative way to provide the
#'   R source. If \code{text} is not \code{NULL}, \code{hair} will be ignored.
#' @param ... additional arguments (not currently used.)
#'
#' @examples
#'
#' spin_comments(hair = system.file("examples/spinner1.R", package = "qwraps2"))
#'
#' @export
spin_comments <- function(hair, comment = c("^[# ]*/[*]", "^.*[*]/ *$"), text = NULL, ...) {
  x <- if (nosrc <- is.null(text)) xfun::read_utf8(hair) else xfun::split_lines(text)
  stopifnot(length(comment) == 2L)
  c1 <- grep(comment[1], x)
  c2 <- grep(comment[2], x)
  check_comments(c1, c2)
}

#' Check Comments
#'
#' A more robust check for open/close matching sets of comments in a spin file.
#'
#' @param c1 index (line numbers) for the start delimiter of comments
#' @param c2 index (line numbers) for the end delimiter of comments
#'
check_comments <- function(c1, c2) {

  cs <- sort(c(openers = c1, closers = c2))
  err <- FALSE
  notes <- character()
  while(length(cs)) {
    i <- 1
    if (grepl("closer", names(cs)[1])) {
      notes <- append(notes, paste0("  * no starting delimiter; ended on line ", cs[1]))
      cs <- cs[-1]
      err <- TRUE
    } else if (all(grepl("opener", names(cs)))) {
      notes <- append(notes, paste0("  * started on line ", cs[1], "; no end delimiter"))
      cs <- cs[-1]
      err <- TRUE
    } else {
      while (i < length(cs)) {
        if (grepl("opener", names(cs)[i]) & grepl("closer", names(cs)[i + 1])) {
          notes <- append(notes, paste0("  * started on line ", cs[i], "; ended on line ", cs[i + 1]))
          cs <- cs[-c(i, i + 1)]
          break
        } else {
          i <- i + 1
        }
      }
    }
  }

  if (err) {
     warning(
             paste0('comments must be put in pairs of start and end delimiters.\n',
                   paste(notes, collapse = '\n')
              , collapse = "\n")
             , call. = FALSE)
  }

  rtn <- !err
  attr(rtn, "notes") <- notes
  rtn
}

library(qwraps2)

# error is non-survfit object is passed to qkmplot_build_data_frame
fit <- lm(mpg ~ wt, data = mtcars)
test <- tryCatch(qkmplot_build_data_frame(fit), error = function(e) e)
stopifnot(inherits(test, "error"))
test <- suppressWarnings(tryCatch(qkmplot_bulid_data_frame(fit), error = function(e) e))
stopifnot(inherits(test, "error"))

# intercept-only survfit objects should work with qrmst
if (requireNamespace("survival", quietly = TRUE)) {
  fit <- survival::survfit(survival::Surv(time, status) ~ 1, data = survival::aml)
  df_old <- suppressWarnings(qkmplot_bulid_data_frame(fit))
  df_new <- qkmplot_build_data_frame(fit)
  stopifnot(identical(df_new, df_old))

  out <- qrmst(fit)
  stopifnot(inherits(out, "data.frame"))
  stopifnot(identical(nrow(out), 1L))
  stopifnot(identical(out$strata, "All"))
  stopifnot(is.finite(out$rmst))
}

#' @title Extract Summary stats from regression objects
#'
#' @description A collection of functions for extracting summary statistics and
#' reporting regression results from \code{lm}, \code{glm} and other regression
#' objects.
#'
#' @seealso
#' \code{\link{lm}} 
#'
#' @param x a \code{lm} object
#'
#' @return a character vector of the formatted numbers
#'
#' @examples
#' fit <- lm(mpg ~ wt + hp + drat, data = mtcars)
#' summary(fit)
#' extract_fstat(fit)
#' extract_fpvalue(fit)
#'
#' @export   
#' @rdname extract
extract_fstat <- function(x) {
  UseMethod("extract_fstat")
}

#' @export
extract_fstat.lm <- function(x) {
  fstat <- summary(x)$fstatistic
  paste0("$F_{", 
         frmt(as.integer(fstat[2L])), ", ", 
         frmt(as.integer(fstat[3L])), "} = ",
         frmt(fstat[1L]), "$")
}

#' @export
#' @return formatted p-value from the F-test
#' @rdname extract
extract_fpvalue <- function(x) { 
  UseMethod("extract_fpvalue")
}

#' @export
#' @rdname extract
extract_fpvalue.lm <- function(x) { 
  fstat <- summary(x)$fstatistic
  frmtp(stats::pf(fstat[1L], fstat[2L], fstat[3L], lower.tail = FALSE))
}

library(qwraps2)

# expect %s% to work like a inline paste0
astr <- "test string 1"
bstr <- "abcd e"
cstr <- "  zyxw."
dstrs <- c("ONE", "TWO", "THREE")

stopifnot(astr %s% bstr %s% cstr == paste0(astr, bstr, cstr))
stopifnot(astr %s% dstrs %s% bstr == paste0(astr, dstrs, bstr))

# expect error if using non-character
stopifnot(class(try(astr %s% "1", silent = TRUE)) == "character")
stopifnot(class(try(astr %s%  1,  silent = TRUE)) == "try-error")

#' Set Differences
#'
#' Function for testing for unique values between two vectors, specifically,
#' which values are in vector1, and not in vector2, which values are not in
#' vector1 and in vector2, which values are in both vector1 and vector2.
#'
#' @param x,y vectors (of the same mode)
#'
#' @return a qwraps2_set_diff object, a list of set comparisons
#'
#' \itemize{
#'   \item \code{all_values} = \code{\link[base]{union}(x, y)}
#'   \item \code{x_only}     = \code{\link[base]{setdiff}(x, y)}
#'   \item \code{y_only}     = \code{\link[base]{setdiff}(y, x)}
#'   \item \code{both}       = \code{\link[base]{intersect}(x, y)}
#'   \item \code{equal}      = \code{\link[base]{setequal}(x, y)}
#' }
#'
#' @examples
#'
#' # example with two sets which as a union are the upper and lower case vowels.
#' set_a <- c("A", "a", "E",      "I", "i", "O", "o", "U", "u", "E", "I")
#' set_b <- c("A", "a", "E", "e",      "i",      "o", "U", "u", "u", "a", "e")
#'
#' set_diff(set_a, set_b)
#' str(set_diff(set_a, set_b))
#'
#' set_diff(set_b, set_a)
#'
#' # example
#' set_a <- 1:90
#' set_b <- set_a[-c(23, 48)]
#' set_diff(set_a, set_b)
#' set_diff(set_b, set_a)
#'
#' # example
#' set_a <- c("A", "A", "B")
#' set_b <- c("B", "A")
#' set_diff(set_a, set_b)
#'
#' # When there are 10 or more unique items in either set, the default print will
#' # omit printing them by default.  Use the verbose argument to change this
#'
#' x <-
#'   set_diff(
#'     x = c(LETTERS, letters[1:10]),
#'     y = letters
#'   )
#' print(x)
#' print(x, verbose = TRUE)
#'
#' @export
set_diff <- function(x, y) {
  out <-
    list(all_values = base::union(x, y),
         x_only     = base::setdiff(x, y),
         y_only     = base::setdiff(y, x),
         both       = base::intersect(x, y),
         equal      = base::setequal(x, y))
  attr(out, "xname") <- deparse(substitute(x))
  attr(out, "yname") <- deparse(substitute(y))
  class(out) <- c("qwraps2_set_diff")
  out
}

#' @param verbose a logical value indicating whether to print the unique
#' elements in each set.
#' @param ... ignored
#'
#' @export
#' @rdname set_diff
print.qwraps2_set_diff <- function(x, verbose = (length(x$x_only) < 10 && length(x$y_only) < 10), ...) {
  stopifnot(is.logical(verbose), length(verbose) == 1L, !is.na(verbose))

  cat("Total number of unique values: ", length(x$all_values), "\n",
      "Number of elements in both ", attr(x, "xname"), " and ", attr(x, "yname"), ": ", length(x$both), "\n",
      "Number of elements only in ", attr(x, "xname"), ": ", length(x$x_only),
      if (verbose && length(x$x_only)) paste0("\n  unique elements: ", paste(x$x_only, collapse = ", "), "\n") else "\n",
      "Number of elements only in ", attr(x, "yname"), ": ", length(x$y_only),
      if (verbose && length(x$y_only)) paste0("\n  unique elements: ", paste(x$y_only, collapse = ", "), "\n") else "\n",
      sep = "")
  invisible(x)
}

## ----label = "setup", include = FALSE-----------------------------------------
knitr::opts_chunk$set(collapse = TRUE)
library(qwraps2)
packageVersion("qwraps2")
has_survival <- requireNamespace("survival", quietly = TRUE)
has_glmnet <- requireNamespace("glmnet", quietly = TRUE)

## -----------------------------------------------------------------------------
set.seed(42)
n <- 250
x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
x1[1] <- runif(1)
x2[1] <- runif(1)
x3[1] <- runif(1)
x4[1] <- runif(1)

# white noise
Z.1 <- rnorm(n, 0, 1)
Z.2 <- rnorm(n, 0, 2)
Z.3 <- rnorm(n, 0, 5)

for(i in 2:n)
{
  x1[i] <- x1[i-1] + Z.1[i] - Z.1[i-1] + x4[i-1] - x2[i-1]
  x2[i] <- x2[i-1] - 2 * Z.2[i] + Z.2[i-1] - x4[i-1]
  x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z.3[i] + Z.3[i-1]
  x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
}
testdf <- data.frame(x1, x2, x3, x4)

# Base acf plot for one variable
acf(testdf$x1)

# qacf plot for one variable
qacf(testdf$x1)
qacf(testdf$x1, show_sig = TRUE)


## ----fig.width = 5, fig.height = 5--------------------------------------------
# more than one variable
acf(testdf)
qacf(testdf)
qacf(testdf, show_sig = TRUE)

## -----------------------------------------------------------------------------
acf_plot_data <- qacf(testdf)$data
head(acf_plot_data)

## -----------------------------------------------------------------------------
pefr_m1 <-
  cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
        "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

## -----------------------------------------------------------------------------
cor(pefr_m1)

ggplot2::ggplot(data = as.data.frame(pefr_m1)) +
  ggplot2::aes(x = Large, y = Mini) +
  ggplot2::geom_point() +
  ggplot2::xlab("Large Meter") +
  ggplot2::ylab("Mini Meter") +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(0, 800) +
  ggplot2::geom_abline(slope = 1)

## -----------------------------------------------------------------------------
# default plot
qblandaltman(pefr_m1)

# modified plot
ggplot2::last_plot() +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(-100, 100) +
  ggplot2::xlab("Average of two meters") +
  ggplot2::ylab("Difference in the measurements")

## -----------------------------------------------------------------------------
pefr_mini <-
  cbind(m1 = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"],
        m2 = pefr[pefr$measurement == 2 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

qblandaltman(pefr_mini)

## ----eval = has_survival, fig.width = 5---------------------------------------
# create a survfit object
leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x, data = survival::aml)

# base R km plot
plot(leukemia.surv, conf.int = TRUE, lty = 2:3, col = 1:2)

# qkmplot
qkmplot(leukemia.surv, conf_int = TRUE)

## ----eval = has_survival, fig.width = 5---------------------------------------
leukemia_km_data <- qkmplot_build_data_frame(leukemia.surv)
head(leukemia_km_data, 3)
qkmplot(leukemia_km_data)

## ----eval = has_survival, fig.width = 5---------------------------------------
intonly_fit <- survival::survfit(survival::Surv(time, status) ~ 1, data = survival::aml)
plot(intonly_fit, conf.int = TRUE)
qkmplot(intonly_fit, conf_int = TRUE)

## -----------------------------------------------------------------------------
set.seed(42)
tidx <- runif(nrow(spambase)) <= 0.80
xidx <- which(names(spambase) != "spam")
yidx <- which(names(spambase) == "spam")
training_set   <- spambase[tidx, ]
validating_set <- spambase[!tidx, ]

## -----------------------------------------------------------------------------
logistic_model <-
  glm(
    spam ~ .
  , data = training_set
  , family = binomial()
  )

## ----eval = has_glmnet--------------------------------------------------------
ridge_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 0
  )

lasso_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 1
  )

## -----------------------------------------------------------------------------
validating_set$logistic_model_prediction <-
  predict(
    logistic_model
  , newdata = validating_set
  , type = "response"
  )

## ----eval = has_glmnet--------------------------------------------------------
validating_set$ridge_model_prediction <-
  as.numeric(
    predict(
      ridge_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

validating_set$lasso_model_prediction <-
  as.numeric(
    predict(
      lasso_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

## -----------------------------------------------------------------------------
cm1 <- confusion_matrix(spam ~ logistic_model_prediction, data = validating_set)

## ----eval = has_glmnet--------------------------------------------------------
cm2 <- confusion_matrix(spam ~ ridge_model_prediction, data = validating_set)
cm3 <- confusion_matrix(spam ~ lasso_model_prediction, data = validating_set)

## -----------------------------------------------------------------------------
qroc(cm1) + ggplot2::ggtitle("Logistic Model")

## ----eval = has_glmnet--------------------------------------------------------
qroc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qroc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

## ----eval = has_glmnet--------------------------------------------------------
roc_plot_data <-
  rbind(
      cbind(Model = paste("Logistic; AUROC =",  frmt(cm1$auroc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUROC =",     frmt(cm2$auroc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUROC =",     frmt(cm3$auroc, 3)), cm3$cm_stats)
    )

qroc(roc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::theme(legend.position = "bottom")

## -----------------------------------------------------------------------------
qprc(cm1) + ggplot2::ggtitle("Logistic Model")

## ----eval = has_glmnet--------------------------------------------------------
qprc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qprc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

prc_plot_data <-
  rbind(
      cbind(Model = paste("Logistic; AUPRC =", frmt(cm1$auprc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUPRC =",     frmt(cm2$auprc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUPRC =",     frmt(cm3$auprc, 3)), cm3$cm_stats)
    )

qprc(prc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::geom_hline(yintercept = cm1$prevalence) +
  ggplot2::theme(legend.position = "bottom")

## ----label = "sessioninfo"----------------------------------------------------
sessionInfo()

#' Geometric Mean, Variance, and Standard Deviation
#'
#' Return the geometric mean, variance, and standard deviation,
#'
#' @param x a numeric vector
#' @param na_rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @return a numeric value
#'
#' @seealso \code{\link{gmean_sd}} for easy formatting of the geometric mean and
#' standard deviation.
#' \code{vignette("qwraps2-formatted-summary-statistics", package = "qwraps2")}.
#'
#' @examples
#'
#' gmean(mtcars$mpg)
#' identical(gmean(mtcars$mpg), exp(mean(log(mtcars$mpg))))
#'
#' gvar(mtcars$mpg)
#' identical(gvar(mtcars$mpg),
#'           exp(var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars)))
#'
#' gsd(mtcars$mpg)
#' identical(gsd(mtcars$mpg),
#'           exp(sqrt( var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars))))
#'
#' #############################################################################
#' set.seed(42)
#' x <- runif(14, min = 4, max = 70)
#'
#' # geometric mean - four equivalent ways to get the same result
#' prod(x) ^ (1 / length(x))
#' exp(mean(log(x)))
#' 1.2 ^ mean(log(x, base = 1.2))
#' gmean(x)
#'
#' # geometric variance
#' gvar(x)
#'
#' # geometric sd
#' exp(sd(log(x)))                                     ## Uses n - 1
#' exp(sqrt((length(x) - 1) / length(x)) * sd(log(x))) ## Correct calculation
#' gsd(x)
#'
#' # Missing data will result in and NA being returned
#' x[c(2, 4, 7)] <- NA
#' gmean(x)
#' gmean(x, na_rm = TRUE)
#' gvar(x, na_rm = TRUE)
#' gsd(x, na_rm = TRUE)
#'
#' @name gmean
NULL

#' @rdname gmean
#' @export
gmean <- function(x, na_rm = FALSE) {
  if (na_rm) {
    x <- stats::na.omit(x)
  }
  exp(mean(log(x)))
}

#' @rdname gmean
#' @export
gvar <- function(x, na_rm = FALSE) {
  if (na_rm) {
    x <- stats::na.omit(x)
  }
  exp((length(x) - 1) / length(x) * stats::var(log(x)))
}

#' @rdname gmean
#' @export
gsd <- function(x, na_rm = FALSE) {
  if (na_rm) {
    x <- stats::na.omit(x)
  }
  exp(sqrt((length(x) - 1) / length(x) * stats::var(log(x))))
}

#' Deprecated logit and invlogit
#'
#' Deprecated logit and invlogit functions.  When originally written they were
#' faster than the base R \code{\link{qlogis}} and \code{\link{plogis}}.
#' However, that is no longer the case.
#' 
#' @param x a numeric vector
#'
#' @name logit
NULL

#' @export
#' @rdname logit
logit <- function(x) {
  .Deprecated(new = "stats::qlogis", msg = "qwraps2::logit and qwraps2::invlogit were nice to haves and faster than stats::qlogis and stats::plogis, but that is no longer true.")
  stats::qlogis(x)
}

#' @export
#' @rdname logit
invlogit <- function(x) {
  .Deprecated(new = "stats::plogis", msg = "qwraps2::logit and qwraps2::invlogit were nice to haves and faster than stats::qlogis and stats::plogis, but that is no longer true.")
  stats::plogis(x)
}

#' Deprecated qkmplot data-frame builder
#'
#' Deprecated misspelled wrapper for \code{\link{qkmplot_build_data_frame}}.
#'
#' @param x object
#'
#' @name qkmplot_bulid_data_frame
NULL

#' @export
#' @rdname qkmplot_bulid_data_frame
qkmplot_bulid_data_frame <- function(x) {
  .Deprecated(new = "qkmplot_build_data_frame",
              msg = "qkmplot_bulid_data_frame() was misspelled; use qkmplot_build_data_frame() instead.")
  qkmplot_build_data_frame(x)
}

#' Deprecated qroc and qprc building of data frames:
#'
#' Deprecated methods for building the data sets needed for plotting
#' Receiver-Operating Curves and Precision-Recall Curves. Use
#' \code{\link{confusion_matrix}} instead.
#'
#' @param fit a \code{glm} fit with \code{family = binomial()}, or predicted
#' values
#' @param truth ignored if \code{fit} is a \code{glm} object.  A vector of
#' observations, 0/1 or FALSE/TRUE values, of equal length to \code{fit}
#' @param n_threshold number of thresholds to use to estimate auroc or auprc
#' @param ... passed to \code{\link[stats]{predict}}
#'
#' @name qroc_build_data_frame
NULL

#' @export
#' @rdname qroc_build_data_frame
qroc_build_data_frame <- function(fit, truth = NULL, n_threshold = 200, ...) {
  .Deprecated(new = "confusion_matrix", msg = "qroc_build_data_frame has been replaced by confusion_matrix(), this will become an error message in a later release of qwraps2")
  UseMethod("qroc_build_data_frame")
}

#' @export
#' @rdname qroc_build_data_frame
qroc_build_data_frame.default <- function(fit, truth = NULL, n_threshold = 200, ...) {
  .Deprecated(new = "confusion_matrix", msg = "qroc_build_data_frame has been replaced by confusion_matrix(), this will become an error message in a later release of qwraps2")
  stopifnot(!is.null(truth))
  stopifnot(length(fit) == length(truth))

  true_pos <- function(threshold){ sum((fit >= threshold) &  (truth)) }
  true_neg <- function(threshold){ sum((fit <  threshold) & !(truth)) }

  false_pos <- function(threshold){ sum((fit >= threshold) & !(truth)) }
  false_neg <- function(threshold){ sum((fit <  threshold) &  (truth)) }

  x <- matrix(seq(1, 0, length = n_threshold))

  true_positives  <- apply(x, 1, true_pos)
  true_negatives  <- apply(x, 1, true_neg)
  false_positives <- apply(x, 1, false_pos)
  false_negatives <- apply(x, 1, false_neg)

  sensitivity <- true_positives / (true_positives + false_negatives)
  specificity <- true_negatives / (true_negatives + false_positives)

  roc_data <- data.frame(false_positives = 1 - specificity,
                         true_positives  = sensitivity)

  # trapezoid rule approximation for the area under the curve
  auc <- sum((roc_data[2:n_threshold, 1] - roc_data[1:(n_threshold-1), 1]) * 1/2 *
             (roc_data[2:n_threshold, 2] + roc_data[1:(n_threshold-1), 2]))

  attr(roc_data, "auc") <- auc
  class(roc_data)       <- c("qwraps2_generated", class(roc_data))

  return(roc_data)
}

#' @export
#' @rdname qroc_build_data_frame
qroc_build_data_frame.glm <- function(fit, truth = NULL, n_threshold = 200, ...) {
  .Deprecated(new = "auc", msg = "qroc_build_data_frame has been replaced by auc(), this will become an error message in a later release of qwraps2")

  stopifnot(fit$family$family %in% c("binomial", "quasibinomial"))

  # find the predicted values
  pred_vals <- stats::predict(fit, ..., type = "response")

  qroc_build_data_frame(pred_vals, fit$y)
}

#' @export
#' @rdname qroc_build_data_frame
qprc_build_data_frame <- function(fit, n_threshold = 200, ...) {
  .Deprecated(new = "auc", msg = "qprc_build_data_frame has been replaced by auc(), this will become an error message in a later release of qwraps2")

  # find the predicted values
  pred_vals <- stats::predict(fit, ..., type = "response")

  true_pos <- function(threshold){ sum((pred_vals >= threshold) &  (fit$y)) }
  true_neg <- function(threshold){ sum((pred_vals <  threshold) & !(fit$y)) }

  false_pos <- function(threshold){ sum((pred_vals >= threshold) & !(fit$y)) }
  false_neg <- function(threshold){ sum((pred_vals <  threshold) &  (fit$y)) }

  x <- matrix(seq(1, 0, length = n_threshold))

  true_positives  <- apply(x, 1, true_pos)
  true_negatives  <- apply(x, 1, true_neg)
  false_positives <- apply(x, 1, false_pos)
  false_negatives <- apply(x, 1, false_neg)

  Recall    <- true_positives / (true_positives + false_negatives) # aka sensitivity
  Precision <- true_positives / (true_positives + false_positives) # aka positive predictive power

  Precision[is.na(Precision)] <- stats::na.omit(Precision)[1]

  prc_data <- data.frame(Recall, Precision)

  # trapezoid rule approximation for the area under the curve
  auc <- sum((prc_data[2:n_threshold, 1] - prc_data[1:(n_threshold-1), 1]) * 1/2 *
             (prc_data[2:n_threshold, 2] + prc_data[1:(n_threshold-1), 2]))

  attr(prc_data, "baseline") <- mean(fit$y)
  attr(prc_data, "auc") <- auc
  class(prc_data)       <- c("qwraps2_generated", class(prc_data))

  return(prc_data)
}

#' Quick Wraps for Reproducible Reports
#'
#' \pkg{qwraps2} is a collection of helper functions for authoring reproducible
#' analysis reports. The package focuses on formatted summary statistics,
#' summary tables, regression-result formatting, and \pkg{ggplot2}-based plots.
#'
#' Several wrappers for \pkg{ggplot2}-style graphics are provided, including
#' receiver operating characteristic curves, precision-recall curves,
#' Bland-Altman plots, autocorrelation plots, and Kaplan-Meier plots. These
#' functions include \code{\link{qroc}}, \code{\link{qprc}},
#' \code{\link{qacf}}, \code{\link{qblandaltman}}, and \code{\link{qkmplot}}.
#'
#' Other functions quickly generate meaningful character strings for reporting
#' results in .Rnw, .Rmd, or similar report files.
#'
#' @section Vignettes:
#' \itemize{
#'   \item \code{vignette("qwraps2-formatted-summary-statistics", package = "qwraps2")}
#'   introduces individual summary-statistic and formatting helpers.
#'   \item \code{vignette("qwraps2-summary-table", package = "qwraps2")}
#'   demonstrates full summary-table workflows.
#'   \item \code{vignette("qwraps2-graphics", package = "qwraps2")}
#'   demonstrates the plotting helpers.
#'   \item \code{vignette("qwraps2-data-sets", package = "qwraps2")}
#'   documents the bundled example data sets.
#' }
#'
#' @section Options:
#' There are several options which can be set via \code{options} and will be
#' used via \code{getOption}.  The following lists, in alphabetical order the
#' different options which are available and what they control.
#'
#' \itemize{
#'   \item \code{getOption("qwraps2_alpha", 0.05)} significance level, used for
#'   generating \code{(1 - getOption("qwraps2_alpha", 0.05)) * 100}\% confidence
#'   intervals, and determining significance for p-value <
#'   \code{getOption("qwraps2_alpha", 0.05)}.
#'
#'   \item \code{getOption("qwraps2_frmt_digits", 2)}  Number of digits to the
#'   right of the decimal point for any value other than p-values.
#'
#'   \item \code{getOption("qwraps2_frmtp_case", "upper")} set to either
#'   'upper' or 'lower' for the case of the 'P' for reporting p-values.
#'
#'   \item \code{getOption("qwraps2_frmtp_digits", 4)}  Number of digits to the
#'     right of the decimal point to report p-values too.  If
#'     \code{log10(p-value) < getOption("qwraps2_frmtp_digits", 4)} then the
#'     output will be "P < 0.01", to however many digits are correct.  Other
#'     options control other parts of the output p-value format.
#'
#'   \item \code{getOption("qwraps2_frmtp_leading0", TRUE)} to display or not
#'   to display the leading zero in p-values, i.e., if TRUE p-values are
#'   reported as 0.02 versus when FALSE p-values are reported as .02.
#'
#'   \item \code{getOption("qwraps2_journal", "default")} if a journal has
#'   specific formatting for p-values or other statistics, this option will
#'   control the output.  Many other options are ignored if this is any other
#'   than default.  Check the GitHub wiki, or this file,  for current lists of
#'   implemented journal style methods.
#'
#'   \item \code{getOption("qwraps2_markup", "latex")} value set to 'latex' or
#'   to 'markdown'.  Output is formatted to meet requirements of either markup
#'   language.
#' }
#'
#' @section Journals with predefined formatting:
#'
#' \itemize{
#'   \item Obstetrics & Gynecology
#'   \itemize{
#'      \item \url{https://www.editorialmanager.com/ong/default.aspx}
#'      \item \code{options(qwraps2_journal = "obstetrics_gynecology")}
#'      \item P-value formatting as of April 2015:
#'
#'       Express P values to no more than three decimal places.
#'
#'       Based on observations of published work, leading 0 will be omitted.
#'    }
#'
#'   \item Pediatric Dentistry:
#'   \itemize{
#'     \item \url{https://www.aapd.org/publications/}
#'     \item \code{options(qwraps2_journal = "pediatric_dentistry")}
#'     \item P-value formatting  as of March 2015.
#'
#'     If P > .01, the actual value for
#'     P should be expressed to 2 digits.  Non-significant values should not be
#'     expressed as "NS" whether or not P is significant, unless rounding a
#'     significant P-value expressed to 3 digits would make it non-significant
#'     (i.e., P=.049, not P=.05).  If P<.01, it should be expressed to 3 digits
#'     (e.g., P=.003, not P<.05).  Actual P-values should be expressed unless
#'     P<.001, in which case they should be so designated.
#'   }
#'
#' }
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @useDynLib qwraps2
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
NULL

#' File and Working Directory Check
#'
#' This check is three-fold: 1) verify the current working directory is as
#' expected, 2) verify the user can access the file, and 3) verify the file
#' contents are as expected (via md5sum).
#'
#' The test for the file access is done to verify the file can be read by the
#' current user.
#'
#' The return of the function is \code{TRUE} if all the files in \code{paths}
#' are accessible, are case matched (optional), and all of requested md5sum
#' checks pass.  Windows and macOS are generally case-insensitive systems, but
#' many Linux systems are case-sensitive.  As such
#' \code{\link[base]{file.exists}} and \code{\link[base]{file.access}} may
#' return different values depending the OS that is active.  \code{file_check}
#' looks for a case match as part of its checks to hopefully prevent issues
#' across operating systems.
#'
#' By default, if the return is \code{TRUE} then only \code{TRUE} will
#' be printed to the console.  If the return is \code{FALSE} then the
#' \code{attr(, "checks")} is printed by default as well.
#'
#' Good practice would be to use relative paths, a warning will be given if any
#' of the \code{paths} are determined to be absolute paths.  That said, there
#' are cases when an absolute path is needed, e.g., a common data file on a
#' server with multiple users accessing the file(s).  Set \code{absolute_paths =
#' c("silent")} to silence the warnings.
#'
#' @param paths a character path to the target file
#' @param md5sums a character string for the expected md5sum of the target file.
#' If \code{NULL} then only a \code{file.exists} check will be done.
#' @param absolute_paths a character string to set the behavior of warning
#' (default), stopping, or silent if/when absolute file paths are used.
#' @param stop if \code{TRUE} then an error is thrown if any of the checks fail,
#' else returns a logical. If \code{FALSE} (default) a logical is returned.
#'
#' @return The function will return a single TRUE/FALSE value with attributes
#' \code{attr(, "checks")}.
#'
#'
#' @examples
#' # create example files
#'
#' relative_example_file1 <-
#'   basename(
#'     tempfile(
#'       pattern = "QWRAPS2_EXAMPLE_1"
#'       , fileext = ".txt"
#'       , tmpdir = getwd()
#'     )
#'   )
#'
#' relative_example_file2 <-
#'   basename(
#'     tempfile(
#'       pattern = "QWRAPS2_EXAMPLE_2"
#'       , fileext = ".txt"
#'       , tmpdir = getwd()
#'     )
#'   )
#'
#' absolute_example_file <- tempfile()
#'
#' cat("example file.", file = relative_example_file1)
#' cat("Another example file.", file = relative_example_file2)
#' cat("Another example file.", file = absolute_example_file)
#'
#' # Check that you have access to the files in the working directory.
#' test1 <- file_check(c(relative_example_file1, relative_example_file2))
#' test1
#'
#' # By default, when the checks return TRUE the details of the checks are not
#' # printed.  You can view the details of the checks as follows:
#' attr(test1, "checks")
#'
#' # access to absolute_example_file will generate a warning about
#' # absolute_paths by default
#' test2 <- file_check(absolute_example_file)
#' test2 <- file_check(absolute_example_file, absolute_paths = "silent")
#' test2
#'
#' # Case Match
#' test_case_match <-
#'   file_check(
#'     c(relative_example_file1, tolower(relative_example_file1))
#'   )
#' test_case_match
#'
#' # If one or more files is not accessible then return is FALSE and the metadata
#' # is printed by default.
#' test_non_existent_file <-
#'   file_check(
#'     c("UNLIKELYFILENAME", relative_example_file1, relative_example_file2)
#'   )
#' test_non_existent_file
#'
#' # Or have an error thrown:
#' \dontrun{
#' file_check(
#'   c("UNLIKELYFILENAME", relative_example_file1, relative_example_file2)
#' , stop = TRUE
#' )
#' }
#'
#' # Verify the md5sums as well as file access:
#' md5_check1 <- file_check(relative_example_file1, "7a3409e17f9de067740e64448a86e708")
#' md5_check1
#'
#' # If you only need to verify a subset of md5sums then use an NA in the md5sums
#' # argument:
#' md5_check2 <-
#'   file_check(c(relative_example_file1, relative_example_file2),
#'              c("7a3409e17f9de067740e64448a86e708", NA))
#' md5_check2
#'
#' # Verify all the md5sums
#' md5_check3 <-
#'   file_check(c(relative_example_file1, relative_example_file2),
#'              c("7a3409e17f9de067740e64448a86e708", "798e52b92e0ae0e60f3f3db1273235d0"))
#' md5_check3
#'
#' # clean up working directory
#' unlink(relative_example_file1)
#' unlink(relative_example_file2)
#' unlink(absolute_example_file)
#'
#' @export
file_check <- function(paths, md5sums = NULL, absolute_paths = c("warn", "stop", "silent"), stop = FALSE) {
  paths <- as.character(paths)
  if (!length(paths)) {
    stop("paths must have length greater than zero.")
  }
  md5sums <- as.character(md5sums)
  current_md5sums <- tools::md5sum(path.expand(paths))
  md5check <- rep(NA, length(paths))

  if (length(md5sums) > 0L) {
    if (length(md5sums) != length(paths)) {
      stop("md5sums vector needs to have length equal to the length of paths.")
    }
    md5check <- mapply(identical, x = current_md5sums, y = md5sums)
    md5check[is.na(md5sums)] <- NA
  }

  # are files paths absolute or relative?
  action <- match.arg(absolute_paths)
  abspths <- NA
  if (action %in% c("warn", "stop")) {
    abspths <- regexpr("^((~|/)|.:(/|\\\\))", paths) != -1L
    if (any(abspths)) {
      if (action == "warn") {
        warning("Absolute path used. It would be preferable if you used a relative path.")
      } else {
        stop("Absolute path used. It would be preferable if you used a relative path.")
      }
    }
  }

  case_match <-
    sapply(paths, function(x) {length(list.files(dirname(x), pattern = basename(x), ignore.case = FALSE)) == 1L})

  checks <-
    data.frame(
               path          = paths
               , absolute_path = abspths
               , accessible    = file.access(paths, mode = 4) != -1L
               , case_match    = case_match
               , current_md5sum = current_md5sums
               , expected_md5sum = NA_character_
               , md5check = md5check
               , stringsAsFactors = FALSE
    )

  if (length(md5sums) > 0L) {
    checks$expected_md5sum <- md5sums
  }

  status <- all(c(checks$accessible, checks$case_match, stats::na.omit(checks$md5check)))
  attr(status, "checks") <- checks
  attr(status, "class") <- "qwraps2_file_check"

  if (!status && stop) {
    print(status)
    stop("At least one path, case match, or md5sum check failed.", call. = FALSE)
  }

  status
}

#' @export
print.qwraps2_file_check <- function(x, ...) {
  if (x) {
    print(TRUE)
  } else {
    attr(x, 'class') <- NULL
    print.default(x)
  }
}

#' Operators
#'
#' A set of helpful operators to make writing and basic data analysis easier.
#'
#' @param e1 a character string
#' @param e2 a character string
#'
#' @export
#' @rdname qwraps2-utilities
#' @examples
#'
#' # base R
#' paste0("A longer string ", "can be ", "built")
#'
#' # with the %s% operator
#' "A longer string " %s% "can be " %s% "built"
#'
"%s%" <- function(e1, e2) {
  if(!inherits(e1, "character") | !inherits(e2, "character")) {
    stop("%s% will not coerce to strings.  That is, 'a' %s% 1 will fail, 'a' %s% '1' will work")
  }
  paste0(e1, e2)
}

# inspired by overloadding the + operator
# `+` <- function(e1, e2) {
#   if(inherits(e1, "character") & inherits(e2, "character")) {
#     paste0(e1, e2)
#   } else {
#     .Primitive("+")(e1, e2)
#   }
# }

library(qwraps2)
set.seed(42)

################################################################################
# testing attributes
x <- qable(mtcars2[, c("mpg", "disp")], markup = "latex")
stopifnot(
  identical(
    attributes(x)
    ,
    list(dim = c(32L, 3L), dimnames = list(NULL, c("", "mpg", "disp")), class = "qwraps2_qable", qable_args = list(rtitle = "", rgroup = numeric(0), rnames = c("1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32"), cnames = c("", "mpg", "disp"), markup = "latex", kable_args = list()))
  )
)

################################################################################
# simple output tests
output_latex <- capture.output(print(qable(mtcars2, markup = "latex")))
output_markdown <- capture.output(print(qable(mtcars2, markup = "markdown")))

# simple print test
stopifnot(any(grepl("142E\\ \\&\\ 21\\.4", output_latex)))
stopifnot(any(grepl("142E\\ +\\|21\\.4", output_markdown)))

# error for unexpected markup language
test <- tryCatch(qable(mtcars2, markup = "rts"), error = function(e) e)
stopifnot(inherits(test, "error"))

# errors for malformed row groups
test <- tryCatch(qable(matrix(1:4, nrow = 2), rgroup = c(A = 1)), error = function(e) e)
stopifnot(inherits(test, "error"))
stopifnot(grepl("sum\\(rgroup\\)", test$message))

test <- tryCatch(qable(matrix(1:4, nrow = 2), rgroup = c(A = 0, B = 2)), error = function(e) e)
stopifnot(inherits(test, "error"))
stopifnot(grepl("positive integer", test$message))

test <- tryCatch(qable(matrix(1:4, nrow = 2), rgroup = c(1, 1)), error = function(e) e)
stopifnot(inherits(test, "error"))
stopifnot(grepl("named vector", test$message))

# rtitle
out <- capture.output(print(qable(mtcars[1, ], rtitle = "user defined rtitle")))
stopifnot(grepl("^user\\ defined\\ rtitle", out[4]))
out <- capture.output(print(qable(mtcars[1, ], markup = "markdown", rtitle = "user defined rtitle")))
stopifnot(grepl("^\\|user\\ defined\\ rtitle", out[3]))

# rgroups
make <- sub("^(\\w+)\\s?(.*)$", "\\1", rownames(mtcars))
make <- c(table(make))
output <- capture.output(print(qable(mtcars[sort(rownames(mtcars)), ], rgroup = make)))

any(grepl("\\\\bf\\{Volvo\\}\\ \\&\\ ~", output))
any(grepl("~~\\ Volvo\\ 142E\\ \\&\\ 21\\.4", output))

output <- capture.output(print(qable(mtcars[sort(rownames(mtcars)), ], rgroup = make, markup = "markdown")))

stopifnot(any(grepl("\\*\\*\\Volvo\\*\\*", output)))
stopifnot(any(grepl("\\&nbsp;\\&nbsp;\\ Volvo", output)))

################################################################################
# Test cbind
qtab_1 <- qable(mtcars[sort(rownames(mtcars)), c("mpg", "cyl")], rgroup = make, kable_args = list(caption = "1"))
qtab_2 <- qable(mtcars[sort(rownames(mtcars)), c("disp", "hp", "drat")], rgroup = make, kable_args = list(caption = "2"))
qtab_0 <- qable(mtcars[sort(rownames(mtcars)), c("mpg", "cyl", "disp", "hp", "drat")], rgroup = make, kable_args = list(caption = "1"))

# the number of column of each of these tables should be one more than the
# number of columns of the data.frame passed.  the "extra" column is the row
# grouping and names.
stopifnot(identical(ncol(qtab_1), 3L))
stopifnot(identical(ncol(qtab_2), 4L))
stopifnot(identical(ncol(qtab_0), 6L))

# qtab_1 and qtab_2 should join together to be identical to qtab_0
stopifnot(identical(cbind(qtab_1, qtab_2), qtab_0))

# should be able to cbind on a vector of values
out <- cbind(qtab_1, a_number = rnorm(nrow(qtab_1)))
stopifnot(identical(colnames(out), c("", "mpg", "cyl", "a_number")))

# should be able to cbind on a vector of values as a matrix
out <- cbind(qtab_1, a_number = matrix(rnorm(nrow(qtab_1)), ncol = 1))
stopifnot(identical(colnames(out), c("", "mpg", "cyl", "a_number")))

# or even another matrix
out <- cbind(qtab_1, a = matrix(rnorm(3 * nrow(qtab_1)), ncol = 3))
stopifnot(identical(colnames(out), c("", "mpg", "cyl", "a1", "a2", "a3")))

# error if nrows are not the same, or if a vector length is different from the
# nrow of the first element
test <-
  tryCatch(
           cbind(qtab_1, qtab_1_2, a_number = rnorm(nrow(qtab_1) - 1)),
           error = function(e) e)
stopifnot(inherits(test, "error"))

test <-
  tryCatch(
           cbind(qtab_1, qtab_1_2[1:3, ]),
           error = function(e) e)
stopifnot(inherits(test, "error"))

# there should be an error if the row groups are not identical
m <- make; names(m) <- toupper(names(m))
qtab_1_2 <- qable(mtcars[, c("mpg", "cyl")], rgroup = m)
test <- tryCatch(cbind(qtab_1, qtab_1_2), error = function(e) e)
stopifnot(inherits(test, "error"))

qtab_1_3 <- qable(mtcars[, c("mpg", "cyl")], rgroup = sample(make))
test <- tryCatch(cbind(qtab_1, qtab_1_3), error = function(e) e)
stopifnot(inherits(test, "error"))

# the attributes of the object from cbind should  be those of the first object
# passed in
stopifnot(identical(attributes(qtab_2)$qable_args$kable_args, list(caption = "2")))
stopifnot(identical(attributes(qtab_1)$qable_args$kable_args, list(caption = "1")))
stopifnot(identical(attributes(cbind(qtab_1, qtab_2))$qable_args$kable_args, list(caption = "1")))
stopifnot(identical(attributes(cbind(qtab_2, qtab_1))$qable_args$kable_args, list(caption = "2")))

################################################################################
# testing rbind
m1 <- mtcars2[grepl("^M", mtcars2$make), ]
m1 <- m1[order(m1$make, m1$model), ]
m2 <- mtcars2[!grepl("^M", mtcars2$make), ]
m2 <- m2[order(m2$make, m2$model), ]

rg1 <- c(table(m1$make))
rg2 <- c(table(m2$make))
m <- rbind(m1, m2)
rg <- c(table(m$make))
rg <- rg[c(names(rg1), names(rg2))]

stopifnot(isTRUE(
  all.equal(
    rbind(qable(m1, rgroup = rg1), qable(m2, rgroup = rg2))
    ,
    qable(m, rgroup = rg)
    )
  )
)

################################################################################
##                                End of File                                 ##
################################################################################

library(qwraps2)

################################################################################
##                               Tests for frmt                               ##
stopifnot(frmt(pi) == "3.14")
stopifnot(frmt(pi, 1) == "3.1")
stopifnot(frmt(pi, 3) == "3.142")
stopifnot(frmt(pi, 4) == "3.1416")
stopifnot(frmt(pi, 5) == "3.14159")
stopifnot(frmt(4) == "4.00")
stopifnot(frmt(4L) == "4")

# append one character
stopifnot(
  identical(
    frmt(c(1, 2, 3)/19 * 100, digits = 2, append = "%")
    ,
    c("5.26%", "10.53%", "15.79%")
  )
)

# append different characters
stopifnot(
  identical(
    frmt(c(1, 2, 3)/19 * 100, digits = 2, append = c("a", "b", "c"))
    ,
    c("5.26a", "10.53b", "15.79c")
  )
)

# error if append has length > 1 and different from length(x)
stopifnot(inherits(tryCatch(frmt(c(1, 2, 3)/19 * 100, digits = 2, append = c("a", "b")), error = function(e) e) , "error"))

################################################################################
##                              Tests for frmtci                              ##
temp <- c(a = 1.23, b = .32, CC = 1.78)
stopifnot(identical(frmtci(temp), "1.23 (0.32, 1.78)"))

# show level uses getOption("qwraps2_alpha", 0.05)
stopifnot(identical(frmtci(temp, show_level = TRUE), "1.23 (95% CI: 0.32, 1.78)"))

# note that the show_level will be ignored in the following
stopifnot(identical(frmtci(temp, format = "est ***lcl, ucl***", show_level = FALSE), "1.23 ***0.32, 1.78***"))
stopifnot(identical(frmtci(temp, format = "est ***lcl, ucl***", show_level = TRUE), "1.23 ***0.32, 1.78***"))

# show_level as a character
stopifnot(identical(frmtci(temp, show_level = "confidence between: "), "1.23 (confidence between: 0.32, 1.78)"))

# For a matrix: the numbers in this example don't mean anything, but the
# formatting should.
set.seed(42)
temp2 <- matrix(rnorm(12), nrow = 4, dimnames = list(c("A", "B", "C", "D"), c("EST", "LOW", "HIGH")))
stopifnot(identical(frmtci(temp2), c(A = "1.37 (0.40, 2.02)", B = "-0.56 (-0.11, -0.06)", C = "0.36 (1.51, 1.30)", D = "0.63 (-0.09, 2.29)")))

df2 <- as.data.frame(temp2)
stopifnot(identical(frmtci(df2), c(A = "1.37 (0.40, 2.02)", B = "-0.56 (-0.11, -0.06)", C = "0.36 (1.51, 1.30)", D = "0.63 (-0.09, 2.29)")))

################################################################################
##                    Rpkg formating and urls in markdown                     ##

# Markdown formating of packages and links to them
stopifnot(identical(Rpkg(qwraps2), "*qwraps2*"))
stopifnot(identical(Rpkg("qwraps2"), "*qwraps2*"))

stopifnot(identical(CRANpkg(qwraps2), "*[qwraps2](https://cran.r-project.org/package=qwraps2)*"))
stopifnot(identical(CRANpkg("qwraps2"), "*[qwraps2](https://cran.r-project.org/package=qwraps2)*"))

stopifnot(identical(Githubpkg(qwraps2, "dewittpe"), "*[qwraps2](https://github.com/dewittpe/package=qwraps2)*"))
stopifnot(identical(Githubpkg("qwraps2", dewittpe), "*[qwraps2](https://github.com/dewittpe/package=qwraps2)*"))

stopifnot(identical(Gitlabpkg(qwraps2, "dewittpe"), "*[qwraps2](https://gitlab.com/dewittpe/package=qwraps2)*"))
stopifnot(identical(Gitlabpkg("qwraps2", dewittpe), "*[qwraps2](https://gitlab.com/dewittpe/package=qwraps2)*"))

################################################################################
##                                  backtick                                  ##
stopifnot(identical(backtick("a string"), "`\"a string\"`"))
stopifnot(identical(backtick("a string", dequote = TRUE), "`a string`"))
stopifnot(identical(backtick("\"a string\"", dequote = TRUE), "`\\\"a string\\\"`"))
stopifnot(identical(backtick(noquotestring), "`noquotestring`"))

#' Step ribbon statistic
#'
#' Provides stair step values for ribbon plots (Copied this from the
#' https://github.com/hrbrmstr/ggalt version 0.6.0, which is not yet on CRAN.
#' Some minor modifications to the file have been made).
#'
#' @inheritParams ggplot2::geom_ribbon
#' @param geom which geom to use; defaults to \code{ribbon}
#' @param direction \code{hv} for horizontal-vertical steps, \code{vh} for
#'   vertical-horizontal steps
#' @references \url{https://groups.google.com/forum/?fromgroups=#!topic/ggplot2/9cFWHaH1CPs}
#' @examples
#' x <- 1:10
#' df <- data.frame(x=x, y=x+10, ymin=x+7, ymax=x+12)
#'
#' # horizontal-vertical steps (default)
#' gg <- ggplot2::ggplot(df, ggplot2::aes(x, y))
#' gg <- gg + ggplot2::geom_ribbon(ggplot2::aes(ymin=ymin, ymax=ymax),
#'                                 stat="stepribbon", fill="#b2b2b2",
#'                                 direction="hv")
#' gg <- gg + ggplot2::geom_step(color="#2b2b2b")
#' gg
#'
#' # vertical-horizontal steps (default)
#' gg <- ggplot2::ggplot(df, ggplot2::aes(x, y))
#' gg <- gg + ggplot2::geom_ribbon(ggplot2::aes(ymin=ymin, ymax=ymax),
#'                                 stat="stepribbon", fill="#b2b2b2",
#'                                 direction="vh")
#' gg <- gg + ggplot2::geom_step(color="#2b2b2b")
#' gg
#'
#' # The same plot calling stat_stepribbon directly
#' gg <- ggplot2::ggplot(df, ggplot2::aes(x, y))
#' gg <- gg + stat_stepribbon(mapping = ggplot2::aes(ymin=ymin, ymax=ymax),
#'                            fill="#b2b2b2", direction="vh")
#' gg <- gg + ggplot2::geom_step(color="#2b2b2b")
#' gg
#'
#' @export
stat_stepribbon <- function(mapping=NULL, data=NULL, geom="ribbon",
                            position="identity",
                            na.rm=FALSE, show.legend=NA, inherit.aes=TRUE,
                            direction="hv", ...) {
  ggplot2::layer(
    data = data,
    mapping = mapping,
    stat = StatStepribbon,
    geom = geom,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      na.rm = na.rm,
      direction = direction,
      ...
    )
  )
}

#' @title Stat Step Ribbon
#' @description
#' Provides stair step values for ribbon plots (Copied this from the
#' https://github.com/hrbrmstr/ggalt version 0.6.0, which is not yet on CRAN.
#' Some minor modifications to the file have been made).
#' @format NULL
#' @usage NULL
#' @references \url{https://groups.google.com/forum/?fromgroups=#!topic/ggplot2/9cFWHaH1CPs}
#' @export
StatStepribbon <-
  ggplot2::ggproto(
    "StatStepRibbon", ggplot2::Stat,

    required_aes = c("x", "ymin", "ymax"),

    compute_group = function(data, scales, direction="hv",
                             yvars=c("ymin", "ymax"), ...) {
      stairstepn(data=data, direction=direction, yvars=yvars)
    }

  )

stairstepn <- function(data, direction="hv", yvars="y") {

  direction <- match.arg(direction, c("hv", "vh"))

  data <- as.data.frame(data)[order(data$x),]

  n <- nrow(data)

  if (direction == "vh") {
    xs <- rep(1:n, each=2)[-2*n]
    ys <- c(1, rep( 2:n, each=2))
  } else {
    ys <- rep(1:n, each=2)[-2*n]
    xs <- c(1, rep(2:n, each=2))
  }

  data.frame(
    x=data$x[xs],
    data[ys, yvars, drop=FALSE],
    data[xs, setdiff(names(data), c("x", yvars)), drop=FALSE]
  )

}

library(qwraps2)
e <- new.env()
example("file_check", local = e)
ls(envir = e)

test <- tryCatch(file_check(character(0)), error = function(e) e)
stopifnot(inherits(test, "error"))
stopifnot(grepl("paths must have length greater than zero", test$message))

################################################################################
#                                    Test 1                                    #
stopifnot(isTRUE(e$test1))
stopifnot(identical(attr(e$test1, "checks")[, c("absolute_path")], c(FALSE, FALSE)))
stopifnot(identical(attr(e$test1, "checks")[, c("case_match")], c(TRUE, TRUE)))
stopifnot(identical(attr(e$test1, "checks")[, c("current_md5sum")], c("7a3409e17f9de067740e64448a86e708", "798e52b92e0ae0e60f3f3db1273235d0")))
stopifnot(identical(attr(e$test1, "checks")[, c("expected_md5sum")], c(NA_character_, NA_character_)))
stopifnot(identical(attr(e$test1, "checks")[, c("md5check")], c(NA,NA)))

################################################################################
#                                    Test 2                                    #

# stopifnot(
#   identical(
#     e$test2
#   ,
#   TRUE
#   )
# )

################################################################################
#                               test case match                                #
stopifnot(all.equal(unclass(e$test_case_match), FALSE, check.attributes = FALSE))

################################################################################
e$md5check3

################################################################################

# verify error is thrown when stop = TRUE
x <-
  tryCatch(file_check(c("UNLIKELYFILENAME", "QWRAPS2_EXAMPLE_1.txt", "QWRAPS2_EXAMPLE_2.txt"), stop = TRUE)
           , error = function(e) e )
stopifnot(!is.null(x))
stopifnot(inherits(x, "error"))

#' @title Confusion Matrices (Contingency Tables)
#'
#' @description Construction of confusion matrices, accuracy, sensitivity,
#' specificity, confidence intervals (Wilson's method and (optional
#' bootstrapping)).
#'
#' @param truth a integer vector with the values \code{0} and \code{1}, or a logical vector.
#' A value of \code{0} or \code{FALSE} is an indication of condition negative;
#' \code{1} or \code{TRUE} is an indication of condition positive.
#' @param predicted a numeric vector.  See Details.
#' @param formula of the form \code{truth ~ predicted}.
#' @param data a data.frame containing the variables reference in
#' \code{formula}.
#' @param thresholds a numeric vector of thresholds to be used to define the
#' confusion matrix (one threshold) or matrices (two or more thresholds).  If
#' \code{NULL} the unique values of \code{predicted} will be used.
#' @param ... pass through
#' @param confint_method character string denoting if the logit (default),
#' binomial, or Wilson Score method for deriving confidence intervals
#' @param alpha alpha level for 100 * (1 - alpha)\% confidence intervals
#'
#' @details
#' The confusion matrix:
#'
#' \tabular{lccc}{
#'                     \tab      \tab True \tab Condition \cr
#'                     \tab      \tab +    \tab -         \cr
#' Predicted Condition \tab +    \tab TP   \tab FP        \cr
#' Predicted Condition \tab -    \tab FN   \tab TN        \cr
#' }
#' where
#' \itemize{
#'   \item FN: False Negative = truth = 1 & prediction < threshold,
#'   \item FP: False Positive = truth = 0 & prediction >= threshold,
#'   \item TN: True Negative  = truth = 0 & prediction < threshold, and
#'   \item TP: True Positive  = truth = 1 & prediction >= threshold.
#' }
#'
#' The statistics returned in the \code{cm_stats} element are:
#' \itemize{
#'   \item accuracy    = (TP + TN) / (TP + TN + FP + FN)
#'   \item sensitivity, aka true positive rate or recall = TP / (TP + FN)
#'   \item specificity, aka true negative rate = TN / (TN + FP)
#'   \item positive predictive value (PPV), aka precision = TP / (TP + FP)
#'   \item negative predictive value (NPV) = TN / (TN + FN)
#'   \item false negative rate (FNR) = 1 - Sensitivity
#'   \item false positive rate (FPR) = 1 - Specificity
#'   \item false discovery rate (FDR) = 1 - PPV
#'   \item false omission rate (FOR) = 1 - NPV
#'   \item F1 score
#'   \item Matthews Correlation Coefficient (MCC) =
#'     ((TP * TN) - (FP * FN)) / sqrt((TP + FP) (TP+FN) (TN+FP) (TN+FN))
#' }
#'
#' Synonyms for the statistics:
#' \itemize{
#' \item Sensitivity: true positive rate (TPR), recall, hit rate
#' \item Specificity: true negative rate (TNR), selectivity
#' \item PPV: precision
#' \item FNR: miss rate
#' }
#'
#' Sensitivity and PPV could, in some cases, be indeterminate due to division by
#' zero.  To address this we use the GERBIL convention from the DICE group
#' \url{https://github.com/dice-group/gerbil/wiki/Precision,-Recall-and-F1-measure}:
#' If TP, FP, and FN are all 0, then PPV, sensitivity, and F1 will be defined to
#' be 1.  If TP are 0 and FP + FN > 0, then PPV, sensitivity, and F1 are all
#' defined to be 0.
#'
#' @return
#' \code{confusion_matrix} returns a list with elements
#' \itemize{
#'   \item \code{cm_stats} a data.frame with columns:
#'   \item \code{auroc} numeric value for the area under the receiver operating
#'   curve
#'   \item \code{auroc_ci} a numeric vector of length two with the lower and
#'   upper bounds for a 100(1-alpha)\% confidence interval about the auroc
#'   \item \code{auprc} numeric value for the area under the precision recall
#'   curve
#'   \item \code{auprc_ci} a numeric vector of length two with the lower and
#'   upper limits for a 100(1-alpha)\% confidence interval about the auprc
#'   \item \code{confint_method} a character string reporting the method used to
#'   build the \code{auroc_ci} and \code{auprc_ci}
#'   \item \code{alpha} the alpha level of the confidence intervals
#'   \item \code{prevalence} the proportion of the input of positive cases, that
#'   is (TP + FN) / (TP + FN + FP + TN) = P / (P + N)
#' }
#'
#' @examples
#'
#' # Example 1: known truth and prediction status
#' df <-
#'   data.frame(
#'       truth = c(1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0)
#'     , pred  = c(1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0)
#'   )
#'
#' confusion_matrix(df$truth, df$pred, thresholds = 1)
#'
#' # Example 2: Use with a logistic regression model
#' mod <- glm(
#'   formula = spam ~ word_freq_our + word_freq_over + capital_run_length_total
#' , data = spambase
#' , family = binomial()
#' )
#'
#' confusion_matrix(mod)
#' confusion_matrix(mod, thresholds = 0.5)
#'
#' @export
#' @rdname confusion_matrix
confusion_matrix <- function(..., thresholds = NULL, confint_method = "logit", alpha = getOption("qwraps2_alpha", 0.05)) {
  UseMethod("confusion_matrix")
}

#' @export
#' @rdname confusion_matrix
confusion_matrix.default <- function(truth, predicted, ..., thresholds = NULL, confint_method = "logit", alpha = getOption("qwraps2_alpha", 0.05)) {

  truth <- as.integer(truth)

  # assumption checks
  stopifnot(!any(is.na(truth)), !any(is.na(predicted)))
  stopifnot(length(truth) == length(predicted))
  stopifnot(all(truth %in% c(0L, 1L)))
  stopifnot(is.numeric(predicted))

  if (is.null(thresholds)) {
    thresholds <- unique(predicted)
  } else {
    stopifnot(is.numeric(thresholds))
    thresholds <- unique(thresholds)
  }
  thresholds <- sort(unique(c(-Inf, thresholds, Inf)))

  cm_cells <-
    lapply(thresholds,
           function(threshold) {
             cells <-
               list(  threshold = threshold
                    , TP        = sum(truth == 1 & predicted >= threshold)
                    , TN        = sum(truth == 0 & predicted <  threshold)
                    , FP        = sum(truth == 0 & predicted >= threshold)
                    , FN        = sum(truth == 1 & predicted <  threshold))
           })
  sen <- sapply(cm_cells, do.call, what = sensitivity)
  spc <- sapply(cm_cells, do.call, what = specificity)
  ppv <- sapply(cm_cells, do.call, what = PPV)
  npv <- sapply(cm_cells, do.call, what = NPV)
  acc <- sapply(cm_cells, do.call, what = accuracy)

  auroc <- -traprule(x = 1 - spc, y = sen)
  auprc <- -traprule(x = sen, y = ppv)

  # marginal sums
  N <- unique(sapply(cm_cells, function(x) x[["TP"]] + x[["FP"]] + x[["TN"]] + x[["FN"]]))
  stopifnot(length(N) == 1L)
  condition_P <- unique(sapply(cm_cells, function(x) x[["TP"]] + x[["FN"]]))
  stopifnot(length(condition_P) == 1L)
  condition_N <- unique(sapply(cm_cells, function(x) x[["TN"]] + x[["FP"]]))
  stopifnot(length(condition_N) == 1L)
  predicted_P <- sapply(cm_cells, function(x) x[["TP"]] + x[["FP"]])
  predicted_N <- sapply(cm_cells, function(x) x[["FN"]] + x[["TN"]])

  # confidence intervals
  sen_ci <- do.call(rbind, Map(proportion_confint, p = sen, n = condition_P, method = confint_method, alpha = alpha))
  spc_ci <- do.call(rbind, Map(proportion_confint, p = spc, n = condition_N, method = confint_method, alpha = alpha))
  ppv_ci <- do.call(rbind, Map(proportion_confint, p = ppv, n = predicted_P, method = confint_method, alpha = alpha))
  npv_ci <- do.call(rbind, Map(proportion_confint, p = npv, n = predicted_N, method = confint_method, alpha = alpha))
  acc_ci <- do.call(rbind, Map(proportion_confint, p = acc, n = N,           method = confint_method, alpha = alpha))

  cm_stats <- lapply(cm_cells, as.data.frame)
  cm_stats <- do.call(rbind, cm_stats)
  cm_stats <- cbind(cm_stats
         , sensitivity     = sen
         , sensitivity_lcl = sen_ci[, 1]
         , sensitivity_ucl = sen_ci[, 2]
         , specificity     = spc
         , specificity_lcl = spc_ci[, 1]
         , specificity_ucl = spc_ci[, 2]
         , ppv             = ppv
         , ppv_lcl         = ppv_ci[, 1]
         , ppv_ucl         = ppv_ci[, 2]
         , npv             = npv
         , npv_lcl         = npv_ci[, 1]
         , npv_ucl         = npv_ci[, 2]
         , accuracy        = acc
         , accuracy_lcl    = acc_ci[, 1]
         , accuracy_ucl    = acc_ci[, 2]
         , youden          = sapply(cm_cells, do.call, what = youden)
         , mcc            = sapply(cm_cells, do.call, what = MCC)
         , f1             = sapply(cm_cells, do.call, what = F1)
         )

  rtn <-
    list(
           cm_stats       = cm_stats
         , auroc          = auroc
         , auroc_ci       = proportion_confint(p = auroc, n = N, method = confint_method, alpha = alpha)
         , auprc          = auprc
         , auprc_ci       = proportion_confint(p = auprc, n = N, method = confint_method, alpha = alpha)
         , confint_method = confint_method
         , alpha          = alpha
         , prevalence     = condition_P / (condition_P + condition_N)
         )

  class(rtn) <- c("qwraps2_confusion_matrix")
  rtn
}

#' @param formula column (known) ~ row (test) for building the confusion matrix
#' @param data environment containing the variables listed in the formula
#' @export
#' @rdname confusion_matrix
confusion_matrix.formula <- function(formula, data = parent.frame(), ..., thresholds = NULL, confint_method = "logit", alpha = getOption("qwraps2_alpha", 0.05)) {
  cl <- as.list(match.call())[-1]

  mf <- stats::model.frame(formula, data)
  cl[["truth"]] <- mf[[1]]
  cl[["predicted"]] <- mf[[2]]
  cl[["formula"]] <- NULL
  cl[["data"]] <- NULL

  do.call(confusion_matrix, cl)
}

#' @param x a \code{glm} object
#' @export
#' @rdname confusion_matrix
confusion_matrix.glm <- function(x, ..., thresholds = NULL, confint_method = "logit", alpha = getOption("qwraps2_alpha", 0.05)) {
  stopifnot(x[["family"]][["family"]] == "binomial")
  truth <- x[["y"]]
  pred  <- stats::predict(x, type = "response")
  confusion_matrix(truth = truth, predicted = pred, ..., thresholds = thresholds, confint_method = confint_method, alpha = alpha)
}

#' @rdname confusion_matrix
#' @export
print.qwraps2_confusion_matrix <- function(x, ...) {
  NextMethod(print, x)
  invisible(x)
}

################################################################################
# non-exported functions
accuracy <- function(TP, TN, FP, FN, ...) {
  (TP + TN) / (TP + TN + FP + FN)
}

sensitivity <- function(TP, TN, FP, FN, ...) {
  # The following rule to deal with division by zero follows the GERBIL
  # convention from the DICE group.
  # <URL: https://github.com/dice-group/gerbil/wiki/Precision,-Recall-and-F1-measure>:
  if ((TP + FP + FN) == 0) {
    rtn <- 1
  } else if ((TP == 0) & (FP + FN > 0)) {
    rtn <- 0
  } else {
    rtn <- TP / (TP + FN)
  }

  rtn
}

specificity <- function(TP, TN, FP, FN, ...) {
  TN  / (TN + FP)
}

precision <- PPV <- function(TP, TN, FP, FN, ...) {
  # The following rule to deal with division by zero follows the GERBIL
  # convention from the DICE group.
  # <URL: https://github.com/dice-group/gerbil/wiki/Precision,-Recall-and-F1-measure>:
  if ((TP + FP + FN) == 0) {
    rtn <- 1
  } else if ((TP == 0) & (FP + FN > 0)) {
    rtn <- 0
  } else {
    rtn <- TP / (TP + FP)
  }
  rtn
}

NPV <- function(TP, TN, FP, FN, ...) {
  rtn <- TN  / (TN + FN)
  rtn
}

FNR <- function(TP, TN, FP, FN, ...) {
  1 - sensitivity(TP, TN, FP, FN)
}

FPR <- function(TP, TN, FP, FN, ...) {
  1 - specificity(TP, TN, FP, FN)
}

FDR <- function(TP, TN, FP, FN, ...) {
  1 - PPV(TP, TN, FP, FN)
}

# False Omission Rate
FOR <- function(TP, TN, FP, FN, ...) {
  1 - NPV(TP, TN, FP, FN)
}

F1 <- function(TP, TN, FP, FN, ...) {
  if ((TP + FP + FN) == 0) {
    rtn <- 1
  } else if ((TP == 0) & (FP + FN > 0)) {
    rtn <- 0
  } else {
    rtn <- (2 * TP) / (2 * TP + FP + FN)
  }
  rtn
}

youden <- function(TP, TN, FP, FN, ...) {
  sensitivity(TP, TN, FP, FN) + specificity(TP, TN, FP, FN) - 1
}

# Matthews Correlation Coefficient
MCC <- function(TP, TN, FP, FN, ...) {
  # ((TP * TN) - (FP * FN)) / sqrt((TP + FP) * (TP+FN) * (TN+FP) * (TN+FN))

  # because there can be very large numbers work with logs when needed
  # denominator <- 0.5 * ( log(TP+FP) + log(TP+FN) + log(TN+FP) + log(TN+FN) )

  if ( ((TP + FP) == 0) | ((TP + FN) == 0) | ((TN + FP) == 0) | ((TN + FN) == 0)) {
    rtn <- NA_real_
  } else {
    denominator <- exp(0.5 * ( log(TP+FP) + log(TP+FN) + log(TN+FP) + log(TN+FN) ))

    if (TP == 0 | TN == 0 | FP == 0 | FN == 0 ) {
      numerator <- (TP * TN) - (FP * FN)
    } else {
      numerator <- exp(log(TP) + log(TN)) - exp(log(FP) + log(FN))
    }

    rtn <- numerator / denominator
  }

  rtn
}

proportion_confint <- function(p, n, method = "logit", alpha = getOption("qwraps2_alpha", 0.05)) {
  if (n == 0L || is.na(p)) {
    return(c(NA_real_, NA_real_))
  }

  z <- stats::qnorm(1 - alpha/2)

  if (method == "logit") {
    if (p <= 0) {
      rtn <- c(0, 0)
    } else if (p >= 1) {
      rtn <- c(1, 1)
    } else {
      m <- stats::qlogis(p)
      tau <- 1 / sqrt( n * p * (1 - p) )
      rtn <- stats::plogis(m + c(-z, z) * tau)
    }
  } else if (method == "binomial") {
    rtn <- p + c(-z, z) * sqrt( p * (1 - p) / n)
  } else if (method == "wilson_score") {
    rtn <- 1 / (1 + 1/n * z^2) * (p + 1 / (2 * n) * z^2 + c(-z, z) * sqrt( 1 / n * p * (1 - p) + 1 / (4 * n^2) * z^2))
  } else {
    stop("method not in c('logit', 'binomial', 'wilson_score')")
  }
  rtn
}

#' @title Geometric Mean and Standard deviation
#'
#' @description A function for calculating and formatting geometric means and
#' standard deviations.
#'
#' @details
#' Given a numeric vector, \code{gmean_sd} will return a character string with
#' the geometric mean and standard deviation.  Formatting of the output will be
#' extended in future versions.
#'
#' @param x a numeric vector
#' @param digits digits to the right of the decimal point to return in the
#' percentage estimate.
#' @param na_rm if true, omit NA values
#' @param show_n defaults to \dQuote{ifNA}.  Other options are \dQuote{always} or
#' \dQuote{never}.
#' @param denote_sd a character string set to either "pm" or "paren" for reporting 'mean
#' \eqn{\pm} sd' or 'mean (sd)'
#' @param markup character string with value \dQuote{latex} or \dQuote{markdown}
#' @param ... pass through
#'
#' @seealso \code{\link{mean_sd}}, \code{\link{gmean}}, \code{\link{gsd}}
#'
#' @return a character vector of the formatted values
#'
#' @examples
#'
#' gmean_sd(mtcars$mpg, markup = "latex")
#' gmean_sd(mtcars$mpg, markup = "markdown")
#'
#' @export
gmean_sd <- function(x,
                     digits = getOption("qwraps2_frmt_digits", 2),
                     na_rm = FALSE,
                     show_n = "ifNA",
                     denote_sd = "pm",
                     markup = getOption("qwraps2_markup", "latex"),
                     ...) {

  cl <- as.list(match.call())
  if ("na.rm" %in% names(cl)) {
    if ("na_rm" %in% names(cl)) {
      warning("qwraps2::gmean_sd uses the argument `na_rm`, not `na.rm`. `na_rm` is used preferentially.")
    } else {
      na_rm <- cl$na.rm
      warning("qwraps2::gmean_sd uses the argument `na_rm`, not `na.rm`.")
    }
  }
  stopifnot(inherits(na_rm, "logical"))
  stopifnot(length(markup) == 1L)
  stopifnot(markup %in% c("latex", "markdown"))
  stopifnot(show_n %in% c("ifNA", "always", "never"))

  n <- sum(!is.na(x))

  m <- gmean(x, na_rm)
  s <- gsd(x, na_rm)

  if (show_n == "always" | (show_n == "ifNA" & any(is.na(x)))) {
    rtn <- paste0(frmt(as.integer(n), digits), "; ", frmt(m, digits), " $\\pm$ ", frmt(s, digits))
  } else {
    rtn <- paste0(frmt(m, digits), " $\\pm$ ", frmt(s, digits))
  }

  if (denote_sd == "paren") {
    rtn <- gsub("\\$\\\\pm\\$\\s(.*)", "\\(\\1\\)", rtn)
  }

  if (markup == "markdown") {
    rtn <- gsub("\\$\\\\pm\\$", "&plusmn;", rtn)
  }

  return(rtn)
}

library(qwraps2)

# use the example and check the results
e <- new.env()
example("lazyload_cache_dir", local = e)

stopifnot(isTRUE(all.equal(e$kenv$mpg_by_wt_hp, e$env1$mpg_by_wt_hp)))
stopifnot(isTRUE(all.equal(e$env1$mpg_by_wt_hp, e$env3$mpg_by_wt_hp)))

stopifnot(isTRUE(all.equal(e$kenv$mpg_by_wt_hp_am, e$env2$mpg_by_wt_hp_am)))
stopifnot(isTRUE(all.equal(e$env2$mpg_by_wt_hp_am, e$env3$mpg_by_wt_hp_am)))

stopifnot(isTRUE(all.equal(e$kenv$x, exp(1))))
stopifnot(isTRUE(all.equal(e$env1$x, pi)))
stopifnot(isTRUE(all.equal(e$env2$x, exp(1))))
stopifnot(isTRUE(all.equal(e$env3$x, exp(1))))

################################################################################
##                              Extended testing                              ##


################################################################################
#                                 End of File                                  #
################################################################################

library(qwraps2)

# basic formatting
g_mean <- exp(mean(log(mtcars$mpg)))
g_var  <- exp( var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars))
g_sd   <- exp(sqrt( var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars)))

stopifnot(identical(gmean(mtcars$mpg), g_mean))
stopifnot(identical(gvar(mtcars$mpg), g_var))
stopifnot(identical(gsd(mtcars$mpg), g_sd))

g_mean <- frmt(g_mean)
g_sd   <- frmt(g_sd)
stopifnot(identical(gmean_sd(mtcars$mpg), paste(g_mean, "$\\pm$", g_sd)))

# error given if show_n is not as expected
stopifnot(inherits(tryCatch(gmean_sd(mtcars$mpg, show_n = TRUE), error = function(e) e), "error"))


# warning if na.rm is used
stopifnot(inherits(tryCatch(gmean_sd(mtcars$mpg, na.rm = TRUE), warning = function(w) w), "warning"))

# warning if na_rm is provided along with na.rm
stopifnot(inherits(tryCatch(gmean_sd(mtcars$mpg, na_rm = TRUE, na.rm = TRUE), warning = function(w) w), "warning"))

#' @title qable: an extended version of knitr::kable
#'
#' @description Create a simple table via \code{\link[knitr]{kable}} with row
#' groups and rownames similar to those of \code{\link[Hmisc]{latex}} from the
#' Hmisc package or \code{\link[htmlTable]{htmlTable}} from the htmlTable
#' package.
#'
#' @details
#'
#' \code{rtitle} can be used to add a title to the column constructed by the
#' \code{rgroup} and \code{rnames}.  The basic layout of a table generated by
#' \code{qable} is:
#'
#' \tabular{lcc}{
#'  rtitle    \tab cnames[1] \tab cnames[2] \cr
#'  rgroup[1] \tab           \tab           \cr
#'  rnames[1] \tab x[1, 1]   \tab x[1, 2]   \cr
#'  rnames[2] \tab x[2, 1]   \tab x[2, 2]   \cr
#'  rnames[3] \tab x[3, 1]   \tab x[3, 2]   \cr
#'  rgroup[2] \tab           \tab           \cr
#'  rnames[4] \tab x[4, 1]   \tab x[4, 1]   \cr
#'  rnames[5] \tab x[5, 1]   \tab x[5, 1]   \cr
#' }
#'
#' Passing arguments to \code{\link[knitr]{kable}} is done via the list
#' \code{kable_args}.  This is an improvement in 0.6.0 to address arguments with
#' different use between qable and kable but the same name, notably
#' \code{format}.  Within the print method for \code{qwraps2_qable} objects,
#' some default arguments for knitr::kable are created.
#'
#' Defaults if the named element of \code{kable_args} is missing:
#' \code{kable_args$format} will be "latex" if \code{markup = "latex"} and will
#' be \code{"pipe"} if \code{markup = "markdown"}.
#'
#' \code{kable_args$escape = !(markup = "latex")}
#'
#' \code{kable_args$row.names} defaults to \code{FALSE}
#'
#' \code{kable_args$col.names} defaults to \code{colnames(x)}
#'
#' @seealso
#' \code{\link[knitr]{kable}}
#'
#' \code{\link{summary_table}}, for an example of building a data summary table.
#'
#' For more detail on arguments you can pass via \code{kable_args} look at the
#' non-exported functions from the knitr package \code{knitr:::kable_latex},
#' \code{knitr:::kable_markdown}, or others.
#'
#' @param x \code{matrix} or \code{data.frame} to be turned into a qable
#' @param rtitle a row grouping title. See Details.
#' @param rgroup a named numeric vector with the name of the row group and the
#' number of rows within the group.  \code{sum(rowgroup) == nrow(x)}.
#' @param rnames a character vector of the row names
#' @param cnames column names
#' @param markup the markup language to use expected to be either "markdown" or
#' "latex"
#' @param kable_args a list of named arguments to send to
#' \code{\link[knitr]{kable}}.  See Details.
#' @param ... pass through
#'
#' @return \code{qable} returns a \code{qwraps2_qable} object that is just a character matrix with
#' some additional attributes and the print method returns, invisibly, the
#' object passed to print.
#'
#' @examples
#' data(mtcars)
#' x <- qable(mtcars)
#' x
#' qable(mtcars, markup = "markdown")
#'
#' # by make
#' make <- sub("^(\\w+)\\s?(.*)$", "\\1", rownames(mtcars))
#' make <- c(table(make))
#'
#' # A LaTeX table with a vertical bar between each column
#' qable(mtcars[sort(rownames(mtcars)), ], rgroup = make)
#'
#' # A LaTeX table with no vertical bars between columns
#' qable(mtcars[sort(rownames(mtcars)), ], rgroup = make, kable_args = list(vline = ""))
#'
#' # a markdown table
#' qable(mtcars[sort(rownames(mtcars)), ], rgroup = make, markup = "markdown")
#'
#' # define your own column names
#' qable(mtcars[sort(rownames(mtcars)), ],
#'       rgroup = make,
#'       cnames = toupper(colnames(mtcars)),
#'       markup = "markdown")
#'
#' # define your own column names and add a title
#' qable(mtcars[sort(rownames(mtcars)), ],
#'       rtitle = "Make & Model",
#'       rgroup = make,
#'       cnames = toupper(colnames(mtcars)),
#'       markup = "markdown")
#'
#' @export
#' @rdname qable
qable <- function(x, rtitle = "", rgroup = numeric(0), rnames = rownames(x), cnames = colnames(x), markup = getOption("qwraps2_markup", "latex"), kable_args = list(), ...) {
  UseMethod("qable")
}

#' @export
qable.data.frame <- function(x, rtitle = "", rgroup = numeric(0), rnames = rownames(x), cnames = colnames(x), markup = getOption("qwraps2_markup", "latex"), kable_args = list(), ...) {
  qable(as.matrix(x), rtitle = rtitle, rgroup = rgroup, rnames = rnames, cnames = cnames, markup = markup, kable_args = kable_args, ...)
}

#' @export
qable.matrix <- function(x, rtitle = "", rgroup = numeric(0), rnames = rownames(x), cnames = colnames(x), markup = getOption("qwraps2_markup", "latex"), kable_args = list(), ...) {

  if (!(markup %in% c("latex", "markdown"))) {
    stop("markup is either 'latex' or 'markdown'")
  }

  if (length(rgroup) > 0) {
    if (is.null(names(rgroup)) || any(!nzchar(names(rgroup)))) {
      stop("rgroup must be a named vector.")
    }
    if (!is.numeric(rgroup) || any(is.na(rgroup)) || any(rgroup <= 0) || any(rgroup != as.integer(rgroup))) {
      stop("rgroup must contain positive integer counts.")
    }
    if (sum(rgroup) != nrow(x)) {
      stop("sum(rgroup) must equal nrow(x).")
    }
  }

  xmat <- matrix("~", nrow = nrow(x) + length(rgroup), ncol = 1 + ncol(x))

  if (length(rgroup) > 0) {
    rg_idx <- cumsum(c(1, 1 + rgroup[-length(rgroup)]))
    xmat[-rg_idx, -1] <- as.matrix(x)

    if (markup == "latex") {
      xmat[rg_idx, 1] <- paste0("\\bf{", names(rgroup), "}")
      xmat[-rg_idx, 1] <- paste("~~", rnames)
    } else {
      xmat[rg_idx, 1] <- paste0("**", names(rgroup), "**")
      xmat[-rg_idx, 1] <- paste("&nbsp;&nbsp;", rnames)
    }
  } else {
    xmat[, 1] <- rnames
    xmat[, -1] <- as.matrix(x)
  }

  if (markup == "markdown") {
    xmat <- apply(xmat, 1:2, function(x) gsub("~", "&nbsp;&nbsp;", x))
  }

  if (missing(rtitle)) {
    cnames <- c("", cnames)
  } else {
    cnames <- c(rtitle, cnames)
  }
  colnames(xmat) <- cnames
  class(xmat) <- "qwraps2_qable"
  attr(xmat, "qable_args") <- list(rtitle = rtitle, rgroup = rgroup, rnames = rnames, cnames = cnames, markup = markup, kable_args = kable_args, ...)
  xmat
}

#' @export
print.qwraps2_qable <- function(x, ...) {
  kargs <- attr(x, "qable_args")[["kable_args"]]

  if (!("format" %in% names(kargs))) {
    if (attr(x, "qable_args")[["markup"]] == "markdown") {
      kargs$format <- "pipe"
    } else {
      kargs$format <- "latex"
    }
  }

  if (!("escape" %in% names(kargs))) {
    kargs$escape <- !(attr(x, "qable_args")[["markup"]] == "latex")
  }

  if (!("row.names" %in% names(kargs))) {
    kargs$row.names <- FALSE
  }

  if (!("col.names" %in% names(kargs))) {
    if (ncol(x) == length(attr(x, "qable_args")[["cnames"]])) {
      kargs$col.names <- c(attr(x, "qable_args")[["cnames"]])
    } else {
      kargs$col.names <- c(attr(x, "qable_args")[["rtitle"]], attr(x, "qable_args")[["cnames"]])
    }
  }

  k <- do.call(what = knitr::kable, args = c(list(x = x), kargs, ...))
  print(k)
  invisible(x)
}

#' @export
#' @export
cbind.qwraps2_qable <- function(...) {
  tabs <- list(...)

  # verify number of rows are same for all inputs
  n <-
    sapply(tabs, function(x) {
             if (is.matrix(x)) {
               n <- nrow(x)
             } else if (is.vector(x)) {
               n <- length(x)
             }
             n
  })
  if (any(n != 1 & n != n[1])) {
    stop(sprintf("nrow, or length of a vector, for all inputs needs to be %s, the nrow of the first object.", n[1]))
  }

  # verify the first column is the same for all the qables, error if not
  rgroups <- lapply(tabs, function(x) { if (inherits(x, "qwraps2_qable")) x[, 1]} )
  rgroups <- Filter(function(x) !is.null(x), x = rgroups)
  rgroups <- sapply(rgroups, identical, y = rgroups[[1]])
  if (length(rgroups) > sum(rgroups)) {
    stop("the rgroup are not identical for all the input qables.")
  }


  # Get all the columns to cbind together
  clnms <- colnames(tabs[[1]])

  for (i in 2:length(tabs)) {
    if (inherits(tabs[[i]], "qwraps2_qable")) {
      clnms <- c(clnms, colnames(tabs[[i]])[-1])
      tabs[[i]] <- matrix(tabs[[i]][, -1], nrow = nrow(tabs[[1]]))
    } else {
      nms <- colnames(tabs[[i]])
      if (is.null(nms)) {
        nms <- names(tabs)[i]
      }
      if (!is.null(ncol(tabs[[i]])) && length(nms) < ncol(tabs[[i]])) {
        nms <- paste0(nms, 1:ncol(tabs[[i]]))
      }
      clnms <- c(clnms, nms)
      tabs[[i]] <- matrix(tabs[[i]], nrow = nrow(tabs[[1]]))
    }
  }

  rtn <- do.call(cbind, lapply(tabs, unclass))

  colnames(rtn) <- clnms
  attr(rtn, "qable_args") <- attr(tabs[[1]], "qable_args")
  attr(rtn, "qable_args")[["cnames"]] <- clnms
  class(rtn) <- class(tabs[[1]])
  rtn
}

#' @seealso \code{rbind}
#' @export
rbind.qwraps2_qable <- function(..., deparse.level = 1) {
  tabs <- list(...)

  out <- do.call(rbind, args = c(lapply(tabs, unclass), list(deparse.level = deparse.level)))

  attr(out, "qable_args") <- attr(tabs[[1]], "qable_args")
  attr(out, "qable_args")[["rgroup"]] <- do.call(c, lapply(tabs, function(x) attr(x, "qable_args")[["rgroup"]]))
  attr(out, "qable_args")[["rnames"]] <- do.call(c, lapply(tabs, function(x) attr(x, "qable_args")[["rnames"]]))

  class(out) <- class(tabs[[1]])
  out
}

#' @title Receiver Operating Characteristic and Precision-Recall Curves
#'
#' @description Construction of ROC and PRC data and plots.
#'
#' @details
#'
#' The area under the curve (AUC) is determined by a trapezoid approximation for
#' both the AUROC and AUPRC.
#'
#' More details and examples for graphics within qwraps2 are in the
#' vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
#'
#' @param x an object
#' @param ... pass through
#'
#' @return a ggplot.  Minimal aesthetics have been used so that the user may
#' modify the graphic as desired with ease.
#'
#' @examples
#' #########################################################
#' # Example 1
#'
#' df <-
#'   data.frame(
#'       truth = c(1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0)
#'     , pred  = c(1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0)
#'   )
#'
#' cm <- confusion_matrix(df$truth, df$pred)
#' qroc(cm)
#' qprc(cm)
#'
#' #########################################################
#' # Getting a ROC or PRC plot from a glm object:
#'
#' mod <- glm(
#'   formula = spam ~ word_freq_our + word_freq_over + capital_run_length_total
#' , data = spambase
#' , family = binomial()
#' )
#'
#' qroc(mod)
#' qprc(mod)
#'
#' #########################################################
#' # View the vignette for more examples
#' \dontrun{
#' vignette("qwraps2-graphics")
#' }
#'
#' @name
#' qroc-qprc
NULL

#' @export
#' @rdname qroc-qprc
qroc <- function(x, ...) {
  UseMethod("qroc")
}

#' @export
#' @rdname qroc-qprc
qroc.default <- function(x, ...) {
  qroc_ggplot(x, ...)
}

#' @export
#' @rdname qroc-qprc
qroc.qwraps2_confusion_matrix <- function(x, ...) {
  qroc_ggplot(x[["cm_stats"]], ...)
}

#' @export
#' @rdname qroc-qprc
qroc.glm <- function(x, ...) {
  qroc(confusion_matrix(x, ...))
}

qroc_ggplot <- function(data) {
  stopifnot("specificity" %in% names(data))
  stopifnot("sensitivity" %in% names(data))
  data[["FPR"]] <- 1 - data[["specificity"]]
  data[["TPR"]] <- data[["sensitivity"]]
  ggplot2::ggplot(data) +
  eval(substitute(ggplot2::aes(x = X, y = Y), list(X = as.name("FPR"), Y = as.name("TPR")))) +
  ggplot2::geom_point() +
  ggplot2::geom_path() +
  ggplot2::xlim(0, 1) +
  ggplot2::ylim(0, 1) +
  ggplot2::annotate(geom = "segment", x = 0, y = 0, xend = 1, yend = 1, color = "black", linetype = 2)
}

#' @export
#' @rdname qroc-qprc
qprc <- function(x, ...) {
  UseMethod("qprc")
}

#' @export
#' @rdname qroc-qprc
qprc.default <- function(x, ...) {
  qprc_ggplot(x, ...)
}

#' @export
#' @rdname qroc-qprc
qprc.qwraps2_confusion_matrix <- function(x, ...) {
  qprc_ggplot(x[["cm_stats"]], prevalence = x$prevalence, ...)
}

#' @export
#' @rdname qroc-qprc
qprc.glm <- function(x, ...) {
  qprc(confusion_matrix(x, ...))
}

qprc_ggplot <- function(data, prevalence = NULL) {
  stopifnot("sensitivity" %in% names(data)) # recall
  stopifnot("ppv" %in% names(data)) # Precision
  g <-
    ggplot2::ggplot(data = data) +
    eval(substitute(ggplot2::aes(x = X, y = Y), list(X = as.name("sensitivity"), Y = as.name("ppv")))) +
    ggplot2::geom_point() +
    ggplot2::geom_path() +
    ggplot2::xlim(0, 1) +
    ggplot2::ylim(0, 1)
  if (!is.null(prevalence)) {
    stopifnot(is.numeric(prevalence))
    g <- g + ggplot2::geom_hline(yintercept = prevalence)
  }
  g
}

#' Data Summary Tables
#'
#' Tools useful for building data summary tables.
#'
#' \code{summary_table} can be used to generate good-looking, simple tables in
#' LaTeX or markdown.  Functions like \code{xtable::print.xtable} and
#' \code{Hmisc::latex} provide many more tools for formatting tables.  The
#' purpose of \code{summary_table} is to generate readable tables quickly while
#' summarizing a data set.
#'
#' Creating a list-of-lists of summary functions to apply to a data set will
#' allow the exploration of the whole data set and grouped data sets.  In the
#' example provided on this page we see a set of summary measures for the
#' \code{\link[datasets]{mtcars}} data set and the construction of a table for
#' the whole data set and for a grouped data set.
#'
#'
#' The list-of-lists should be thought of as follows:  the outer list defines
#' row groups, the inner lists define the rows within each row group.
#'
#' More detailed use of these functions can be found in the
#' \code{vignette("qwraps2-summary-table", package = "qwraps2")}.
#'
#' The \code{print} method for the \code{qwraps2_summary_table} objects is just
#' a simple wrapper for \code{\link{qable}}.
#'
#' @param x a \code{data.frame}.
#' @param summaries a list of lists of formulas for summarizing the data set.
#' See Details and examples.
#' @param by a character vector of variable names to generate the summary by,
#' that is, one column for each unique value or combination of values of the
#' variables specified.
#' @param qable_args additional values passed to \code{\link{qable}}
#' @param ... pass through
#'
#' @seealso \code{\link{qsummary}} for generating the summaries,
#' \code{\link{qable}} for marking up \code{qwraps2_summary_table} objects.
#' The \code{vignette("qwraps2-summary-table", package = "qwraps2")} for
#' detailed use of these functions and caveats.
#'
#' @return a \code{qwraps2_summary_table} object.
#'
#' @examples
#' summary_table(mtcars[, c("mpg", "cyl", "am")])
#' summary_table(mtcars[, c("mpg", "cyl", "am")], by = "am")
#'
#' # A list-of-lists for the summaries arg.  This object is of the basic form:
#' #
#' # list("row group A" =
#' #      list("row 1A" = ~ <summary function>,
#' #           "row 2A" = ~ <summary function>),
#' #      "row group B" =
#' #      list("row 1B" = ~ <summary function>,
#' #           "row 2B" = ~ <summary function>,
#' #           "row 3B" = ~ <summary function>))
#'
#' our_summaries <-
#'   list("Miles Per Gallon" =
#'          list("min"  = ~ min(mpg),
#'               "mean" = ~ mean(mpg),
#'               "mean &plusmn; sd" = ~ qwraps2::mean_sd(mpg),
#'               "max"  = ~ max(mpg)),
#'        "Weight" =
#'          list("median" = ~ median(wt)),
#'        "Cylinders" =
#'          list("4 cyl: n (%)" = ~ qwraps2::n_perc0(cyl == 4),
#'               "6 cyl: n (%)" = ~ qwraps2::n_perc0(cyl == 6),
#'               "8 cyl: n (%)" = ~ qwraps2::n_perc0(cyl == 8)))
#'
#' # Going to use markdown for the markup language in this example,  the original
#' # option will be reset at the end of the example.
#' orig_opt <- options()$qwraps2_markup
#' options(qwraps2_markup = "markdown")
#'
#' # The summary table for the whole mtcars data set
#' whole_table <- summary_table(mtcars, our_summaries)
#' whole_table
#'
#' # The summary table for mtcars grouped by am (automatic or manual transmission)
#' # This will generate one column for each level of mtcars$am
#' grouped_by_table <-
#'   summary_table(mtcars, our_summaries, by = "am")
#' grouped_by_table
#'
#' # an equivalent call if you are using the tidyverse:
#' if (requireNamespace("dplyr", quietly = TRUE)) {
#'   summary_table(dplyr::group_by(mtcars, am), our_summaries)
#'
#'   # To build a table with a column for the whole data set and each of the am
#'   # levels
#'   cbind(whole_table, grouped_by_table)
#'
#'   # Adding a caption for a LaTeX table
#'   print(whole_table, caption = "Hello world", markup = "latex")
#'
#'   # A **warning** about grouped_df objects.
#'   # If you use dplyr::group_by or
#'   # dplyr::rowwise to manipulate a data set and fail to use dplyr::ungroup you
#'   # might find a table that takes a long time to create and does not summarize the
#'   # data as expected.  For example, let's build a data set with twenty subjects
#'   # and injury severity scores for head and face injuries.  We'll clean the data
#'   # by finding the max ISS score for each subject and then reporting summary
#'   # statistics thereof.
#'   set.seed(42)
#'   dat <- data.frame(id = letters[1:20],
#'                     head_iss = sample(1:6, 20, replace = TRUE, prob = 10 * (6:1)),
#'                     face_iss = sample(1:6, 20, replace = TRUE, prob = 10 * (6:1)))
#'   dat <- dplyr::group_by(dat, id)
#'   dat <- dplyr::mutate(dat, iss = max(head_iss, face_iss))
#'
#'   iss_summary <-
#'     list("Head ISS" =
#'          list("min"    = ~ min(head_iss),
#'               "median" = ~ median(head_iss),
#'               "max"    = ~ max(head_iss)),
#'          "Face ISS" =
#'          list("min"    = ~ min(face_iss),
#'               "median" = ~ median(face_iss),
#'               "max"    = ~ max(face_iss)),
#'          "Max ISS" =
#'          list("min"    = ~ min(iss),
#'               "median" = ~ median(iss),
#'               "max"    = ~ max(iss)))
#'
#'   # Want: a table with one column for all subjects with nine rows divided up into
#'   # three row groups.  However, the following call will create a table with 20
#'   # columns, one for each subject because dat is a grouped_df
#'   summary_table(dat, iss_summary)
#'
#'   # Ungroup the data.frame to get the correct output
#'   summary_table(dplyr::ungroup(dat), iss_summary)
#' }
#'
#' ################################################################################
#' # The Default call will work with non-syntactically valid names and will
#' # generate a table with statistics defined by the qsummary call.
#' summary_table(mtcars, by = "cyl")
#'
#' # Another example from the diamonds data
#' data("diamonds", package = "ggplot2")
#' diamonds["The Price"] <- diamonds$price
#' diamonds["A Logical"] <- sample(c(TRUE, FALSE), size = nrow(diamonds), replace = TRUE)
#'
#' # the next two lines are equivalent.
#' summary_table(diamonds)
#' summary_table(diamonds, qsummary(diamonds))
#'
#' summary_table(diamonds, by = "cut")
#'
#' summary_table(diamonds,
#'               summaries =
#'               list("My Summary of Price" =
#'                    list("min price" = ~ min(price),
#'                         "IQR"       = ~ stats::IQR(price))),
#'               by = "cut")
#'
#' ################################################################################
#' # Data sets with missing values
#' temp <- mtcars
#' temp$cyl[5] <- NA
#' temp$am[c(1, 5, 10)] <- NA
#' temp$am <- factor(temp$am, levels = 0:1, labels = c("Automatic", "Manual"))
#' temp$vs <- as.logical(temp$vs)
#' temp$vs[c(2, 6)] <- NA
#' qsummary(temp[, c("cyl", "am", "vs")])
#' summary_table(temp[, c("cyl", "am", "vs")])
#'
#' ################################################################################
#' # Group by Multiple Variables
#' temp <- mtcars
#' temp$trans <- factor(temp$am, 0:1, c("Manual", "Auto"))
#' temp$engine <- factor(temp$vs, 0:1, c("V-Shaped", "Straight"))
#' summary_table(temp, our_summaries, by = c("trans", "engine"))
#'
#' ################################################################################
#' # binding tables together.  The original design and expected use of
#' # summary_table did not require a rbind, as all rows are defined in the
#' # summaries argument.  That said, here are examples of using cbind and rbind to
#' # build several different tables.
#' our_summary1 <-
#'   list("Miles Per Gallon" =
#'        list("min" = ~ min(mpg),
#'             "max" = ~ max(mpg),
#'             "mean (sd)" = ~ qwraps2::mean_sd(mpg)),
#'        "Displacement" =
#'        list("min" = ~ min(disp),
#'             "max" = ~ max(disp),
#'             "mean (sd)" = ~ qwraps2::mean_sd(disp)))
#'
#' our_summary2 <-
#'   list(
#'        "Weight (1000 lbs)" =
#'        list("min" = ~ min(wt),
#'             "max" = ~ max(wt),
#'             "mean (sd)" = ~ qwraps2::mean_sd(wt)),
#'        "Forward Gears" =
#'        list("Three" = ~ qwraps2::n_perc0(gear == 3),
#'             "Four"  = ~ qwraps2::n_perc0(gear == 4),
#'             "Five"  = ~ qwraps2::n_perc0(gear == 5))
#'        )
#'
#' tab1 <- summary_table(mtcars, our_summary1)
#' tab2 <- summary_table(mtcars, our_summary1, by = "am")
#' tab3 <- summary_table(mtcars, our_summary1, by = "vs")
#'
#' tab4 <- summary_table(mtcars, our_summary2)
#' tab5 <- summary_table(mtcars, our_summary2, by = "am")
#' tab6 <- summary_table(mtcars, our_summary2, by = "vs")
#'
#' cbind(tab1, tab2, tab3)
#' cbind(tab4, tab5, tab6)
#'
#' # row bind is possible, but it is recommended to extend the summary instead.
#' rbind(tab1, tab4)
#' summary_table(mtcars, summaries = c(our_summary1, our_summary2))
#'
#' \dontrun{
#'   cbind(tab1, tab4) # error because rows are not the same
#'   rbind(tab1, tab2) # error because columns are not the same
#' }
#'
#' ################################################################################
#' # reset the original markup option that was used before this example was
#' # evaluated.
#' options(qwraps2_markup = orig_opt)
#'
#' # Detailed examples in the vignette
#' # vignette("qwraps2-summary-table", package = "qwraps2")
#'
#'
#' @export
#' @rdname summary_table
summary_table <- function(x, summaries = qsummary(x), by = NULL, qable_args = list(), ...) {
  UseMethod("summary_table")
}

#' @export
summary_table.grouped_df <- function(x, summaries = qsummary(x), by = NULL, qable_args = list(), ...) {
  if (!is.null(by)) {
    warning("You've passed a grouped_df to summary_table and specified the `by` argument.  The `by` argument will be ignored.")
  }

  # this assumes dplyr version 0.8.0 or newer
  lbs <- names(attr(x, "groups"))
  lbs <- lbs[-length(lbs)]
  warning(paste0("grouped_df detected. Setting `by` argument to\n  c('", paste(lbs, collapse = "', '"), "')"))
  NextMethod(object = x, by = lbs, qable_args = qable_args, ...)
}

#' @export
summary_table.data.frame <- function(x, summaries = qsummary(x), by = NULL, qable_args = list(), ...) {

  if (!missing(summaries)) {
    if ( any(grepl("\\.data\\$", parse(text = summaries))) ) {
      warning("Use of the data pronoun is no longer required/encouraged.  The ability to use it has been deprecated.  See the documentation for summary_table, qsummary, and the vignettes for more detail.  The use of the data pronoun will be supported in version 0.5.0 of qwraps2 with this warning.")
    }
  }

  if (!is.null(by)) {
    subsets <- split(x, interaction(x[, by]))
  } else {
    subsets <- list(x)
  }

  rtn <- lapply(subsets, apply_summaries, summaries = summaries)

  if (length(rtn) > 1L) {
    clnms <- paste0(names(rtn), " (N = ", sapply(rtn, attr, "n"), ")")
  } else {
    clnms <- paste0(deparse(substitute(x), nlines = 1L, backtick = TRUE), " (N = ", frmt(nrow(x)), ")")
  }

  for (i in 1:length(rtn)) {
    colnames(rtn[[i]]) <- clnms[i]
    rtn[[i]] <- do.call(qable, c(list(x = rtn[[i]]), list(rgroup = attr(rtn[[i]], "rgroup")), qable_args)
    )
  }

  if (length(rtn) > 1) {
    rtn <- do.call(cbind.qwraps2_qable, rtn)
  } else {
    rtn <- rtn[[1]]
  }

  class(rtn) <- c("qwraps2_summary_table", "qwraps2_qable")
  rtn
}

apply_summaries <- function(summaries, x) {
  rtn <- lapply(summaries, lapply, stats::model.frame, x)
  rtn <- lapply(rtn, lapply, function(xx) if(nrow(xx) == 0) data.frame("NA") else xx)
  rtn <- lapply(rtn, lapply, function(y) {attr(y, "terms") <- NULL; y})
  rtn <- lapply(rtn, lapply, unlist)
  rtn <- lapply(rtn, lapply, function(y) {attr(y, "names") <- NULL; y})
  rtn <- lapply(rtn, unlist)
  rtn <- lapply(rtn, as.matrix, ncol = 1)

  rgroup <- sapply(rtn, nrow)

  rtn <- do.call(rbind, rtn)
  attr(rtn, "rgroup") <- rgroup
  attr(rtn, "n") <- nrow(x)
  rtn
}


#' @param numeric_summaries a list of functions to use for summarizing numeric
#' variables.  The functions need to be provided as character strings with the
#' single argument defined by the \code{\%s} symbol.
#' @param n_perc_args a list of arguments to pass to
#' \code{\link[qwraps2]{n_perc}} to be used with \code{character} or
#' \code{factor} variables within \code{x}.
#' @param env environment to assign to the resulting formulae

#' @rdname summary_table
#' @export
qsummary <- function(x, numeric_summaries, n_perc_args, env = parent.frame()) {
  UseMethod("qsummary")
}

#' @export
qsummary.grouped_df <- function(x, numeric_summaries, n_perc_args, env = parent.frame()) {
  NextMethod()
}

#' @export
qsummary.data.frame <- function(x,
                                numeric_summaries =
                                  list("minimum"      = "~ qwraps2::frmt(min(%s))",
                                       "median (IQR)" = "~ qwraps2::median_iqr(%s)",
                                       "mean (sd)"    = "~ qwraps2::mean_sd(%s)",
                                       "maximum"      = "~ qwraps2::frmt(max(%s))"),
                                n_perc_args = list(digits = 0, show_symbol = FALSE)
                                ,
                                env = parent.frame()) {

  npa <- lapply(n_perc_args, deparse)
  npa <- paste(paste(names(npa), npa, sep = " = "), collapse = ", ")

  rtn <-
    lapply(names(x),
           function(variable) {
             if (is.numeric(x[[variable]])) {
               summaries <- numeric_summaries
             } else if (is.character(x[[variable]]) | is.factor(x[[variable]])) {
               lvls <- levels(as.factor(x[[variable]]))
               qlvls <- vapply(lvls, deparse, character(1L))
               summaries <- as.list(paste0("~ qwraps2::n_perc(%s == ", qlvls, ", ", npa, ")"))
               summaries <- stats::setNames(summaries, lvls)
             } else if (is.logical(x[[variable]])) {
               summaries <- as.list(paste0("~ qwraps2::n_perc(%s, ", npa, ")"))
             } else if (inherits(x[[variable]], "Date")) {
               summaries <- list("first" = " ~ as.character(min(%s))", "last" = "~ as.character(max(%s))")
             } else {
               warning(sprintf("no default summary method for class '%s'", class(x[[variable]])))
               return(NULL)
             }

             if (any(is.na(x[[variable]]))) {
               summaries <- lapply(summaries, sprintf, "na.omit(%s)")
               summaries <- c(summaries, list("Unknown/Missing" = "~ qwraps2::n_perc(is.na(%s))"))
             }

             if (make.names(variable) == variable) {
               lapply(summaries, sprintf, variable)
             } else {
               lapply(summaries, sprintf, paste0("`", variable, "`"))
             }

           })
  rtn <- stats::setNames(rtn, names(x))

  rtn <- lapply(rtn, lapply, stats::as.formula, env = env)

  # if there is a label for the column use that as the name for the summary
  labs <- lapply(x, attr, "label")
  if (!all(sapply(labs, is.null))) {
    labs_i <- which(!sapply(labs, is.null))
    names(rtn)[labs_i] <- labs[labs_i]
  }

  rtn
}

#' @export
print.qwraps2_summary_table <- function(x, qable_args = list(), ...) {

  for (nm in names(qable_args)) {
    if (nm == "kable_args") {
      for (nm2 in names(qable_args$kable_args)) {
        attr(x, "qable_args")[["kable_args"]][[nm2]] <- qable_args$kable_args[[nm2]]
      }
    } else {
      attr(x, "qable_args")[[nm]] <- qable_args[[nm]]
    }
  }

  NextMethod(x, ...)

  invisible(x)
}

library(qwraps2)

e <- new.env()
stopifnot(identical(ll(e), character(0)))

e$fit <- lm(mpg ~ wt, mtcars)
e$fit2 <- lm(mpg ~ wt + am + vs, data = mtcars)
e$x <- rnorm(1e5)
e$y <- runif(1e4)
e$z <- with(e, x * y)
e$w <- sum(e$z)
e$vroom <- mtcars
e$foo <- function(x) {2 + x}

stopifnot(identical(rownames(ll(e)), as.character(1:nrow(ll(e)))))
stopifnot(identical(ll(e)$object, c("x", "z", "y", "fit2", "fit", "vroom", "foo", "w")))
stopifnot(identical(ll(e, order_by = "object")$object, c("fit", "fit2", "foo", "vroom", "w", "x", "y", "z")))
stopifnot(identical(ll(e, order_by = "object", decreasing = TRUE)$object, rev(c("fit", "fit2", "foo", "vroom", "w", "x", "y", "z"))))

#' @title Autocorrelation plot
#'
#' @description ggplot2 style autocorrelation plot
#'
#' @details qacf calls \code{\link[stats]{acf}} to generate a data set which is
#' then plotted via ggplot2.
#'
#' More details and examples for graphics within qwraps2 are in the
#' vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
#'
#' @param x object
#' @param conf_level confidence level for determining \sQuote{significant}
#' correlations
#' @param show_sig logical, highlight significant correlations
#' @param ... Other arguments passed to \code{\link[stats]{acf}}
#'
#' @return a ggplot.
#'
#' @seealso \code{\link[stats]{acf}}.
#'
#' @examples
#' # Generate a random data set
#' set.seed(42)
#' n <- 250
#' x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
#' x1[1] <- runif(1)
#' x2[1] <- runif(1)
#' x3[1] <- runif(1)
#' x4[1] <- runif(1)
#'
#' # white noise
#' Z_1 <- rnorm(n, 0, 1)
#' Z_2 <- rnorm(n, 0, 2)
#' Z_3 <- rnorm(n, 0, 5)
#'
#' for(i in 2:n)
#' {
#'   x1[i] <- x1[i-1] + Z_1[i] - Z_1[i-1] + x4[i-1] - x2[i-1]
#'   x2[i] <- x2[i-1] - 2 * Z_2[i] + Z_2[i-1] - x4[i-1]
#'   x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z_3[i] + Z_3[i-1]
#'   x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
#' }
#' testdf <- data.frame(x1, x2, x3, x4)
#'
#' # qacf plot for one variable
#' qacf(testdf$x1)
#' qacf(testdf$x1, show_sig = TRUE)
#'
#' # more than one variable
#' qacf(testdf)
#' qacf(testdf, show_sig = TRUE)
#'
#' @export
#' @rdname qacf
qacf <- function(x, conf_level = 1 - getOption("qwraps2_alpha", 0.05), show_sig = FALSE, ...) {
  UseMethod("qacf")
}

#' @export
qacf.default <- function(x, conf_level = 1 - getOption("qwraps2_alpha", 0.05), show_sig = FALSE, ...) {
  do.call(qacf.data.frame, list(x = data.frame(x), conf_level = conf_level, show_sig = show_sig, ...))
}

#' @export
#' @method qacf data.frame
qacf.data.frame <- function(x, conf_level = 1 - getOption("qwraps2_alpha", 0.05), show_sig = FALSE, ...) {
  acf_data <- stats::acf(x, plot = FALSE, ...)
  ciline <- stats::qnorm((1 - conf_level) / 2) / sqrt(acf_data$n.used)

  lags <- as.data.frame(acf_data$lag)
  acfs <- as.data.frame(acf_data$acf)
  lags <- utils::stack(lags)
  acfs <- utils::stack(acfs)
  names(lags) <- c("lag", "variable")
  names(acfs) <- c("value", "variable")
  acf_df <- cbind(lags, value = acfs[["value"]])

  acf_df$significant <- factor(abs(acf_df$value) > abs(ciline))

  g <-
    ggplot2::ggplot() +
    eval(substitute(ggplot2::aes(x = X, y = Y), list(X = as.name("lag"), Y = as.name("value")))) +
    ggplot2::geom_bar(stat = "identity", position = "identity") +
    ggplot2::ylab(acf_data$type)

  if (ncol(x) > 1) {
    facets <- rep(apply(expand.grid(acf_data$snames, acf_data$snames),
                        1, function(x) {if(x[1] == x[2]) x[1] else paste(x, collapse = " & ")}),
                  each = dim(acf_data$acf)[1])
    facets_levels <-
      apply(expand.grid(acf_data$snames, acf_data$snames)[, 2:1],
            1, function(x) {if(x[1] == x[2]) x[1] else paste(x, collapse = " & ")})

    acf_df$facets <- factor(facets, levels = facets_levels)

    g <- g + ggplot2::facet_wrap( ~ facets, scales = "free_x")
  }

  if(show_sig) {
    if (ncol(x) > 1) {
      g <- g +
        ggplot2::geom_hline(yintercept = ciline) +
        ggplot2::geom_hline(yintercept = -ciline)
    } else {
      g <- g + ggplot2::geom_hline(yintercept = -ciline)
    }
    g <- g + eval(substitute(ggplot2::aes(fill = S), list(S = as.name("significant"))))
  }

  g <- ggplot2::`%+%`(g, acf_df)
  g
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @title Trapezoid Rule Numeric Integration
#'
#' @description Compute the integral of y with respect to x via trapezoid rule.
#'
#' @param x,y numeric vectors of equal length
#'
#' @return a numeric value, the estimated integral
#'
#' @examples
#' xvec <- seq(-2 * pi, 3 * pi, length = 560)
#' foo  <- function(x) { sin(x) + x * cos(x) + 12 }
#' yvec <- foo(xvec)
#' plot(xvec, yvec, type = "l")
#'
#' integrate(f = foo, lower = -2 * pi, upper = 3 * pi)
#' traprule(xvec, yvec)
#'
#' @export
traprule <- function(x, y) {
    .Call('_qwraps2_traprule', PACKAGE = 'qwraps2', x, y)
}

library(qwraps2)

set_a <- c("A", "a", "E",      "I", "i", "O", "o", "U", "u", "E", "I")
set_b <- c("A", "a", "E", "e",      "i",      "o", "U", "u", "u", "a", "e")
set_c <- c("A", "e")
sd_ab <- set_diff(set_a, set_b)
sd_ba <- set_diff(set_b, set_a)

stopifnot(!is.null(attr(sd_ab, "class")))
stopifnot(!is.null(attr(sd_ba, "class")))
stopifnot(!is.null(attr(sd_ab, "yname")))
stopifnot(!is.null(attr(sd_ba, "yname")))
stopifnot(!is.null(attr(sd_ab, "xname")))
stopifnot(!is.null(attr(sd_ba, "xname")))

stopifnot(attr(sd_ab, "class") == "qwraps2_set_diff")
stopifnot(attr(sd_ba, "class") == "qwraps2_set_diff")

stopifnot(attr(sd_ab, "xname") == "set_a")
stopifnot(attr(sd_ba, "xname") == "set_b")

stopifnot(attr(sd_ab, "yname") == "set_b")
stopifnot(attr(sd_ba, "yname") == "set_a")

stopifnot(names(sd_ab) == c("all_values", "x_only", "y_only", "both", "equal"))
stopifnot(names(sd_ba) == c("all_values", "x_only", "y_only", "both", "equal"))

stopifnot(sort(sd_ab$all_values) == sort(sd_ba$all_values))
stopifnot(sd_ab$x_only == sd_ba$y_only)
stopifnot(sd_ab$y_only == sd_ba$x_only)
stopifnot(sd_ab$both == sd_ba$both)
stopifnot(sd_ab$equal == sd_ba$equal)
stopifnot(set_diff(set_b, set_b)$equal)

stopifnot(sd_ab$all_values == base::union(set_a, set_b))
stopifnot(sd_ab$x_only     == setdiff(set_a, set_b))
stopifnot(sd_ab$y_only     == setdiff(set_b, set_a))
stopifnot(sd_ab$both       == intersect(set_a, set_b))
stopifnot(sd_ab$both       == intersect(set_b, set_a))
stopifnot(sd_ab$equal      == setequal(set_a, set_b))
stopifnot(sd_ab$equal      == setequal(set_b, set_a))


# verify the print method returns the original object
sd_ab_printed <- print(sd_ab)
stopifnot(!is.null(sd_ab_printed))
stopifnot(identical(sd_ab_printed, sd_ab))

# verify the verbose argument controls printing of unique elements
sd_long <- set_diff(letters[1:15], letters[11:26])
default_print <- capture.output(print(sd_long))
verbose_print <- capture.output(print(sd_long, verbose = TRUE))
quiet_print <- capture.output(print(sd_ab, verbose = FALSE))

stopifnot(!any(grepl("unique elements", default_print, fixed = TRUE)))
stopifnot(any(grepl("unique elements", verbose_print, fixed = TRUE)))
stopifnot(!any(grepl("unique elements", quiet_print, fixed = TRUE)))

test <- tryCatch(print(sd_ab, verbose = NA), error = function(e) e)
stopifnot(inherits(test, "error"))
test <- tryCatch(print(sd_ab, verbose = c(TRUE, FALSE)), error = function(e) e)
stopifnot(inherits(test, "error"))

library(qwraps2)

# basic formatting
stopifnot(identical(median_iqr(mtcars$mpg), "19.20 (15.43, 22.80)"))

# warning given if show_n is not as expected
stopifnot(inherits(tryCatch(median_iqr(mtcars$mpg, show_n = TRUE), error = function(e) e), "error"))

# show n with no missing data
mpg <- mtcars$mpg
stopifnot(identical(median_iqr(mpg, show_n = "always"), paste0(nrow(mtcars), "; 19.20 (15.43, 22.80)")))
stopifnot(identical(median_iqr(mpg, show_n = "ifNA"), "19.20 (15.43, 22.80)"))
stopifnot(identical(median_iqr(mpg, show_n = "never"), "19.20 (15.43, 22.80)"))

# show n with missing data
mpg[c(3, 12)] <- NA_real_

stopifnot(identical(median_iqr(mpg, show_n = "always", na_rm = TRUE), paste0(nrow(mtcars) - 2, "; 19.20 (15.27, 22.48)")))
stopifnot(identical(median_iqr(mpg, show_n = "ifNA", na_rm = TRUE), "30; 19.20 (15.27, 22.48)"))
stopifnot(identical(median_iqr(mpg, show_n = "never", na_rm = TRUE), "19.20 (15.27, 22.48)"))

# non logical na_rm errors
x <- c(1, NA, 2:10)
stopifnot(inherits(tryCatch(median_iqr(x, na_rm = "Yes"), error = function(e) e, warning = function(w) w), "error"))
stopifnot(inherits(tryCatch(median_iqr(x, na.rm = "Yes"), error = function(e) e, warning = function(w) w), "warning"))
stopifnot(inherits(tryCatch(suppressWarnings(median_iqr(x, na.rm = "Yes")), error = function(e) e), "error"))

# missing value will result in an error
x <- c(1, NA, 2:10)
stopifnot(inherits(tryCatch(median_iqr(x), error = function(e) e, warning = function(w) w), "error"))

library(qwraps2)

# basic formatting
stopifnot(identical(n_perc(mtcars2$cyl == 6),  "7 (21.88\\%)"))
stopifnot(identical(n_perc0(mtcars2$cyl == 6), "7 (22)"))
stopifnot(identical(perc_n(mtcars2$cyl == 6), paste0("21.88\\% (n = ", nrow(mtcars2), ")")))

# errors with bad arguments
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl), error = function(e) e), "error"))
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl == 6, na_rm = "please"), error = function(e) e), "error"))
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl == 6, show_denom = "please"), error = function(e) e), "error"))
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl == 6, show_symbol = "please"), error = function(e) e), "error"))
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl == 6, markup = "please"), error = function(e) e), "error"))


# warning if na.rm used instead of na_rm
stopifnot(inherits(tryCatch(n_perc(mtcars2$cyl == 6, na.rm = TRUE), warning = function(w) w), "warning"))
stopifnot(inherits(tryCatch(perc_n(mtcars2$cyl == 6, na.rm = TRUE), warning = function(w) w), "warning"))
stopifnot(inherits(tryCatch(n_perc0(mtcars2$cyl == 6, na.rm = TRUE), warning = function(w) w), "warning"))

# with missing data
x <- mtcars2$cyl == 6
x[c(3, 13)] <- NA

stopifnot(identical(n_perc(x), "NA/30 ( NA\\%)"))
stopifnot(identical(n_perc0(x), "NA (NA)"))
stopifnot(identical(perc_n(x), " NA\\% (n = 30 non-missing)"))

stopifnot(identical(n_perc(x, na_rm = TRUE), "7/30 (23.33\\%)"))
stopifnot(identical(n_perc0(x, na_rm = TRUE), "7 (23)"))
stopifnot(identical(perc_n(x, na_rm = TRUE), "23.33\\% (n = 30 non-missing)"))

stopifnot(identical(n_perc(x, na_rm = TRUE, show_denom = "never"), "7 (23.33\\%)"))
stopifnot(identical(n_perc0(x, na_rm = TRUE, show_denom = "never"), "7 (23)"))
stopifnot(identical(perc_n(x, na_rm = TRUE, show_denom = "never"), "23.33\\% (n = 30 non-missing)"))

stopifnot(identical(n_perc(x, na_rm = TRUE, show_denom = "always"), "7/30 (23.33\\%)"))
stopifnot(identical(n_perc0(x, na_rm = TRUE, show_denom = "always"), "7/30 (23)"))
stopifnot(identical(perc_n(x, na_rm = TRUE, show_denom = "always"), "23.33\\% (n = 30 non-missing)"))

stopifnot(identical(n_perc(x, na_rm = TRUE, show_denom = "ifNA"), "7/30 (23.33\\%)"))
stopifnot(identical(n_perc0(x, na_rm = TRUE, show_denom = "ifNA"), "7/30 (23)"))
stopifnot(identical(perc_n(x, na_rm = TRUE, show_denom = "ifNA"), "23.33\\% (n = 30 non-missing)"))

# show denom with no missing data
x <- mtcars2$cyl == 6

stopifnot(identical(n_perc(mtcars2$cyl == 6, show_denom = "always"),  "7/32 (21.88\\%)"))
stopifnot(identical(n_perc0(mtcars2$cyl == 6, show_denom = "always"), "7/32 (22)"))
stopifnot(identical(perc_n(mtcars2$cyl == 6, show_denom = "always"), paste0("21.88\\% (n = ", nrow(mtcars2), " non-missing)")))

library(qwraps2)

stopifnot(identical(traprule(x = c(0, 0, 1, 1), y = c(0, 1, 1, 0)), 1.0))
stopifnot(identical(traprule(x = c(0, 0, 1, 1), y = c(0, 2, 1, 0)), 1.5))

two_to_the_x <- function(x) {
  2 ** x
}

x <- seq(-1, 3, by = 1)
stopifnot(identical(traprule(x, two_to_the_x(x)), 11.25))

x <- seq(-1, 3, length = 100)
stopifnot(identical(sprintf("%.16f", traprule(x, two_to_the_x(x))), "10.8209200182984446"))

# error if inputs are of unequal length
x <- tryCatch(traprule(c(1, 2, 3, 4), c(2, 3)), error = function(e) {e})
stopifnot(!is.null(x))
stopifnot(inherits(x, "error"))

# error if inputs are of length < 2
x <- tryCatch(traprule(c(1), c(2)), error = function(e) {e})
stopifnot(!is.null(x))
stopifnot(inherits(x, "error"))

library(qwraps2)

# simple ci
stopifnot(
  all.equal(
              unclass(mean_ci(mtcars$mpg))
            ,
              c(mean(mtcars$mpg),
                mean(mtcars$mpg) + qnorm(0.025)*sd(mtcars$mpg)/sqrt(nrow(mtcars)),
                mean(mtcars$mpg) + qnorm(0.975)*sd(mtcars$mpg)/sqrt(nrow(mtcars)))
            , check.attributes = FALSE
  )
)

# Printing nicely
out <- capture.output(print(mean_ci(mtcars$mpg)))
stopifnot(grepl("20\\.09 \\(18\\.00, 22\\.18\\)", out))

out <- capture.output(print(mean_ci(mtcars$mpg), show_level = TRUE))
stopifnot(grepl("20.09 \\(95% CI: 18\\.00, 22.18\\)", out))

out <- capture.output(print(mean_ci(mtcars$mpg, alpha = 0.01), show_level = TRUE))
stopifnot(grepl("20.09 \\(99% CI: 17\\.35, 22\\.83\\)", out))

# Compare to the ci that comes form t.test
stopifnot(
          all.equal(
            t.test(mtcars$mpg)$conf.int,
            mean_ci(mtcars$mpg, qdist = stats::qt, qdist.args = list(df = 31))[2:3],
            check.attributes = FALSE
          ))

# simple ci with missing data
x <- mtcars$mpg
x[c(12, 14)] <- NA

stopifnot(
          all.equal(unclass(mean_ci(x, na_rm = TRUE)),
                    c(mean(x, na.rm = TRUE),
                      mean(x, na.rm = TRUE) + qnorm(0.025)*sd(x, na.rm = TRUE)/sqrt(sum(!is.na(x))),
                      mean(x, na.rm = TRUE) + qnorm(0.975)*sd(x, na.rm = TRUE)/sqrt(sum(!is.na(x))))
                    , check.attributes = FALSE)
)

# warnings
stopifnot(inherits(tryCatch(mean_ci(mtcars$mpg, na.rm = TRUE), warning = function(w) w), "warning"))

# errors
stopifnot(inherits(tryCatch(mean_ci(mtcars$mpg, transform = "exp"), error = function(e) e), "error"))

#' @title Bland Altman Plots
#'
#' @description Construct and plot a Bland Altman plot in ggplot2.
#'
#' @details
#' Providing a \code{data.frame} with two columns, the function returns a ggplot
#' version of a Bland Altman plot with the specified confidence intervals.
#'
#' Two ways to call the plotting function.  If you submit a \code{data.frame}
#' \code{qblandaltman} then the data needed to produce the Bland Altman plot is
#' automatically generated by a call to \code{qblandaltman_build_data_frame}.
#' Alternatively, you may call \code{qblandaltman_build_data_frame} directly and
#' then call \code{qblandaltman}.  This might be helpful if you are putting
#' multiple Bland Altman plots together into one ggplot object.  See Examples.
#'
#' More details and examples for graphics within qwraps2 are in the
#' vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
#'
#' @param x a \code{data.frame} with two columns, or an object that can be
#' coerced to a data frame.  If a \code{data.frame} with more than two columns
#' is used only the first two columns will be used.
#' @param alpha (Defaults to 0.05) place (1 - alpha)*100% confidence levels to
#' place on the plot.
#' @param generate_data logical, defaults to TRUE.  If TRUE, then the call to
#' \code{qblandaltman_build_data_frame} is done automatically for you.  If
#' FALSE, then you should explicitly call \code{qblandaltman_build_data_frame}
#' before calling \code{qblandaltman}.
#'
#' @return a ggplot.  Minimal aesthetics have been used so that the user may
#' modify the graphic as desired with ease.
#'
#' @references
#' Altman, Douglas G., and J. Martin Bland. "Measurement in medicine: the analysis
#' of method comparison studies." The statistician (1983): 307-317.
#'
#' Bland, J. Martin, and Douglas G Altman. "Statistical methods for assessing
#' agreement between two methods of clinical measurement." The lancet 327, no. 8476
#' (1986): 307-310.
#'
#' @examples
#'
#' data(pefr)
#' pefr_m1 <-
#'   cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
#'         "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])
#'
#' # The Bland Altman plot plots the average value on the x-axis and the
#' # difference in the measurements on the y-axis:
#' qblandaltman(pefr_m1) +
#'   ggplot2::xlim(0, 800) +
#'   ggplot2::ylim(-100, 100) +
#'   ggplot2::xlab("Average of two meters") +
#'   ggplot2::ylab("Difference in the measurements")
#'
#' @export
#' @rdname qblandaltman
qblandaltman <- function(x, alpha = getOption("qwraps2_alpha", 0.05), generate_data = TRUE) {
  UseMethod("qblandaltman")
}

#' @export
qblandaltman.default <- function(x, alpha = getOption("qwraps2_alpha", 0.05), generate_data = TRUE) {
  qblandaltman(as.data.frame(x), alpha = alpha, generate_data = generate_data)
}

#' @export
qblandaltman.data.frame <- function(x, alpha = getOption("qwraps2_alpha", 0.05), generate_data = TRUE) {

  if (is.null(attr(x, "qwraps2_generated"))) {
    if (generate_data) {
      x <- qblandaltman_build_data_frame(x, alpha)
    }
  }

  ggplot2::ggplot(x) +
  eval(substitute(ggplot2::aes(x = X, y = Y), list(X = as.name('avg'), Y = as.name('diff')))) +
  ggplot2::geom_point() +
  ggplot2::geom_hline(mapping = eval(substitute(ggplot2::aes(yintercept = Y), list(Y = as.name('lcl')))), lty = 2) +
  ggplot2::geom_hline(mapping = eval(substitute(ggplot2::aes(yintercept = Y), list(Y = as.name('ucl')))), lty = 2) +
  ggplot2::geom_hline(mapping = eval(substitute(ggplot2::aes(yintercept = Y), list(Y = as.name('mean_diff')))), lty = 3)
}

#' @export
#' @rdname qblandaltman
qblandaltman_build_data_frame <- function(x, alpha = getOption("qwraps2_alpha", 0.05)) {
  rtn      <- data.frame(x1 = x[[1]], x2 = x[[2]])
  rtn$avg  <- (rtn$x1 + rtn$x2) / 2
  rtn$diff <- (rtn$x2 - rtn$x1)
  rtn$mean_diff <- mean(rtn$diff)
  rtn$sd_diff   <- stats::sd(rtn$diff)
  rtn$lcl       <- rtn$mean_diff + stats::qnorm(alpha / 2) * rtn$sd_diff
  rtn$ucl       <- rtn$mean_diff + stats::qnorm(1 - alpha / 2) * rtn$sd_diff

  attr(rtn, "qwraps2_generated") = TRUE

  return(rtn)
}

---
title: "qwraps2: Graphics"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Graphics}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
library(qwraps2)
packageVersion("qwraps2")
```


There are several graphics generated within qwraps2.  The naming convention
for the "quick" plots was inspired by the (deprecated)
`r  CRANpkg(ggplot2)  `
function
`r  backtick(qplot) %s% "."  `
The development, flexibility, and robustness of these functions vary.  Some
"tips and tricks" are provided.


# qacf: Autocorrelation Plots

Generate an example data set.

```{r}
set.seed(42)
n <- 250
x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
x1[1] <- runif(1)
x2[1] <- runif(1)
x3[1] <- runif(1)
x4[1] <- runif(1)

# white noise
Z.1 <- rnorm(n, 0, 1)
Z.2 <- rnorm(n, 0, 2)
Z.3 <- rnorm(n, 0, 5)

for(i in 2:n)
{
  x1[i] <- x1[i-1] + Z.1[i] - Z.1[i-1] + x4[i-1] - x2[i-1]
  x2[i] <- x2[i-1] - 2 * Z.2[i] + Z.2[i-1] - x4[i-1]
  x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z.3[i] + Z.3[i-1]
  x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
}
testdf <- data.frame(x1, x2, x3, x4)

# Base acf plot for one variable
acf(testdf$x1)

# qacf plot for one variable
qacf(testdf$x1)
qacf(testdf$x1, show_sig = TRUE)

```
```{r fig.width = 5, fig.height = 5}
# more than one variable
acf(testdf)
qacf(testdf)
qacf(testdf, show_sig = TRUE)
```


## Tips and tricks

The implementation of qacf is based on the use of
`r  backtick(stats::acf)  `
to produce the statistics needed for the plot.  If you want to get at the
data itself to build your own acf plot you can extract the data frame from
the qacf return:

```{r}
acf_plot_data <- qacf(testdf)$data
head(acf_plot_data)
```



# qblandaltman: Bland Altman Plot

Introduced in [@altman1983measurement] and [@bland1986statistical], the
qblandaltman method builds ggplot2 style Bland Altman plots.  For examples we
use the provided pefr data set which was transcribed from
[@bland1986statistical].  See
`r  backtick(vignette("qwraps2-data-sets", package = "qwraps2"))  `
For more details on that data set.

The following replicates the figures in [@bland1986statistical].

Using the first measurement only:

```{r}
pefr_m1 <-
  cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
        "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])
```


A standard x-y style plot and a correlation coefficient suggests that the two
meters provide reasonably similar results.

```{r}
cor(pefr_m1)

ggplot2::ggplot(data = as.data.frame(pefr_m1)) +
  ggplot2::aes(x = Large, y = Mini) +
  ggplot2::geom_point() +
  ggplot2::xlab("Large Meter") +
  ggplot2::ylab("Mini Meter") +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(0, 800) +
  ggplot2::geom_abline(slope = 1)
```


However, for many reasons, the above is misleading.  One simple note:
correlation is not a metric for agreement, i.e., perfect agreement would be
shown if all the data points fell on the line of equality whereas perfect
correlation occurs when the data points are simply co-linear.

The Bland Altman plot plots the average value on the x-axis and the
difference in the measurements on the y-axis:

```{r}
# default plot
qblandaltman(pefr_m1)

# modified plot
ggplot2::last_plot() +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(-100, 100) +
  ggplot2::xlab("Average of two meters") +
  ggplot2::ylab("Difference in the measurements")
```


There is no distinct relationship between the differences and the average,
but the difference in the measurements between the two meters was observed to
range between
`r  paste(range(apply(pefr_m1, 1, diff)), collapse = " and ")  `
liters per minute.  Such a discrepancy between the meters is not observable
from the simple x-y plot.

Reliability, or repeatability, of measurements can also be investigated with
a Bland Altman plot.

```{r}
pefr_mini <-
  cbind(m1 = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"],
        m2 = pefr[pefr$measurement == 2 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

qblandaltman(pefr_mini)
```




# qkmplot: Kaplan Meier Plots


```{r}
# create a survfit object
require(survival)
leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x, data = survival::aml)

# base R km plot
survival:::plot.survfit(leukemia.surv, conf.int = TRUE, lty = 2:3, col = 1:2)

```
```{r fig.width = 5}
# qkmplot
qkmplot(leukemia.surv, conf_int = TRUE)
```


The function
`r  backtick(qkmplot_bulid_data_frame)  `
can be used to generate a data.frame needed for building a KM plot.  This
could be helpful for creating bespoke plots.

```{r}
leukemia_km_data <- qkmplot_bulid_data_frame(leukemia.surv)
head(leukemia_km_data, 3)

```
```{r fig.width = 5}
qkmplot(leukemia_km_data)
```


Intercept only models are easy to plot too.

```{r fig.width = 5}
intonly_fit <- survival::survfit(survival::Surv(time, status) ~ 1, data = survival::aml)
survival:::plot.survfit(intonly_fit, conf.int = TRUE)
qkmplot(intonly_fit, conf_int = TRUE)
```



# qroc and qprc: Receiver Operating Curve and Precision Recall Curve

Starting in
`r  Rpkg(qwraps2)  `
version 0.6.0, the methods for building these graphics have been
fundamentally changed as part of a major refactor of the
`r  backtick(confusion_matrix)  `
method which replaces a lot of the code that
`r  backtick(qroc)  `
and
`r  backtick(qprc)  `
were built on.

For this work we will consider a couple models for categorizing email and
spam or not based on the Spambase [@spambase] data.  More details on this
data set can be found in the
`r  backtick(vignette('qwraps2-data-sets', package = "qwraps2"))  `

Start by defining a training and validation splits of the spambase data

```{r}
set.seed(42)
tidx <- runif(nrow(spambase)) <= 0.80
xidx <- which(names(spambase) != "spam")
yidx <- which(names(spambase) == "spam")
training_set   <- spambase[tidx, ]
validating_set <- spambase[!tidx, ]
```





Train a few models:

```{r}
logistic_model <-
  glm(
    spam ~ .
  , data = training_set
  , family = binomial()
  )

ridge_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 0
  )

lasso_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 1
  )
```


Generate the predicted values on the validation set:

```{r}
validating_set$logistic_model_prediction <-
  predict(
    logistic_model
  , newdata = validating_set
  , type = "response"
  )

validating_set$ridge_model_prediction <-
  as.numeric(
    predict(
      ridge_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

validating_set$lasso_model_prediction <-
  as.numeric(
    predict(
      lasso_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )
```


To build ROC and/or PRC plots start by building the confusion matrix for each
model.  The qwraps2 function
`r  backtick(confusion_matrix)  `
makes this easy.

```{r}
cm1 <- confusion_matrix(spam ~ logistic_model_prediction, data = validating_set)
cm2 <- confusion_matrix(spam ~ ridge_model_prediction, data = validating_set)
cm3 <- confusion_matrix(spam ~ lasso_model_prediction, data = validating_set)
```


The ROC and PRC plots are ggplot objects and can be modified as you would any
other ggplot object.

```{r}
qroc(cm1) + ggplot2::ggtitle("Logisitic Model")
qroc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qroc(cm3) + ggplot2::ggtitle("LASSO Regression Model")
```


Graphing all three curves in one image with AUROC in the legend:

```{r}
roc_plot_data <-
  rbind(
      cbind(Model = paste("Logisitic; AUROC =", frmt(cm1$auroc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUROC =",     frmt(cm2$auroc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUROC =",     frmt(cm3$auroc, 3)), cm3$cm_stats)
    )

qroc(roc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::theme(legend.position = "bottom")
```


Similar for PRC:

```{r}
qprc(cm1) + ggplot2::ggtitle("Logisitic Model")
qprc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qprc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

prc_plot_data <-
  rbind(
      cbind(Model = paste("Logisitic; AUPRC =", frmt(cm1$auprc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUPRC =",     frmt(cm2$auprc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUPRC =",     frmt(cm3$auprc, 3)), cm3$cm_stats)
    )

qprc(prc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::geom_hline(yintercept = cm1$prevalence) +
  ggplot2::theme(legend.position = "bottom")
```



# References

<div id="refs"></div>

# Session Info

```{r label = "sessioninfo"}
sessionInfo()
```

---
title: "qwraps2: Summary Table"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Summary Table}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---



```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
```



```{r}
set.seed(42)
library(qwraps2)
options(qwraps2_markup = "markdown")
```


The
`r  backtick(summary_table)  `
method appears to be the most popular and widely used feature of the
`r  CRANpkg(qwraps2)  `
package.  As such, this vignette is provided to give as much detail on the
use of the method, and the underlying
`r  backtick(qable)  `
method for quickly building well formatted summary tables.

# qable

`r  backtick(qable)  `
builds a formatted character matrix from inputs and then renders a table via
knitr::kable.  The primary objective of this function is to allow for easy
construction of row groups.

## kable vs qable

For a simple example we will use the following data set with a grouping
variable, subject id, and two variables, V2, and V3.  For simplicity, we will
order the data by group and id as well.

```{r label = 'build_example_data_for_qable'}
d <- data.frame(
       group = sample(size = 15, paste0("grp", 1:5), replace = TRUE)
     , id = sample(size = 15, x = LETTERS)
     , V2 = rnorm(15)
     , V3 = rep(c(1, 2, NA), times = 5)
     )
d <- d[order(d$group, d$id), ]
```


Making a simple table via kable:


```{r label = "kable1", results = "asis"}
knitr::kable(d, row.names = FALSE)
```


The group column is great for data analysis, but is not the best for human
readability.  This is where
`r  backtick(qable)  `
can be useful.  Start by building a named numeric column with the name being
the row group name and the value the number of rows.  For the _ordered_ data
this is a simple call to table:

```{r}
c(table(d$group))
```


If we pass that named vector to
`r  backtick(qable)  `
as the rgroup and with specify the id column as the row names we have the
same information but in format that is better for humans:


```{r label = "qable1", results = "asis"}
qable(  x = d[, c("V2", "V3")]
      , rgroup = c(table(d$group)) # row group
      , rnames = d$id              # row names
)
```


The return object from
`r  backtick(qable)  `
is a character matrix.  Also, when a data.frame is passed to
`r  backtick(qable)  `
it is coerced to a matrix before anything else, as such, any formatting of
numeric values or other strings should be done before calling
`r  backtick(qable) %s% "." `

To pass arguments to knitr::kable do so via the
`r  backtick(kable_args)  `
argument.

## Example: Regression Model Summary Table

We will build a summary table for a regression model with row groups for
conceptually similar predictors.


```{r results = "asis"}
model <-
  glm(spam ~
        word_freq_your + word_freq_conference + word_freq_business +
        char_freq_semicolon + char_freq_exclamation_point +
        capital_run_length_total + capital_run_length_longest
    , data = spambase
    , family = binomial()
  )

model_summary <-
  data.frame(
    parameter = names(coef(model))
  , odd_ratio = frmt(exp(coef(model)), digits = 3)
  , lcl       = frmt(exp(coef(model) + qnorm(0.025) * sqrt(diag(vcov(model)))), digits = 3)
  , ucl       = frmt(exp(coef(model) + qnorm(0.975) * sqrt(diag(vcov(model)))), digits = 3)
  , pval      = frmtp(summary(model)$coef[, 4])
  )

qable(model_summary[-1, c('odd_ratio', 'lcl', 'ucl', 'pval')]
      , rtitle = "Parameter"
      , rgroup = c("Word Frequency" = 3, "Character Frequency" = 2, "Capital Run Length" = 2)
      , rnames = c("Your", "Conference", "Business", ";", "!", "Total", "Longest")
      , kable_args = list(align = "lrrrr", caption = "Regression Model Summary")
      , cnames = c("Odds Ratio", "Lower Conf. Limit", "Upper Conf. Limit", "P-value")
      )
```





# summary_table

`r  backtick(summary_table)  `
was developed with the primary objective to build well formatted and easy to
read data summary tables.  Conceptually, the construction of these tables
start by building a "list-of-lists" of summaries and then generating these
summaries for specific groupings of the data set.

## Defining a Summary

We will use the
`r  backtick(mtcars2)  `
data set for these examples.  We'll start with
something very simple and build up to something bigger.

Let's report the min, max, and mean (sd) for continuous variables and n (%) for
categorical variables.  We will report mpg, displacement (disp), wt (weight),
and gear overall and by number of cylinders and transmission type.

The use of the
`r  backtick(summary_table)  `
use to define a summary, that is, a list-of-lists of formulas for summarizing
the data.frame.

The inner lists are named formulae defining the wanted
summary.  The names are important, as they are used to label row groups and row
names in the table.


```{r}
our_summary1 <-
  list("Miles Per Gallon" =
       list("min"       = ~ min(mpg),
            "max"       = ~ max(mpg),
            "mean (sd)" = ~ qwraps2::mean_sd(mpg)),
       "Displacement" =
       list("min"       = ~ min(disp),
            "median"    = ~ median(disp),
            "max"       = ~ max(disp),
            "mean (sd)" = ~ qwraps2::mean_sd(disp)),
       "Weight (1000 lbs)" =
       list("min"       = ~ min(wt),
            "max"       = ~ max(wt),
            "mean (sd)" = ~ qwraps2::mean_sd(wt)),
       "Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5))
       )
```


Building the table is done with a call to
`r  backtick(summary_table)  `
and rendered in Table \@ref(tab:mtcars_whole).


```{r label = "mtcars2_whole", results = "asis"}
whole <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , qable_args = list(kable_args = list(caption = "mtcars2 data summary"))
  )
whole
```


## Summarize by

Use the
`r  backtick(by)  `
argument to specify a grouping variable and generate the same summary as
above but for subsets of the data.  When the
`r  backtick(by)  `
column is a factor, the columns will be in the order of the levels of the
factor.  In comparison, the column order
is alphabetical if the variable is just a character.


```{r label = "mtcars2_by_cylf", results = "asis"}
by_cylf <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_factor"))
  )
by_cylf

```
```{r label = "mtcars2_by_cylc", results = "asis"}
by_cylc <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_character")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_character"))
  )
by_cylc
```


You are also able to generate summaries by multiple columns. For example, Table
\@ref(tab:mtcars2_by_cyl_transmission) reports the summary by the combination
of the number of cylinders and the type of transmission.

```{r label = "mtcars2_by_cyl_transmission", results = "asis"}
by_cyl_am <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor", "transmission")
  )
by_cyl_am
```


## cbind summary_table

It is common that I will want to have a summary table with the first column
reporting for the whole data sets and the additional columns for subsets of
the data set.  The returned objects from
`r  backtick(summary_table)  `
can be joined together via
`r  backtick(cbind)  `
assuming that the row groupings (summaries) are the same.

Note: the
`r  backtick(kable_args)  `
of the first item passed to
`r  backtick(cbind)  `
will be assigned to the resulting object (Table \@ref(tab:mtcars2_cbind)).
However, there is an easy way to modify the qable_args and kable_args via the
print method.


```{r label = "mtcars2_cbind", results = "asis"}
both <- cbind(whole, by_cylf)
both
```


If you want to update how a summary table is printed, you can do so by
calling the print method explicitly while passing a new set of
`r  backtick(qable_args) %s% ","  `
see Table \@ref(tab:updated_both).

```{r label = "updated_both", results = "asis"}

print(both,
      qable_args = list(
        rtitle = "ROW-TITLE",
        cnames = c("Col 0", "Col 1", "Col 2", "Col 3"),
        kable_args = list(
          align = "lcrcr",
          caption = "mtcars2 data summary - new caption"
        )
      ))
```



## Adding P-values to a Summary Table

There are many different ways to format data summary tables. Adding
p-values to a table is just one thing that can be done in more than one way.
For example, if a row group reports the counts and percentages for each level
of a categorical variable across multiple (column) groups, then I would argue
that the p-value resulting from a chi square test or a Fisher exact test
would be best placed on the line of the table labeling the row group.
However, say we reported the minimum, median, mean, and maximum with in a
row group for one variable.  The p-value from a t-test, or other meaningful
test for the difference in mean, I would suggest should be reported on the
line of the summary table for the mean, not the row group itself.

With so many possibilities I have reserved construction of a p-value column
to be ad hoc.  Perhaps an additional column wouldn't be used and the p-values
are edited into row group labels, for example.

If you want to add a p-value column, or any other column(s) to a
`r  backtick(qwraps2_summary_table)  `
object you can with some degree of ease.  Note that
`r  backtick(qwraps2_summary_table)  `
objects are just character matrices with additional attributes.

```{r}
str(both)
```


For this example, we will added p-values for testing the difference in the
mean between the three cylinder groups and the distribution of forward gears
by cylinder groups.

```{r}
# difference in means
mpvals <-
  sapply(
         list(mpg  = lm(mpg  ~ cyl_factor, data = mtcars2),
              disp = lm(disp ~ cyl_factor, data = mtcars2),
              wt   = lm(wt   ~ cyl_factor, data = mtcars2)),
         extract_fpvalue)

# Fisher test
fpval <- frmtp(fisher.test(table(mtcars2$gear, mtcars2$cyl_factor))$p.value)
```


In this case, adding the p-value column, is done by creating a empty column
and then writing in the needed p-value on the wanted rows.  This could be
within a row group (tests for means) or for a row group (Fisher test).

```{r}
both <- cbind(both, "P-value" = "")
both[grepl("mean \\(sd\\)", both[, 1]), "P-value"] <- mpvals
both[grepl("Forward Gears", both[, 1]), "P-value"] <- fpval
```




```{r label = "both_with_pvals", results = "asis"}
print(both, qable_args = list(kable_args = list(caption = "mtcars2 summary with p-values")))
```


Another option you might consider is to have the p-value in the row group
name.  Consider the following construction.  The p-values are added to the
names of the row groups when building the summary table.

```{r results = "asis"}
gear_summary <-
  list("Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5)),
       "Transmission" =
       list("Automatic" = ~ qwraps2::n_perc0(am == 0),
            "Manual"    = ~ qwraps2::n_perc0(am == 1))
       )

gear_summary <-
setNames(gear_summary,
         c(
         paste("Forward Gears: ", frmtp(fisher.test(xtabs( ~ gear + cyl_factor, data = mtcars2))$p.value)),
         paste("Transmission: ",  frmtp(fisher.test(xtabs( ~ am + cyl_factor, data = mtcars2))$p.value)))
         )

summary_table(mtcars2, gear_summary, by = "cyl_factor")
```


## rbind summary_table

There is a rbind method of summary tables.  This can be useful when building
a large a table in smaller sections would be advantageous.  For example, it
might be helpful to add p-values to a summary table with just one row group
and then rbind all the tables together for printing.  Consider that in the
above example for adding p-values we have made an assumption that the order
of the summary and the
`r  backtick(mpvals)  `
will be static. Remembering to make the
sequence changes in more than one location can be more difficult than we
would like to admit.  Writing code to be robust to such changes is
preferable.


```{r}
t_mpg  <- summary_table(mtcars2, summaries = our_summary1["Miles Per Gallon"], by = "cyl_factor")
t_disp <- summary_table(mtcars2, summaries = our_summary1["Displacement"], by = "cyl_factor")
t_wt   <- summary_table(mtcars2, summaries = our_summary1["Weight (1000 lbs)"], by = "cyl_factor")

t_mpg  <- cbind(t_mpg,  "pvalue" = "")
t_disp <- cbind(t_disp, "pvalue" = "")
t_wt   <- cbind(t_wt,   "pvalue" = "")

t_mpg[ grepl("mean", t_mpg[, 1]),  "pvalue"] <- "mpg-pvalue"
t_disp[grepl("mean", t_disp[, 1]), "pvalue"] <- "disp-pvalue"
t_wt[  grepl("mean", t_wt[, 1]),   "pvalue"] <- "wt-pvalue"
```


Calling rbind now will let us have the table in different sequences without
having to worry about the alignment of rows between different elements:

```{r}
rbind(t_mpg, t_disp, t_wt)
rbind(t_wt, t_disp, t_mpg)
```



## Using Variable Labels

Some data management paradigms will use attributes to keep a label associated
with a variable in a data.frame.  Notable examples are the
`r  CRANpkg(Hmisc)  `
and
`r  CRANpkg(sjPlot) %s% "."  `
If you associate a label with a variable in the data frame the that label
will be used when building a summary table.  This feature was suggested
https://github.com/dewittpe/qwraps2/issues/74 and implemented thusly:

```{r}
new_data_frame <-
  data.frame(age = c(18, 20, 24, 17, 43),
             edu = c(1, 3, 1, 5, 2),
             rt  = c(0.01, 0.04, 0.02, 0.10, 0.06))

# Set a label for the variables
attr(new_data_frame$age, "label") <- "Age in years"
attr(new_data_frame$rt,  "label") <- "Reaction time"

# mistakenly set the attribute to name instead of label
attr(new_data_frame$edu, "name") <- "Education"
```


When calling
`r  backtick(qsummary)  `
the provide labels for the age and rt variables will
be used.  Since the attribute "label" does not exist for the edu variable,
edu will be used in the output.

```{r}
qsummary(new_data_frame)
```


This behavior is also seen with the
`r  backtick(summary_table)  `
call.

```{r results = "asis"}
summary_table(new_data_frame)
```


## Alternative building of the summaries

The task of building the
`r  backtick(summaries)  `
list-of-lists can be tedious. The function
`r  backtick(qummaries)  `
is designed to make it easier.
`r  backtick(qummaries)  `
will use a set of predefined
functions to summarize numeric columns of a data.frame, a set of arguments
to pass to
`r  backtick(n_perc)  `
for categorical (character and factor) variables.

By default, calling
`r  backtick(summary_table)  `
will use the default summary metrics
defined by
`r  paste0(backtick(qsummary), ".")  `
The purpose of
`r  backtick(qsummary)  `
is to provide the same
summary for all numeric variables within a data.frame and a single style of
summary for categorical variables within the data.frame.  For example, the
default summary for a set of variables from the
`r  backtick(mtcars2)  `
data set is

```{r}
qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")])
```


That default summary is used for a table as follows:

```{r label="summary_table_mtcars2_default", results = "asis"}
summary_table(mtcars2[, c("mpg", "cyl_factor", "wt")])
```


Now, say we want to only report the minimum and maximum for each of the
numeric variables and for the categorical variables we want two show the
denominator for each category and for the percentage, to one digit with the
percent symbol in the table.
Note that when defining the list of numeric_summaries that the argument place
holder is the
`r  backtick("%s%", dequote = TRUE)  `
character.

```{r}
new_summary <-
  qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")],
           numeric_summaries = list("Minimum" = "~ min(%s)",
                                    "Maximum" = "~ max(%s)"),
           n_perc_args = list(digits = 1, show_symbol = TRUE, show_denom = "always"))
str(new_summary)
```


The resulting table is:

```{r results = "asis"}
summary_table(mtcars2, new_summary)
```



# Session Info


```{r}
print(sessionInfo(), local = FALSE)
```

---
title: "qwraps2: Data Sets"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{qwraps2: Data Sets}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r}
library(qwraps2)
packageVersion("qwraps2")
```


# mtcars2

The base R package datasets provides the mtcars data set.  The information in
mtcars is the fuel consumption and automobile characteristics of
`r  nrow(mtcars)  `
automobiles as reported in the March, April, June and July 1974 issues of
_Motor Trend_ magazine [@hocking1976biometrics].

That dataset is modified and extended to provide support for examples within
the qwraps2 package documentation.  This vignette documents the construction
of mtcars2.

# Construction of mtcars2

Starting with the original mtcars:

```{r}
mtcars2 <- mtcars
str(mtcars2)
```


The cyl column provides the number of cylinders for in the engine of the
automobiles.  We will use two additional versions of this information, one as
character column and one as a factor.  Please note that the order of the
factor levels is intentionally set to be non-sequential.  This will help to
illustrate the ordering or results when using a factor or a character vector
as a grouping variable.

```{r}
mtcars2$cyl_character <- paste(mtcars2$cyl, "cylinders")
mtcars2$cyl_factor    <- factor(mtcars2$cyl,
                                levels = c(6, 4, 8),
                                labels = paste( c(6, 4, 8), "cylinders"))
```


Create other factor variables.

```{r}
mtcars2$gear_factor <-
  factor(mtcars2$gear, levels = c(3, 4, 5), labels = paste(c(3, 4, 5), "forward gears"))
```


Engine configuration: the
`r  backtick(vs)  `
column is an integer vector for indicating
V-shaped or straight.  The constructed column engine is a factor the same
information as a labeled factor.

```{r}
mtcars2$engine <-
  factor(mtcars2$vs, levels = c(0, 1), labels = c("V-shaped", "straight"))
```


Transmission: the
`r  backtick(am)  `
column is an integer vector indicating if the transmission is automatic or
manual.  We construct a
`r  backtick(transmission)  `
column to provide the same information as a factor.

```{r}
mtcars2$transmission <-
  factor(mtcars2$am, levels = c(0, 1), labels = c("Automatic", "Manual"))
```


The rownames of the mtcars2 data set provide the make and model of the
automobiles.  Here we will create columns for make and model and then omit
the rownames.

```{r}
mtcars2$make  <- sub("^(\\w+)\\s(.+)", "\\1", rownames(mtcars2))
mtcars2$model <- sub("^(\\w+)\\s(.+)", "\\2", rownames(mtcars2))
rownames(mtcars2) <- NULL
```


To have some dates to use in examples we are going to add an mostly
arbitrary date column to mtcars2.  Given that the data came from the March
through July issues of _Motor Trend_ in 1974, we will create a
`r  backtick(test_date)  `
column starting in January 1974 forward with one to three tests per week
through May 1974.  This assumes the data is in chronological order of the data.

```{r}
set.seed(42)
mtcars2$test_date <-
  as.POSIXct("1974-01-03", tz = "GMT") +
  cumsum(sample(c(2, 3, 4, 7) * 3600 * 24, size = nrow(mtcars2), replace = TRUE))
```


Lastly we will order the columns of mtcars2 so similar columns are next to
each other.

```{r}
mtcars2 <-
  mtcars2[, c("make", "model", "mpg", "disp", "hp", "drat", "wt", "qsec",
              "cyl", "cyl_character", "cyl_factor",
              "vs", "engine",
              "am", "transmission",
              "gear", "gear_factor",
              "carb",
              "test_date")]
```


# Summary of mtcars2

mtcars2 is a data frame with
`r  nrow(mtcars2)  `
observations with
`r  ncol(mtcars2)  `
variables.  Some of the variables tell us the same information, but in
different formats.

```{r}
str(mtcars2)
```


| Element | Name          | Description                                                                        |
| ------: | :------------ | :---------------------------------------                                           |
| [,  1]  | make          | Vehicle Manufacturer                                                               |
| [,  2]  | model         | Vehicle model                                                                      |
| [,  3]  | mpg           | Miles/(US) gallon                                                                  |
| [,  4]  | disp          | Displacement (cu.in.)                                                              |
| [,  5]  | hp            | Gross horsepower                                                                   |
| [,  6]  | drat          | Rear axle ratio                                                                    |
| [,  7]  | wt            | Weight (1000 lbs)                                                                  |
| [,  8]  | qsec          | 1/4 mile time                                                                      |
| [,  9]  | cyl           | Number of cylinders                                                                |
| [, 10]  | cyl_character | Number of cylinders as a character string                                          |
| [, 11]  | cyl_factor    | Number of cylinders as a factor                                                    |
| [, 12]  | vs            | Engine (0 = V-shaped, 1 = straight)                                                |
| [, 13]  | engine        | same info as vs, but as a factor                                                   |
| [, 14]  | am            | Transmission (0 = automatic, 1 = manual)                                           |
| [, 15]  | transmission  | same info as am as a factor                                                        |
| [, 16]  | gear          | Number of forward gears                                                            |
| [, 17]  | gear_factor   | Number of forward gears as a factor                                                |
| [, 18]  | carb          | Number of carburetors                                                              |
| [, 19]  | test_date     | arbitrary date - created to approximate when the vehicle would have been assessed. |


# pefr

Peak expiratory flow rate (pefr) data is used for examples within the qwraps2
package.  The data has been transcribed from [@bland1986statistical].

> The sample comprised colleagues and family of J.M.B. chosen to give a wide
> range of PEFR but in no way representative of any defined population. Two
> measurements were made with a Wright peak flow meter and two with a mini
> Wright meter, in random order. All measurements were taken by J.M.B., using
> the same two instruments. (These data were collected to demonstrate the
> statistical method and provide no evidence on the comparability of these two
> instruments.) We did not repeat suspect readings and took a single reading as
> our measurement of PEFR. Only the first measurement by each method is used to
> illustrate the comparison of methods, the second measurements being used in
> the study of repeatability.

The units of measure for the pefr are liters per minute (L/min).


```{r}
# copied text from the manuscript
pefr_table <-
  read.delim(
             header = FALSE,
             text = "
1	494	490	512	525
2	395	397	430	415
3	516	512	520	508
4	434	401	428	444
5	476	470	500	500
6	557	611	600	625
7	413	415	364	460
8	442	431	380	390
9	650	638	658	642
10	433	429	445	432
11	417	420	432	420
12	656	633	626	605
13	267	275	260	227
14	478	492	477	467
15	178	165	259	268
16	423	372	350	370
17	427	421	451	443")
```


Build the data set

```{r}
pefr <-
  expand.grid(subject = 1:17,
              measurement = 1:2,
              meter   = c("Wright peak flow meter", "Mini Wright peak flow meter"),
              KEEP.OUT.ATTRS = FALSE,
              stringsAsFactors = FALSE)
pefr$pefr <- do.call(c, pefr_table[, 2:5])

head(pefr)
```


See
`r  backtick(vignette("qwraps2-graphics", package = "qwraps2"))  `
for examples using this data set, specifically in the construction and use of
Bland-Altman plots via
`r  backtick(qblandaltman) %s% "." `


# Spambase

Spambase [@spambase] is a useful data set for example needed a binary outcome
and several possible predictors.  The data set and documentation can be found
in this package in the directory on your machine at:

```{r}
system.file("spambase", package = "qwraps2")
```


The data set
`r  backtick(spambase)  `
was generated thusly:

```{r}
nms <-
  scan(system.file("spambase", "spambase.names", package = "qwraps2")
       , what = character()
       , skip = 33
       , sep = "\n"
       , quiet = TRUE
  )
nms <- sapply(strsplit(nms, split = ":"), getElement, 1)
nms <- c(nms, "spam")

# clean up char_freq names
nms <-
  nms |>
  sub(";", "semicolon",         x = _, fixed = TRUE) |>
  sub("(", "parenthesis",       x = _, fixed = TRUE) |>
  sub("[", "square_bracket",    x = _, fixed = TRUE) |>
  sub("!", "exclamation_point", x = _, fixed = TRUE) |>
  sub("$", "dollar_sign",       x = _, fixed = TRUE) |>
  sub("#", "pound",             x = _, fixed = TRUE)

spambase <- read.csv(
    file = system.file("spambase", "spambase.data", package = "qwraps2")
    , header = FALSE
    , col.names = nms
)
```


There are
`r  frmt(nrow(spambase))  `
rows of data with
`r  frmt(ncol(spambase) - 1L) `
predictors for the binary outcome
`r  backtick(spam)  `


```{r}
n_perc(spambase$spam) # count and percent of spam messages
```




# References

<div id="refs"></div>

# Session Info

```{r label = "sessioninfo"}
sessionInfo()
```

---
title: "qwraps2: Formatted Summary Statistics"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Formatted Summary Statistics}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
```



```{r}
set.seed(42)
library(qwraps2)
# define the markup language we are working in.
# options(qwraps2_markup = "latex") is also supported.
options(qwraps2_markup = "markdown")
```


# Introduction

It is common for a manuscript to require a data summary table.  The table might
include simple summary statistics for the whole sample and for subgroups.
There are several tools available to build such tables.  In my opinion, though,
most of those tools have nuances imposed by the creators/authors such that other
users need not only understand the tool, but also think like the authors.
I wrote this package to be as flexible and general as possible.  I hope you like
these tools and will be able to use them in your work.

This vignette presents the use of the
`r  paste0(backtick(summary_table), ",")  `
`r  paste0(backtick(qsummary), ",")  `
and
`r  backtick(qable)  `
functions for quickly building data summary tables.  We will be using summary
statistic functions,
`r  paste0(backtick(mean_sd), ",")  `
`r  paste0(backtick(median_iqr), ",")  `
`r  paste0(backtick(n_perc), ",")  `
and others, from
`r  CRANpkg(qwraps2)  `
as well.

## Prerequisites Example Data Set

```{r}
library(qwraps2)
```


We will use the data set
`r  backtick(mtcars2)  `
for the examples throughout this vignette
data set for examples throughout this vignette.
`r  backtick(mtcars2)  `
is a modified and extended version of the base R data set
`r  paste(backtick(mtcars), ".")  `
For details on the construction of the
`r  backtick(mtcars2)  `
data set please view the vignette:
`r  backtick(vignette('mtcars', package = "qwraps2"))  `

```{r}
data(mtcars2)
str(mtcars2)
```


# Review of Summary Statistic Functions and Formatting


## Means and Standard Deviations

`r  backtick(mean_sd)  `
returns the (arithmetic) mean and standard deviation for numeric
vector as a formatted character string. For example,
`r  backtick(mean_sd(mtcars2$mpg))  `
returns the formatted string
`r  paste0(mean_sd(mtcars2$mpg), ".")  `
There are other options for formatting character string:

```{r}
mean_sd(mtcars2$mpg)
mean_sd(mtcars2$mpg, denote_sd = "paren")
```


## Mean and Confidence intervals

If you need the mean and a confidence interval there is the function
`r  paste0(backtick(mean_ci), ".")  `
which returns a
`r  backtick(qwraps2_mean_ci)  `
object which is a
named vector with the mean, lower confidence limit, and the upper confidence
limit.   The printing method for
`r  backtick(qwraps2_mean_ci)  `
objects is a call to the
`r  backtick(frmtci)  `
function.  You an modify the formatting of printed result by adjusting
the arguments pasted to
`r  paste0(backtick(frmtci), ".")  `

```{r}
mci <- mean_ci(mtcars2$mpg)
str(mci)
mci
print(mci, show_level = TRUE)
```


## Median and Inner Quartile Range

Similar to the
`r  backtick(mean_sd)  `
function, the
`r  backtick(median_iqr)  `
returns the median and the inner quartile range (IQR) of a data vector.

```{r}
median_iqr(mtcars2$mpg)
```


## Count and Percentages

The
`r  backtick(n_perc)  `
function is the workhorse.
`r  backtick(n_perc0)  `
is also provided for ease of use in the same way that base R has
`r  backtick(paste)  `
and
`r  paste(backtick(paste0), ".")  `
`r  backtick(n_perc)  `
returns the n (%) with the percentage sign in the string,
`r  backtick(n_perc0)  `
omits the percentage sign from the string.  The latter is good for tables,
the former for in-line text.


```{r}
n_perc(mtcars2$cyl == 4)
n_perc0(mtcars2$cyl == 4)

n_perc(mtcars2$cyl_factor == 4)  # this returns 0 (0.00%)
n_perc(mtcars2$cyl_factor == "4 cylinders")
n_perc(mtcars2$cyl_factor == levels(mtcars2$cyl_factor)[2])

# The count and percentage of 4 or 6 cylinders vehicles in the data set is
n_perc(mtcars2$cyl %in% c(4, 6))
```


## Geometric Means and Standard Deviations

Let $\left\{x_1, x_2, x_3, \ldots, x_n \right\}$ be a sample of size $n$ with
$x_i > 0$ for all $i.$  Then the geometric mean, $\mu_g,$ and geometric standard
deviation are

$$
\begin{equation}
  \mu_g = \left( \prod_{i = 1}^{n} x_i \right)^{\frac{1}{n}} = b^{ \sum_{i =
  1}^{n} \log_{b} x_i },
\end{equation}
$$
and
$$
\begin{equation}
  \sigma_g = b ^ {
  \sqrt{
    \frac{\sum_{i = 1}^{n} \left( \log_{b} \frac{x_i}{\mu_g} \right)^2}{n}
  }
  }
\end{equation}
$$
or, for clarity,
$$
\begin{equation}
  \log_{b} \sigma_g =
  \sqrt{ \frac{\sum_{i = 1}^{n} \left( \log_{b} \frac{x_i}{\mu_g}
  \right)^2}{n}}
\end{equation}
$$

When looking for the geometric standard deviation in R, the simple
`r  backtick(exp(sd(log(x))))  `
is not exactly correct.  The geometric standard deviation
uses $n,$ the full sample size, in the denominator, where as
the
`r  backtick(sd)  `
and
`r  backtick(var)  `
functions in R use the denominator $n - 1.$  To get
the geometric standard deviation one should adjust the result by multiplying the
variance by $(n - 1) / n$ or the standard deviation by $\sqrt{(n - 1) / n}.$
See the example below.

```{r}
x <- runif(6, min = 4, max = 70)

# geometric mean
mu_g <- prod(x) ** (1 / length(x))
mu_g
exp(mean(log(x)))
1.2 ** mean(log(x, base = 1.2))

# geometric standard deviation
exp(sd(log(x)))  ## This is wrong

# these equations are correct
sigma_g <- exp(sqrt(sum(log(x / mu_g) ** 2) / length(x)))
sigma_g

exp(sqrt((length(x) - 1) / length(x)) * sd(log(x)))
```


The functions
`r  paste0(backtick(gmean), ",")  `
`r  paste0(backtick(gvar), ",")  `
and
`r  backtick(gsd)  `
provide the geometric mean, variance, and standard deviation for a numeric
vector.

```{r}
gmean(x)
all.equal(gmean(x), mu_g)

gvar(x)
all.equal(gvar(x), sigma_g^2)  # This is supposed to be FALSE
all.equal(gvar(x), exp(log(sigma_g)^2))

gsd(x)
all.equal(gsd(x), sigma_g)
```


`r  backtick(gmean_sd)  `
will provide a quick way for reporting the geometric mean and
geometric standard deviation in the same way that
`r  backtick(mean_sd)  `
does for the arithmetic mean and arithmetic standard deviation:

```{r}
gmean_sd(x)
```




# Building a Data Summary Table

The function
`r  backtick(summary_table)  `
appears to be the most widely used tool provided by the qwraps2 package.  As
such, that function has earned its own vignette.

```{r eval = FALSE}
vignette("qwraps2-summary-table")
```



# Session Info


```{r}
print(sessionInfo(), local = FALSE)
```

---
title: "qwraps2: Data Sets"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{qwraps2: Data Sets}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r}
library(qwraps2)
packageVersion("qwraps2")
```


# mtcars2

The base R package datasets provides the mtcars data set.  The information in
mtcars is the fuel consumption and automobile characteristics of
`r  nrow(mtcars)  `
automobiles as reported in the March, April, June and July 1974 issues of
_Motor Trend_ magazine [@hocking1976biometrics].

That dataset is modified and extended to provide support for examples within
the qwraps2 package documentation.  This vignette documents the construction
of mtcars2.

# Construction of mtcars2

Starting with the original mtcars:

```{r}
mtcars2 <- mtcars
str(mtcars2)
```


The cyl column provides the number of cylinders in the engine of the
automobiles.  We will use two additional versions of this information, one as
character column and one as a factor.  Please note that the order of the
factor levels is intentionally set to be non-sequential.  This will help to
illustrate the ordering of results when using a factor or a character vector
as a grouping variable.

```{r}
mtcars2$cyl_character <- paste(mtcars2$cyl, "cylinders")
mtcars2$cyl_factor    <- factor(mtcars2$cyl,
                                levels = c(6, 4, 8),
                                labels = paste( c(6, 4, 8), "cylinders"))
```


Create other factor variables.

```{r}
mtcars2$gear_factor <-
  factor(mtcars2$gear, levels = c(3, 4, 5), labels = paste(c(3, 4, 5), "forward gears"))
```


Engine configuration: the
`r  backtick(vs)  `
column is an integer vector indicating
V-shaped or straight.  The constructed column engine is a factor with the same
information as a labeled factor.

```{r}
mtcars2$engine <-
  factor(mtcars2$vs, levels = c(0, 1), labels = c("V-shaped", "straight"))
```


Transmission: the
`r  backtick(am)  `
column is an integer vector indicating if the transmission is automatic or
manual.  We construct a
`r  backtick(transmission)  `
column to provide the same information as a factor.

```{r}
mtcars2$transmission <-
  factor(mtcars2$am, levels = c(0, 1), labels = c("Automatic", "Manual"))
```


The rownames of the mtcars2 data set provide the make and model of the
automobiles.  Here we will create columns for make and model and then omit
the rownames.

```{r}
mtcars2$make  <- sub("^(\\w+)\\s(.+)", "\\1", rownames(mtcars2))
mtcars2$model <- sub("^(\\w+)\\s(.+)", "\\2", rownames(mtcars2))
rownames(mtcars2) <- NULL
```


To have some dates to use in examples we are going to add a mostly
arbitrary date column to mtcars2.  Given that the data came from the March
through July issues of _Motor Trend_ in 1974, we will create a
`r  backtick(test_date)  `
column starting in January 1974 forward with one to three tests per week
through May 1974.  This assumes the data is in chronological order.

```{r}
set.seed(42)
mtcars2$test_date <-
  as.POSIXct("1974-01-03", tz = "GMT") +
  cumsum(sample(c(2, 3, 4, 7) * 3600 * 24, size = nrow(mtcars2), replace = TRUE))
```


Lastly we will order the columns of mtcars2 so similar columns are next to
each other.

```{r}
mtcars2 <-
  mtcars2[, c("make", "model", "mpg", "disp", "hp", "drat", "wt", "qsec",
              "cyl", "cyl_character", "cyl_factor",
              "vs", "engine",
              "am", "transmission",
              "gear", "gear_factor",
              "carb",
              "test_date")]
```


# Summary of mtcars2

mtcars2 is a data frame with
`r  nrow(mtcars2)  `
observations with
`r  ncol(mtcars2)  `
variables.  Some of the variables tell us the same information, but in
different formats.

```{r}
str(mtcars2)
```


| Element | Name          | Description                                                                        |
| ------: | :------------ | :---------------------------------------                                           |
| [,  1]  | make          | Vehicle Manufacturer                                                               |
| [,  2]  | model         | Vehicle model                                                                      |
| [,  3]  | mpg           | Miles/(US) gallon                                                                  |
| [,  4]  | disp          | Displacement (cu.in.)                                                              |
| [,  5]  | hp            | Gross horsepower                                                                   |
| [,  6]  | drat          | Rear axle ratio                                                                    |
| [,  7]  | wt            | Weight (1000 lbs)                                                                  |
| [,  8]  | qsec          | 1/4 mile time                                                                      |
| [,  9]  | cyl           | Number of cylinders                                                                |
| [, 10]  | cyl_character | Number of cylinders as a character string                                          |
| [, 11]  | cyl_factor    | Number of cylinders as a factor                                                    |
| [, 12]  | vs            | Engine (0 = V-shaped, 1 = straight)                                                |
| [, 13]  | engine        | same info as vs, but as a factor                                                   |
| [, 14]  | am            | Transmission (0 = automatic, 1 = manual)                                           |
| [, 15]  | transmission  | same info as am as a factor                                                        |
| [, 16]  | gear          | Number of forward gears                                                            |
| [, 17]  | gear_factor   | Number of forward gears as a factor                                                |
| [, 18]  | carb          | Number of carburetors                                                              |
| [, 19]  | test_date     | arbitrary date - created to approximate when the vehicle would have been assessed. |


# pefr

Peak expiratory flow rate (pefr) data is used for examples within the qwraps2
package.  The data has been transcribed from [@bland1986statistical].

> The sample comprised colleagues and family of J.M.B. chosen to give a wide
> range of PEFR but in no way representative of any defined population. Two
> measurements were made with a Wright peak flow meter and two with a mini
> Wright meter, in random order. All measurements were taken by J.M.B., using
> the same two instruments. (These data were collected to demonstrate the
> statistical method and provide no evidence on the comparability of these two
> instruments.) We did not repeat suspect readings and took a single reading as
> our measurement of PEFR. Only the first measurement by each method is used to
> illustrate the comparison of methods, the second measurements being used in
> the study of repeatability.

The units of measure for the pefr are liters per minute (L/min).


```{r}
# copied text from the manuscript
pefr_table <-
  read.delim(
             header = FALSE,
             text = "
1	494	490	512	525
2	395	397	430	415
3	516	512	520	508
4	434	401	428	444
5	476	470	500	500
6	557	611	600	625
7	413	415	364	460
8	442	431	380	390
9	650	638	658	642
10	433	429	445	432
11	417	420	432	420
12	656	633	626	605
13	267	275	260	227
14	478	492	477	467
15	178	165	259	268
16	423	372	350	370
17	427	421	451	443")
```


Build the data set

```{r}
pefr <-
  expand.grid(subject = 1:17,
              measurement = 1:2,
              meter   = c("Wright peak flow meter", "Mini Wright peak flow meter"),
              KEEP.OUT.ATTRS = FALSE,
              stringsAsFactors = FALSE)
pefr$pefr <- do.call(c, pefr_table[, 2:5])

head(pefr)
```


See
`r  backtick(vignette("qwraps2-graphics", package = "qwraps2"))  `
for examples using this data set, specifically in the construction and use of
Bland-Altman plots via
`r  backtick(qblandaltman) %s% "." `


# Spambase

Spambase [@spambase] is a useful data set for examples needing a binary outcome
and several possible predictors.  The data set and documentation can be found
in this package in the directory on your machine at:

```{r}
system.file("spambase", package = "qwraps2")
```


The data set
`r  backtick(spambase)  `
was generated thusly:

```{r}
nms <-
  scan(system.file("spambase", "spambase.names", package = "qwraps2")
       , what = character()
       , skip = 33
       , sep = "\n"
       , quiet = TRUE
  )
nms <- sapply(strsplit(nms, split = ":"), getElement, 1)
nms <- c(nms, "spam")

# clean up char_freq names
nms <-
  nms |>
  sub(";", "semicolon",         x = _, fixed = TRUE) |>
  sub("(", "parenthesis",       x = _, fixed = TRUE) |>
  sub("[", "square_bracket",    x = _, fixed = TRUE) |>
  sub("!", "exclamation_point", x = _, fixed = TRUE) |>
  sub("$", "dollar_sign",       x = _, fixed = TRUE) |>
  sub("#", "pound",             x = _, fixed = TRUE)

spambase <- read.csv(
    file = system.file("spambase", "spambase.data", package = "qwraps2")
    , header = FALSE
    , col.names = nms
)
```


There are
`r  frmt(nrow(spambase))  `
rows of data with
`r  frmt(ncol(spambase) - 1L) `
predictors for the binary outcome
`r  backtick(spam)  `


```{r}
n_perc(spambase$spam) # count and percent of spam messages
```




# References

<div id="refs"></div>

# Session Info

```{r label = "sessioninfo"}
sessionInfo()
```

---
title: "qwraps2: Summary Table"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Summary Table}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---



```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
```



```{r}
set.seed(42)
library(qwraps2)
options(qwraps2_markup = "markdown")
```


The
`r  backtick(summary_table)  `
method appears to be the most popular and widely used feature of the
`r  CRANpkg(qwraps2)  `
package.  As such, this vignette is provided to give as much detail on the
use of the method, and the underlying
`r  backtick(qable)  `
method for quickly building well formatted summary tables.

# qable

`r  backtick(qable)  `
builds a formatted character matrix from inputs and then renders a table via
knitr::kable.  The primary objective of this function is to allow for easy
construction of row groups.

## kable vs qable

For a simple example we will use the following data set with a grouping
variable, subject id, and two variables, V2, and V3.  For simplicity, we will
order the data by group and id as well.

```{r label = 'build_example_data_for_qable'}
d <- data.frame(
       group = sample(size = 15, paste0("grp", 1:5), replace = TRUE)
     , id = sample(size = 15, x = LETTERS)
     , V2 = rnorm(15)
     , V3 = rep(c(1, 2, NA), times = 5)
     )
d <- d[order(d$group, d$id), ]
```


Making a simple table via kable:


```{r label = "kable1", results = "asis"}
knitr::kable(d, row.names = FALSE)
```


The group column is great for data analysis, but is not the best for human
readability.  This is where
`r  backtick(qable)  `
can be useful.  Start by building a named numeric column with the name being
the row group name and the value the number of rows.  For the _ordered_ data
this is a simple call to table:

```{r}
c(table(d$group))
```


If we pass that named vector to
`r  backtick(qable)  `
as the rgroup and with specify the id column as the row names we have the
same information but in format that is better for humans:


```{r label = "qable1", results = "asis"}
qable(  x = d[, c("V2", "V3")]
      , rgroup = c(table(d$group)) # row group
      , rnames = d$id              # row names
)
```


The return object from
`r  backtick(qable)  `
is a character matrix.  Also, when a data.frame is passed to
`r  backtick(qable)  `
it is coerced to a matrix before anything else, as such, any formatting of
numeric values or other strings should be done before calling
`r  backtick(qable) %s% "." `

To pass arguments to knitr::kable do so via the
`r  backtick(kable_args)  `
argument.

## Example: Regression Model Summary Table

We will build a summary table for a regression model with row groups for
conceptually similar predictors.


```{r results = "asis"}
model <-
  glm(spam ~
        word_freq_your + word_freq_conference + word_freq_business +
        char_freq_semicolon + char_freq_exclamation_point +
        capital_run_length_total + capital_run_length_longest
    , data = spambase
    , family = binomial()
  )

model_summary <-
  data.frame(
    parameter = names(coef(model))
  , odd_ratio = frmt(exp(coef(model)), digits = 3)
  , lcl       = frmt(exp(coef(model) + qnorm(0.025) * sqrt(diag(vcov(model)))), digits = 3)
  , ucl       = frmt(exp(coef(model) + qnorm(0.975) * sqrt(diag(vcov(model)))), digits = 3)
  , pval      = frmtp(summary(model)$coef[, 4])
  )

qable(model_summary[-1, c('odd_ratio', 'lcl', 'ucl', 'pval')]
      , rtitle = "Parameter"
      , rgroup = c("Word Frequency" = 3, "Character Frequency" = 2, "Capital Run Length" = 2)
      , rnames = c("Your", "Conference", "Business", ";", "!", "Total", "Longest")
      , kable_args = list(align = "lrrrr", caption = "Regression Model Summary")
      , cnames = c("Odds Ratio", "Lower Conf. Limit", "Upper Conf. Limit", "P-value")
      )
```





# summary_table

`r  backtick(summary_table)  `
was developed with the primary objective to build well formatted and easy to
read data summary tables.  Conceptually, the construction of these tables
start by building a "list-of-lists" of summaries and then generating these
summaries for specific groupings of the data set.

## Defining a Summary

We will use the
`r  backtick(mtcars2)  `
data set for these examples.  We'll start with
something very simple and build up to something bigger.

Let's report the min, max, and mean (sd) for continuous variables and n (%) for
categorical variables.  We will report mpg, displacement (disp), wt (weight),
and gear overall and by number of cylinders and transmission type.

The use of the
`r  backtick(summary_table)  `
use to define a summary, that is, a list-of-lists of formulas for summarizing
the data.frame.

The inner lists are named formulae defining the wanted
summary.  The names are important, as they are used to label row groups and row
names in the table.


```{r}
our_summary1 <-
  list("Miles Per Gallon" =
       list("min"       = ~ min(mpg),
            "max"       = ~ max(mpg),
            "mean (sd)" = ~ qwraps2::mean_sd(mpg)),
       "Displacement" =
       list("min"       = ~ min(disp),
            "median"    = ~ median(disp),
            "max"       = ~ max(disp),
            "mean (sd)" = ~ qwraps2::mean_sd(disp)),
       "Weight (1000 lbs)" =
       list("min"       = ~ min(wt),
            "max"       = ~ max(wt),
            "mean (sd)" = ~ qwraps2::mean_sd(wt)),
       "Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5))
       )
```


Building the table is done with a call to
`r  backtick(summary_table)  `
and rendered in Table \@ref(tab:mtcars_whole).


```{r label = "mtcars2_whole", results = "asis"}
whole <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , qable_args = list(kable_args = list(caption = "mtcars2 data summary"))
  )
whole
```


## Summarize by

Use the
`r  backtick(by)  `
argument to specify a grouping variable and generate the same summary as
above but for subsets of the data.  When the
`r  backtick(by)  `
column is a factor, the columns will be in the order of the levels of the
factor.  In comparison, the column order
is alphabetical if the variable is just a character.


```{r label = "mtcars2_by_cylf", results = "asis"}
by_cylf <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_factor"))
  )
by_cylf

```
```{r label = "mtcars2_by_cylc", results = "asis"}
by_cylc <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_character")
  , qable_args = list(rtitle = "Summary Statistics"
                      , kable_args = list(caption = "mtcars2 data summary by cyl_character"))
  )
by_cylc
```


You are also able to generate summaries by multiple columns. For example, Table
\@ref(tab:mtcars2_by_cyl_transmission) reports the summary by the combination
of the number of cylinders and the type of transmission.

```{r label = "mtcars2_by_cyl_transmission", results = "asis"}
by_cyl_am <-
  summary_table(
    x = mtcars2
  , summaries = our_summary1
  , by = c("cyl_factor", "transmission")
  )
by_cyl_am
```


## cbind summary_table

It is common that I will want to have a summary table with the first column
reporting for the whole data sets and the additional columns for subsets of
the data set.  The returned objects from
`r  backtick(summary_table)  `
can be joined together via
`r  backtick(cbind)  `
assuming that the row groupings (summaries) are the same.

Note: the
`r  backtick(kable_args)  `
of the first item passed to
`r  backtick(cbind)  `
will be assigned to the resulting object (Table \@ref(tab:mtcars2_cbind)).
However, there is an easy way to modify the qable_args and kable_args via the
print method.


```{r label = "mtcars2_cbind", results = "asis"}
both <- cbind(whole, by_cylf)
both
```


If you want to update how a summary table is printed, you can do so by
calling the print method explicitly while passing a new set of
`r  backtick(qable_args) %s% ","  `
see Table \@ref(tab:updated_both).

```{r label = "updated_both", results = "asis"}

print(both,
      qable_args = list(
        rtitle = "ROW-TITLE",
        cnames = c("Col 0", "Col 1", "Col 2", "Col 3"),
        kable_args = list(
          align = "lcrcr",
          caption = "mtcars2 data summary - new caption"
        )
      ))
```



## Adding P-values to a Summary Table

There are many different ways to format data summary tables. Adding
p-values to a table is just one thing that can be done in more than one way.
For example, if a row group reports the counts and percentages for each level
of a categorical variable across multiple (column) groups, then I would argue
that the p-value resulting from a chi square test or a Fisher exact test
would be best placed on the line of the table labeling the row group.
However, say we reported the minimum, median, mean, and maximum within a
row group for one variable.  The p-value from a t-test, or other meaningful
test for the difference in mean, I would suggest should be reported on the
line of the summary table for the mean, not the row group itself.

With so many possibilities I have reserved construction of a p-value column
to be ad hoc.  Perhaps an additional column wouldn't be used and the p-values
are edited into row group labels, for example.

If you want to add a p-value column, or any other column(s) to a
`r  backtick(qwraps2_summary_table)  `
object you can with some degree of ease.  Note that
`r  backtick(qwraps2_summary_table)  `
objects are just character matrices with additional attributes.

```{r}
str(both)
```


For this example, we will add p-values for testing the difference in the
mean between the three cylinder groups and the distribution of forward gears
by cylinder groups.

```{r}
# difference in means
mpvals <-
  sapply(
         list(mpg  = lm(mpg  ~ cyl_factor, data = mtcars2),
              disp = lm(disp ~ cyl_factor, data = mtcars2),
              wt   = lm(wt   ~ cyl_factor, data = mtcars2)),
         extract_fpvalue)

# Fisher test
fpval <- frmtp(fisher.test(table(mtcars2$gear, mtcars2$cyl_factor))$p.value)
```


In this case, adding the p-value column, is done by creating an empty column
and then writing in the needed p-value on the wanted rows.  This could be
within a row group (tests for means) or for a row group (Fisher test).

```{r}
both <- cbind(both, "P-value" = "")
both[grepl("mean \\(sd\\)", both[, 1]), "P-value"] <- mpvals
both[grepl("Forward Gears", both[, 1]), "P-value"] <- fpval
```




```{r label = "both_with_pvals", results = "asis"}
print(both, qable_args = list(kable_args = list(caption = "mtcars2 summary with p-values")))
```


Another option you might consider is to have the p-value in the row group
name.  Consider the following construction.  The p-values are added to the
names of the row groups when building the summary table.

```{r results = "asis"}
gear_summary <-
  list("Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5)),
       "Transmission" =
       list("Automatic" = ~ qwraps2::n_perc0(am == 0),
            "Manual"    = ~ qwraps2::n_perc0(am == 1))
       )

gear_summary <-
setNames(gear_summary,
         c(
         paste("Forward Gears: ", frmtp(fisher.test(xtabs( ~ gear + cyl_factor, data = mtcars2))$p.value)),
         paste("Transmission: ",  frmtp(fisher.test(xtabs( ~ am + cyl_factor, data = mtcars2))$p.value)))
         )

summary_table(mtcars2, gear_summary, by = "cyl_factor")
```


## rbind summary_table

There is a rbind method of summary tables.  This can be useful when building
a large table in smaller sections would be advantageous.  For example, it
might be helpful to add p-values to a summary table with just one row group
and then rbind all the tables together for printing.  Consider that in the
above example for adding p-values we have made an assumption that the order
of the summary and the
`r  backtick(mpvals)  `
will be static. Remembering to make the
sequence changes in more than one location can be more difficult than we
would like to admit.  Writing code to be robust to such changes is
preferable.


```{r}
t_mpg  <- summary_table(mtcars2, summaries = our_summary1["Miles Per Gallon"], by = "cyl_factor")
t_disp <- summary_table(mtcars2, summaries = our_summary1["Displacement"], by = "cyl_factor")
t_wt   <- summary_table(mtcars2, summaries = our_summary1["Weight (1000 lbs)"], by = "cyl_factor")

t_mpg  <- cbind(t_mpg,  "pvalue" = "")
t_disp <- cbind(t_disp, "pvalue" = "")
t_wt   <- cbind(t_wt,   "pvalue" = "")

t_mpg[ grepl("mean", t_mpg[, 1]),  "pvalue"] <- "mpg-pvalue"
t_disp[grepl("mean", t_disp[, 1]), "pvalue"] <- "disp-pvalue"
t_wt[  grepl("mean", t_wt[, 1]),   "pvalue"] <- "wt-pvalue"
```


Calling rbind now will let us have the table in different sequences without
having to worry about the alignment of rows between different elements:

```{r}
rbind(t_mpg, t_disp, t_wt)
rbind(t_wt, t_disp, t_mpg)
```



## Using Variable Labels

Some data management paradigms will use attributes to keep a label associated
with a variable in a data.frame.  Notable examples are the
`r  CRANpkg(Hmisc)  `
and
`r  CRANpkg(sjPlot) %s% "."  `
If you associate a label with a variable in the data frame, that label
will be used when building a summary table.  This feature was suggested
https://github.com/dewittpe/qwraps2/issues/74 and implemented thusly:

```{r}
new_data_frame <-
  data.frame(age = c(18, 20, 24, 17, 43),
             edu = c(1, 3, 1, 5, 2),
             rt  = c(0.01, 0.04, 0.02, 0.10, 0.06))

# Set a label for the variables
attr(new_data_frame$age, "label") <- "Age in years"
attr(new_data_frame$rt,  "label") <- "Reaction time"

# mistakenly set the attribute to name instead of label
attr(new_data_frame$edu, "name") <- "Education"
```


When calling
`r  backtick(qsummary)  `
the provided labels for the age and rt variables will
be used.  Since the attribute "label" does not exist for the edu variable,
edu will be used in the output.

```{r}
qsummary(new_data_frame)
```


This behavior is also seen with the
`r  backtick(summary_table)  `
call.

```{r results = "asis"}
summary_table(new_data_frame)
```


## Alternative building of the summaries

The task of building the
`r  backtick(summaries)  `
list-of-lists can be tedious. The function
`r  backtick(qsummary)  `
is designed to make it easier.
`r  backtick(qsummary)  `
will use a set of predefined
functions to summarize numeric columns of a data.frame, a set of arguments
to pass to
`r  backtick(n_perc)  `
for categorical (character and factor) variables.

By default, calling
`r  backtick(summary_table)  `
will use the default summary metrics
defined by
`r  paste0(backtick(qsummary), ".")  `
The purpose of
`r  backtick(qsummary)  `
is to provide the same
summary for all numeric variables within a data.frame and a single style of
summary for categorical variables within the data.frame.  For example, the
default summary for a set of variables from the
`r  backtick(mtcars2)  `
data set is

```{r}
qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")])
```


That default summary is used for a table as follows:

```{r label="summary_table_mtcars2_default", results = "asis"}
summary_table(mtcars2[, c("mpg", "cyl_factor", "wt")])
```


Now, say we want to only report the minimum and maximum for each of the
numeric variables and for the categorical variables we want to show the
denominator for each category and for the percentage, to one digit with the
percent symbol in the table.
Note that when defining the list of numeric_summaries that the argument
placeholder is the
`r  backtick("%s%", dequote = TRUE)  `
character.

```{r}
new_summary <-
  qsummary(mtcars2[, c("mpg", "cyl_factor", "wt")],
           numeric_summaries = list("Minimum" = "~ min(%s)",
                                    "Maximum" = "~ max(%s)"),
           n_perc_args = list(digits = 1, show_symbol = TRUE, show_denom = "always"))
str(new_summary)
```


The resulting table is:

```{r results = "asis"}
summary_table(mtcars2, new_summary)
```



# Session Info


```{r}
print(sessionInfo(), local = FALSE)
```

---
title: "qwraps2: Formatted Summary Statistics"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Formatted Summary Statistics}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
```



```{r}
set.seed(42)
library(qwraps2)
# define the markup language we are working in.
# options(qwraps2_markup = "latex") is also supported.
options(qwraps2_markup = "markdown")
```


# Introduction

It is common for a manuscript to require a data summary table.  The table might
include simple summary statistics for the whole sample and for subgroups.
There are several tools available to build such tables.  In my opinion, though,
most of those tools have nuances imposed by the creators/authors such that other
users need not only understand the tool, but also think like the authors.
I wrote this package to be as flexible and general as possible.  I hope you like
these tools and will be able to use them in your work.

This vignette presents the use of the
`r  paste0(backtick(summary_table), ",")  `
`r  paste0(backtick(qsummary), ",")  `
and
`r  backtick(qable)  `
functions for quickly building data summary tables.  We will be using summary
statistic functions,
`r  paste0(backtick(mean_sd), ",")  `
`r  paste0(backtick(median_iqr), ",")  `
`r  paste0(backtick(n_perc), ",")  `
and others, from
`r  CRANpkg(qwraps2)  `
as well.

## Prerequisites Example Data Set

```{r}
library(qwraps2)
```


We will use the data set
`r  backtick(mtcars2)  `
for the examples throughout this vignette. It is a modified and extended
version of the base R data set
`r  paste(backtick(mtcars), ".")  `
For details on the construction of the
`r  backtick(mtcars2)  `
data set please view the vignette:
`r  backtick(vignette('qwraps2-data-sets', package = "qwraps2"))  `

```{r}
data(mtcars2)
str(mtcars2)
```


# Review of Summary Statistic Functions and Formatting


## Means and Standard Deviations

`r  backtick(mean_sd)  `
returns the (arithmetic) mean and standard deviation for numeric
vector as a formatted character string. For example,
`r  backtick(mean_sd(mtcars2$mpg))  `
returns the formatted string
`r  paste0(mean_sd(mtcars2$mpg), ".")  `
There are other options for formatting the character string:

```{r}
mean_sd(mtcars2$mpg)
mean_sd(mtcars2$mpg, denote_sd = "paren")
```


## Mean and Confidence intervals

If you need the mean and a confidence interval there is the function
`r  paste0(backtick(mean_ci), ".")  `
which returns a
`r  backtick(qwraps2_mean_ci)  `
object which is a
named vector with the mean, lower confidence limit, and the upper confidence
limit.   The printing method for
`r  backtick(qwraps2_mean_ci)  `
objects is a call to the
`r  backtick(frmtci)  `
function.  You can modify the formatting of the printed result by adjusting
the arguments passed to
`r  paste0(backtick(frmtci), ".")  `

```{r}
mci <- mean_ci(mtcars2$mpg)
str(mci)
mci
print(mci, show_level = TRUE)
```


## Median and Inner Quartile Range

Similar to the
`r  backtick(mean_sd)  `
function, the
`r  backtick(median_iqr)  `
returns the median and the inner quartile range (IQR) of a data vector.

```{r}
median_iqr(mtcars2$mpg)
```


## Count and Percentages

The
`r  backtick(n_perc)  `
function is the workhorse.
`r  backtick(n_perc0)  `
is also provided for ease of use in the same way that base R has
`r  backtick(paste)  `
and
`r  paste(backtick(paste0), ".")  `
`r  backtick(n_perc)  `
returns the n (%) with the percentage sign in the string,
`r  backtick(n_perc0)  `
omits the percentage sign from the string.  The latter is good for tables,
the former for in-line text.


```{r}
n_perc(mtcars2$cyl == 4)
n_perc0(mtcars2$cyl == 4)

n_perc(mtcars2$cyl_factor == 4)  # this returns 0 (0.00%)
n_perc(mtcars2$cyl_factor == "4 cylinders")
n_perc(mtcars2$cyl_factor == levels(mtcars2$cyl_factor)[2])

# The count and percentage of 4 or 6 cylinder vehicles in the data set is
n_perc(mtcars2$cyl %in% c(4, 6))
```


## Geometric Means and Standard Deviations

Let $\left\{x_1, x_2, x_3, \ldots, x_n \right\}$ be a sample of size $n$ with
$x_i > 0$ for all $i.$  Then the geometric mean, $\mu_g,$ and geometric standard
deviation are

$$
\begin{equation}
  \mu_g = \left( \prod_{i = 1}^{n} x_i \right)^{\frac{1}{n}} =
  b^{ \frac{1}{n} \sum_{i = 1}^{n} \log_{b} x_i },
\end{equation}
$$
and
$$
\begin{equation}
  \sigma_g = b ^ {
  \sqrt{
    \frac{\sum_{i = 1}^{n} \left( \log_{b} \frac{x_i}{\mu_g} \right)^2}{n}
  }
  }
\end{equation}
$$
or, for clarity,
$$
\begin{equation}
  \log_{b} \sigma_g =
  \sqrt{ \frac{\sum_{i = 1}^{n} \left( \log_{b} \frac{x_i}{\mu_g}
  \right)^2}{n}}
\end{equation}
$$

When looking for the geometric standard deviation in R, the simple
`r  backtick(exp(sd(log(x))))  `
does not match the population-denominator definition used here.  The
geometric standard deviation above uses $n,$ the full sample size, in the
denominator, whereas
the
`r  backtick(sd)  `
and
`r  backtick(var)  `
functions in R use the denominator $n - 1.$  To get
the geometric standard deviation one should adjust the result by multiplying the
variance by $(n - 1) / n$ or the standard deviation by $\sqrt{(n - 1) / n}.$
See the example below.

```{r}
x <- runif(6, min = 4, max = 70)

# geometric mean
mu_g <- prod(x) ** (1 / length(x))
mu_g
exp(mean(log(x)))
1.2 ** mean(log(x, base = 1.2))

# geometric standard deviation
exp(sd(log(x)))  ## Uses the sample denominator, n - 1

# these equations are correct
sigma_g <- exp(sqrt(sum(log(x / mu_g) ** 2) / length(x)))
sigma_g

exp(sqrt((length(x) - 1) / length(x)) * sd(log(x)))

# geometric variance
geometric_variance <- exp(log(sigma_g) ** 2)
geometric_variance
```


The functions
`r  paste0(backtick(gmean), ",")  `
`r  paste0(backtick(gvar), ",")  `
and
`r  backtick(gsd)  `
provide the geometric mean, variance, and standard deviation for a numeric
vector.  Here, geometric variance is defined as $\exp((\log \sigma_g)^2),$
not as $\sigma_g^2.$

```{r}
gmean(x)
all.equal(gmean(x), mu_g)

gvar(x)
all.equal(gvar(x), sigma_g^2)       # This is supposed to be FALSE
all.equal(gvar(x), geometric_variance)

gsd(x)
all.equal(gsd(x), sigma_g)
```


`r  backtick(gmean_sd)  `
will provide a quick way for reporting the geometric mean and
geometric standard deviation in the same way that
`r  backtick(mean_sd)  `
does for the arithmetic mean and arithmetic standard deviation:

```{r}
gmean_sd(x)
```




# Building a Data Summary Table

The function
`r  backtick(summary_table)  `
appears to be the most widely used tool provided by the qwraps2 package.  As
such, that function has earned its own vignette.

```{r eval = FALSE}
vignette("qwraps2-summary-table")
```



# Session Info


```{r}
print(sessionInfo(), local = FALSE)
```

---
title: "qwraps2: Graphics"
author: "Peter E. DeWitt"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: true
vignette: >
 %\VignetteIndexEntry{qwraps2: Graphics}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---


```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)
library(qwraps2)
packageVersion("qwraps2")
has_survival <- requireNamespace("survival", quietly = TRUE)
has_glmnet <- requireNamespace("glmnet", quietly = TRUE)
```


There are several graphics generated within qwraps2.  The naming convention
for the "quick" plots was inspired by the (deprecated)
`r  CRANpkg(ggplot2)  `
function
`r  backtick(qplot) %s% "."  `
The development, flexibility, and robustness of these functions vary.  Some
"tips and tricks" are provided.


# qacf: Autocorrelation Plots

Generate an example data set.

```{r}
set.seed(42)
n <- 250
x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
x1[1] <- runif(1)
x2[1] <- runif(1)
x3[1] <- runif(1)
x4[1] <- runif(1)

# white noise
Z.1 <- rnorm(n, 0, 1)
Z.2 <- rnorm(n, 0, 2)
Z.3 <- rnorm(n, 0, 5)

for(i in 2:n)
{
  x1[i] <- x1[i-1] + Z.1[i] - Z.1[i-1] + x4[i-1] - x2[i-1]
  x2[i] <- x2[i-1] - 2 * Z.2[i] + Z.2[i-1] - x4[i-1]
  x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z.3[i] + Z.3[i-1]
  x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
}
testdf <- data.frame(x1, x2, x3, x4)

# Base acf plot for one variable
acf(testdf$x1)

# qacf plot for one variable
qacf(testdf$x1)
qacf(testdf$x1, show_sig = TRUE)

```
```{r fig.width = 5, fig.height = 5}
# more than one variable
acf(testdf)
qacf(testdf)
qacf(testdf, show_sig = TRUE)
```


## Tips and tricks

The implementation of qacf is based on the use of
`r  backtick(stats::acf)  `
to produce the statistics needed for the plot.  If you want to get at the
data itself to build your own acf plot you can extract the data frame from
the qacf return:

```{r}
acf_plot_data <- qacf(testdf)$data
head(acf_plot_data)
```



# qblandaltman: Bland Altman Plot

Introduced in [@altman1983measurement] and [@bland1986statistical], the
qblandaltman method builds ggplot2 style Bland Altman plots.  For examples we
use the provided pefr data set which was transcribed from
[@bland1986statistical].  See
`r  backtick(vignette("qwraps2-data-sets", package = "qwraps2"))  `
For more details on that data set.

The following replicates the figures in [@bland1986statistical].

Using the first measurement only:

```{r}
pefr_m1 <-
  cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
        "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])
```


A standard x-y style plot and a correlation coefficient suggests that the two
meters provide reasonably similar results.

```{r}
cor(pefr_m1)

ggplot2::ggplot(data = as.data.frame(pefr_m1)) +
  ggplot2::aes(x = Large, y = Mini) +
  ggplot2::geom_point() +
  ggplot2::xlab("Large Meter") +
  ggplot2::ylab("Mini Meter") +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(0, 800) +
  ggplot2::geom_abline(slope = 1)
```


However, for many reasons, the above is misleading.  One simple note:
correlation is not a metric for agreement, i.e., perfect agreement would be
shown if all the data points fell on the line of equality whereas perfect
correlation occurs when the data points are simply co-linear.

The Bland Altman plot plots the average value on the x-axis and the
difference in the measurements on the y-axis:

```{r}
# default plot
qblandaltman(pefr_m1)

# modified plot
ggplot2::last_plot() +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(-100, 100) +
  ggplot2::xlab("Average of two meters") +
  ggplot2::ylab("Difference in the measurements")
```


There is no distinct relationship between the differences and the average,
but the difference in the measurements between the two meters was observed to
range between
`r  paste(range(apply(pefr_m1, 1, diff)), collapse = " and ")  `
liters per minute.  Such a discrepancy between the meters is not observable
from the simple x-y plot.

Reliability, or repeatability, of measurements can also be investigated with
a Bland Altman plot.

```{r}
pefr_mini <-
  cbind(m1 = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"],
        m2 = pefr[pefr$measurement == 2 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

qblandaltman(pefr_mini)
```




# qkmplot: Kaplan Meier Plots


```{r eval = has_survival, fig.width = 5}
# create a survfit object
leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x, data = survival::aml)

# base R km plot
plot(leukemia.surv, conf.int = TRUE, lty = 2:3, col = 1:2)

# qkmplot
qkmplot(leukemia.surv, conf_int = TRUE)
```


The function
`r  backtick(qkmplot_build_data_frame)  `
can be used to generate a data.frame needed for building a KM plot.  This
could be helpful for creating bespoke plots.

```{r eval = has_survival, fig.width = 5}
leukemia_km_data <- qkmplot_build_data_frame(leukemia.surv)
head(leukemia_km_data, 3)
qkmplot(leukemia_km_data)
```


Intercept only models are easy to plot too.

```{r eval = has_survival, fig.width = 5}
intonly_fit <- survival::survfit(survival::Surv(time, status) ~ 1, data = survival::aml)
plot(intonly_fit, conf.int = TRUE)
qkmplot(intonly_fit, conf_int = TRUE)
```



# qroc and qprc: Receiver Operating Characteristic Curve and Precision-Recall Curve

Starting in
`r  Rpkg(qwraps2)  `
version 0.6.0, the methods for building these graphics have been
fundamentally changed as part of a major refactor of the
`r  backtick(confusion_matrix)  `
method which replaces a lot of the code that
`r  backtick(qroc)  `
and
`r  backtick(qprc)  `
were built on.

For this work we will consider a couple models for categorizing email as
spam or not spam based on the Spambase [@spambase] data.  More details on this
data set can be found in the
`r  backtick(vignette('qwraps2-data-sets', package = "qwraps2"))  `

Start by defining training and validation splits of the spambase data

```{r}
set.seed(42)
tidx <- runif(nrow(spambase)) <= 0.80
xidx <- which(names(spambase) != "spam")
yidx <- which(names(spambase) == "spam")
training_set   <- spambase[tidx, ]
validating_set <- spambase[!tidx, ]
```





Train a few models.  Start with base R glm.

```{r}
logistic_model <-
  glm(
    spam ~ .
  , data = training_set
  , family = binomial()
  )
```


Use glmnet to build a ridge and a lasso model

```{r eval = has_glmnet}
ridge_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 0
  )

lasso_model <-
  glmnet::cv.glmnet(
    y = training_set[, yidx]
  , x = as.matrix(training_set[, xidx])
  , family = binomial()
  , alpha = 1
  )
```


Generate the predicted values on the validation set:

```{r}
validating_set$logistic_model_prediction <-
  predict(
    logistic_model
  , newdata = validating_set
  , type = "response"
  )
```




```{r eval = has_glmnet}
validating_set$ridge_model_prediction <-
  as.numeric(
    predict(
      ridge_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )

validating_set$lasso_model_prediction <-
  as.numeric(
    predict(
      lasso_model
    , newx = as.matrix(validating_set[, xidx])
    , type = "response"
    , s = "lambda.1se"
    )
  )
```


To build ROC and/or PRC plots start by building the confusion matrix for each
model.  The qwraps2 function
`r  backtick(confusion_matrix)  `
makes this easy.

```{r}
cm1 <- confusion_matrix(spam ~ logistic_model_prediction, data = validating_set)
```



```{r eval = has_glmnet}
cm2 <- confusion_matrix(spam ~ ridge_model_prediction, data = validating_set)
cm3 <- confusion_matrix(spam ~ lasso_model_prediction, data = validating_set)
```


The ROC and PRC plots are ggplot objects and can be modified as you would any
other ggplot object.

```{r}
qroc(cm1) + ggplot2::ggtitle("Logistic Model")
```



```{r eval = has_glmnet}
qroc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qroc(cm3) + ggplot2::ggtitle("LASSO Regression Model")
```


Graphing all three curves in one image with AUROC in the legend:

```{r eval = has_glmnet}
roc_plot_data <-
  rbind(
      cbind(Model = paste("Logistic; AUROC =",  frmt(cm1$auroc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUROC =",     frmt(cm2$auroc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUROC =",     frmt(cm3$auroc, 3)), cm3$cm_stats)
    )

qroc(roc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::theme(legend.position = "bottom")
```


Similar for PRC:

```{r}
qprc(cm1) + ggplot2::ggtitle("Logistic Model")
```



```{r eval = has_glmnet}
qprc(cm2) + ggplot2::ggtitle("Ridge Regression Model")
qprc(cm3) + ggplot2::ggtitle("LASSO Regression Model")

prc_plot_data <-
  rbind(
      cbind(Model = paste("Logistic; AUPRC =", frmt(cm1$auprc, 3)), cm1$cm_stats)
    , cbind(Model = paste("Ridge; AUPRC =",     frmt(cm2$auprc, 3)), cm2$cm_stats)
    , cbind(Model = paste("LASSO; AUPRC =",     frmt(cm3$auprc, 3)), cm3$cm_stats)
    )

qprc(prc_plot_data) +
  ggplot2::aes(color = Model) +
  ggplot2::geom_hline(yintercept = cm1$prevalence) +
  ggplot2::theme(legend.position = "bottom")
```



# References

<div id="refs"></div>

# Session Info

```{r label = "sessioninfo"}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{qroc_build_data_frame}
\alias{qroc_build_data_frame}
\alias{qroc_build_data_frame.default}
\alias{qroc_build_data_frame.glm}
\alias{qprc_build_data_frame}
\title{Deprecated qroc and qprc building of data frames:}
\usage{
qroc_build_data_frame(fit, truth = NULL, n_threshold = 200, ...)

\method{qroc_build_data_frame}{default}(fit, truth = NULL, n_threshold = 200, ...)

\method{qroc_build_data_frame}{glm}(fit, truth = NULL, n_threshold = 200, ...)

qprc_build_data_frame(fit, n_threshold = 200, ...)
}
\arguments{
\item{fit}{a \code{glm} fit with \code{family = binomial()}, or predicted
values}

\item{truth}{ignored if \code{fit} is a \code{glm} object.  A vector of
observations, 0/1 or FALSE/TRUE values, of equal length to \code{fit}}

\item{n_threshold}{number of thresholds to use to estimate auroc or auprc}

\item{...}{passed to \code{\link[stats]{predict}}}
}
\description{
Deprecated methods for building the data sets needed for plotting
Receiver-Operating Curves and Precision-Recall Curves. Use
\code{\link{confusion_matrix}} instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{extract_fstat}
\alias{extract_fstat}
\alias{extract_fpvalue}
\alias{extract_fpvalue.lm}
\title{Extract Summary stats from regression objects}
\usage{
extract_fstat(x)

extract_fpvalue(x)

\method{extract_fpvalue}{lm}(x)
}
\arguments{
\item{x}{a \code{lm} object}
}
\value{
a character vector of the formatted numbers

formatted p-value from the F-test
}
\description{
A collection of functions for extracting summary statistics and
reporting regression results from \code{lm}, \code{glm} and other regression
objects.
}
\examples{
fit <- lm(mpg ~ wt + hp + drat, data = mtcars)
summary(fit)
extract_fstat(fit)
extract_fpvalue(fit)

}
\seealso{
\code{\link{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gmean_sd.R
\name{gmean_sd}
\alias{gmean_sd}
\title{Geometric Mean and Standard deviation}
\usage{
gmean_sd(
  x,
  digits = getOption("qwraps2_frmt_digits", 2),
  na_rm = FALSE,
  show_n = "ifNA",
  denote_sd = "pm",
  markup = getOption("qwraps2_markup", "latex"),
  ...
)
}
\arguments{
\item{x}{a numeric vector}

\item{digits}{digits to the right of the decimal point to return in the
percentage estimate.}

\item{na_rm}{if true, omit NA values}

\item{show_n}{defaults to \dQuote{ifNA}.  Other options are \dQuote{always} or
\dQuote{never}.}

\item{denote_sd}{a character string set to either "pm" or "paren" for reporting 'mean
\eqn{\pm} sd' or 'mean (sd)'}

\item{markup}{character string with value \dQuote{latex} or \dQuote{markdown}}

\item{...}{pass through}
}
\value{
a character vector of the formatted values
}
\description{
A function for calculating and formatting geometric means and
standard deviations.
}
\details{
Given a numeric vector, \code{gmean_sd} will return a character string with
the geometric mean and standard deviation.  Formatting of the output will be
extended in future versions.
}
\examples{

gmean_sd(mtcars$mpg, markup = "latex")
gmean_sd(mtcars$mpg, markup = "markdown")

}
\seealso{
\code{\link{mean_sd}}, \code{\link{gmean}}, \code{\link{gsd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2_tools.R
\name{ggplot2_extract_legend}
\alias{ggplot2_extract_legend}
\title{ggplot2 tools}
\usage{
ggplot2_extract_legend(x, ...)
}
\arguments{
\item{x}{a ggplot object}

\item{...}{not currently used}
}
\value{
a list with each elements
\describe{
  \item{legend}{}
  \item{plot}{the x}
}
}
\description{
A few handy tools for working with ggplot2.
}
\details{
The \code{ggplot2_extract_legend} function returns a list with the first
element being the legend and the second the original plot with the legend
omitted.
}
\examples{
# a simple plot
my_plot <-
  ggplot2::ggplot(mtcars) +
  ggplot2::aes(x = wt, y = mpg, color = wt, shape = factor(cyl)) +
  ggplot2::geom_point()

my_plot

# extract the legend.  the return object is a list with two elements, the first
# element is the legend, the second is the original plot sans legend.
temp <- ggplot2_extract_legend(my_plot)

# view just the legend.  This can be done via a call to the object or using
# plot or print.
temp
plot(temp[[1]])

# the original plot without the legened
plot(temp[[2]])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_table.R
\name{summary_table}
\alias{summary_table}
\alias{qsummary}
\title{Data Summary Tables}
\usage{
summary_table(x, summaries = qsummary(x), by = NULL, qable_args = list(), ...)

qsummary(x, numeric_summaries, n_perc_args, env = parent.frame())
}
\arguments{
\item{x}{a \code{data.frame}.}

\item{summaries}{a list of lists of formulas for summarizing the data set.
See Details and examples.}

\item{by}{a character vector of variable names to generate the summary by,
that is, one column for each unique value or combination of values of the
variables specified.}

\item{qable_args}{additional values passed to \code{\link{qable}}}

\item{...}{pass through}

\item{numeric_summaries}{a list of functions to use for summarizing numeric
variables.  The functions need to be provided as character strings with the
single argument defined by the \code{\%s} symbol.}

\item{n_perc_args}{a list of arguments to pass to
\code{\link[qwraps2]{n_perc}} to be used with \code{character} or
\code{factor} variables within \code{x}.}

\item{env}{environment to assign to the resulting formulae}
}
\value{
a \code{qwraps2_summary_table} object.
}
\description{
Tools useful for building data summary tables.
}
\details{
\code{summary_table} can be used to generate good-looking, simple tables in
LaTeX or markdown.  Functions like \code{xtable::print.xtable} and
\code{Hmisc::latex} provide many more tools for formatting tables.  The
purpose of \code{summary_table} is to generate readable tables quickly while
summarizing a data set.

Creating a list-of-lists of summary functions to apply to a data set will
allow the exploration of the whole data set and grouped data sets.  In the
example provided on this page we see a set of summary measures for the
\code{\link[datasets]{mtcars}} data set and the construction of a table for
the whole data set and for a grouped data set.


The list-of-lists should be thought of as follows:  the outer list defines
row groups, the inner lists define the rows within each row group.

More detailed use of these functions can be found in the
\code{vignette("qwraps2-summary-table", package = "qwraps2")}.

The \code{print} method for the \code{qwraps2_summary_table} objects is just
a simple wrapper for \code{\link{qable}}.
}
\examples{
summary_table(mtcars[, c("mpg", "cyl", "am")])
summary_table(mtcars[, c("mpg", "cyl", "am")], by = "am")

# A list-of-lists for the summaries arg.  This object is of the basic form:
#
# list("row group A" =
#      list("row 1A" = ~ <summary function>,
#           "row 2A" = ~ <summary function>),
#      "row group B" =
#      list("row 1B" = ~ <summary function>,
#           "row 2B" = ~ <summary function>,
#           "row 3B" = ~ <summary function>))

our_summaries <-
  list("Miles Per Gallon" =
         list("min"  = ~ min(mpg),
              "mean" = ~ mean(mpg),
              "mean &plusmn; sd" = ~ qwraps2::mean_sd(mpg),
              "max"  = ~ max(mpg)),
       "Weight" =
         list("median" = ~ median(wt)),
       "Cylinders" =
         list("4 cyl: n (\%)" = ~ qwraps2::n_perc0(cyl == 4),
              "6 cyl: n (\%)" = ~ qwraps2::n_perc0(cyl == 6),
              "8 cyl: n (\%)" = ~ qwraps2::n_perc0(cyl == 8)))

# Going to use markdown for the markup language in this example,  the original
# option will be reset at the end of the example.
orig_opt <- options()$qwraps2_markup
options(qwraps2_markup = "markdown")

# The summary table for the whole mtcars data set
whole_table <- summary_table(mtcars, our_summaries)
whole_table

# The summary table for mtcars grouped by am (automatic or manual transmission)
# This will generate one column for each level of mtcars$am
grouped_by_table <-
  summary_table(mtcars, our_summaries, by = "am")
grouped_by_table

# an equivalent call if you are using the tidyverse:
if (requireNamespace("dplyr", quietly = TRUE)) {
  summary_table(dplyr::group_by(mtcars, am), our_summaries)

  # To build a table with a column for the whole data set and each of the am
  # levels
  cbind(whole_table, grouped_by_table)

  # Adding a caption for a LaTeX table
  print(whole_table, caption = "Hello world", markup = "latex")

  # A **warning** about grouped_df objects.
  # If you use dplyr::group_by or
  # dplyr::rowwise to manipulate a data set and fail to use dplyr::ungroup you
  # might find a table that takes a long time to create and does not summarize the
  # data as expected.  For example, let's build a data set with twenty subjects
  # and injury severity scores for head and face injuries.  We'll clean the data
  # by finding the max ISS score for each subject and then reporting summary
  # statistics thereof.
  set.seed(42)
  dat <- data.frame(id = letters[1:20],
                    head_iss = sample(1:6, 20, replace = TRUE, prob = 10 * (6:1)),
                    face_iss = sample(1:6, 20, replace = TRUE, prob = 10 * (6:1)))
  dat <- dplyr::group_by(dat, id)
  dat <- dplyr::mutate(dat, iss = max(head_iss, face_iss))

  iss_summary <-
    list("Head ISS" =
         list("min"    = ~ min(head_iss),
              "median" = ~ median(head_iss),
              "max"    = ~ max(head_iss)),
         "Face ISS" =
         list("min"    = ~ min(face_iss),
              "median" = ~ median(face_iss),
              "max"    = ~ max(face_iss)),
         "Max ISS" =
         list("min"    = ~ min(iss),
              "median" = ~ median(iss),
              "max"    = ~ max(iss)))

  # Want: a table with one column for all subjects with nine rows divided up into
  # three row groups.  However, the following call will create a table with 20
  # columns, one for each subject because dat is a grouped_df
  summary_table(dat, iss_summary)

  # Ungroup the data.frame to get the correct output
  summary_table(dplyr::ungroup(dat), iss_summary)
}

################################################################################
# The Default call will work with non-syntactically valid names and will
# generate a table with statistics defined by the qsummary call.
summary_table(mtcars, by = "cyl")

# Another example from the diamonds data
data("diamonds", package = "ggplot2")
diamonds["The Price"] <- diamonds$price
diamonds["A Logical"] <- sample(c(TRUE, FALSE), size = nrow(diamonds), replace = TRUE)

# the next two lines are equivalent.
summary_table(diamonds)
summary_table(diamonds, qsummary(diamonds))

summary_table(diamonds, by = "cut")

summary_table(diamonds,
              summaries =
              list("My Summary of Price" =
                   list("min price" = ~ min(price),
                        "IQR"       = ~ stats::IQR(price))),
              by = "cut")

################################################################################
# Data sets with missing values
temp <- mtcars
temp$cyl[5] <- NA
temp$am[c(1, 5, 10)] <- NA
temp$am <- factor(temp$am, levels = 0:1, labels = c("Automatic", "Manual"))
temp$vs <- as.logical(temp$vs)
temp$vs[c(2, 6)] <- NA
qsummary(temp[, c("cyl", "am", "vs")])
summary_table(temp[, c("cyl", "am", "vs")])

################################################################################
# Group by Multiple Variables
temp <- mtcars
temp$trans <- factor(temp$am, 0:1, c("Manual", "Auto"))
temp$engine <- factor(temp$vs, 0:1, c("V-Shaped", "Straight"))
summary_table(temp, our_summaries, by = c("trans", "engine"))

################################################################################
# binding tables together.  The original design and expected use of
# summary_table did not require a rbind, as all rows are defined in the
# summaries argument.  That said, here are examples of using cbind and rbind to
# build several different tables.
our_summary1 <-
  list("Miles Per Gallon" =
       list("min" = ~ min(mpg),
            "max" = ~ max(mpg),
            "mean (sd)" = ~ qwraps2::mean_sd(mpg)),
       "Displacement" =
       list("min" = ~ min(disp),
            "max" = ~ max(disp),
            "mean (sd)" = ~ qwraps2::mean_sd(disp)))

our_summary2 <-
  list(
       "Weight (1000 lbs)" =
       list("min" = ~ min(wt),
            "max" = ~ max(wt),
            "mean (sd)" = ~ qwraps2::mean_sd(wt)),
       "Forward Gears" =
       list("Three" = ~ qwraps2::n_perc0(gear == 3),
            "Four"  = ~ qwraps2::n_perc0(gear == 4),
            "Five"  = ~ qwraps2::n_perc0(gear == 5))
       )

tab1 <- summary_table(mtcars, our_summary1)
tab2 <- summary_table(mtcars, our_summary1, by = "am")
tab3 <- summary_table(mtcars, our_summary1, by = "vs")

tab4 <- summary_table(mtcars, our_summary2)
tab5 <- summary_table(mtcars, our_summary2, by = "am")
tab6 <- summary_table(mtcars, our_summary2, by = "vs")

cbind(tab1, tab2, tab3)
cbind(tab4, tab5, tab6)

# row bind is possible, but it is recommended to extend the summary instead.
rbind(tab1, tab4)
summary_table(mtcars, summaries = c(our_summary1, our_summary2))

\dontrun{
  cbind(tab1, tab4) # error because rows are not the same
  rbind(tab1, tab2) # error because columns are not the same
}

################################################################################
# reset the original markup option that was used before this example was
# evaluated.
options(qwraps2_markup = orig_opt)

# Detailed examples in the vignette
# vignette("qwraps2-summary-table", package = "qwraps2")


}
\seealso{
\code{\link{qsummary}} for generating the summaries,
\code{\link{qable}} for marking up \code{qwraps2_summary_table} objects.
The \code{vignette("qwraps2-summary-table", package = "qwraps2")} for
detailed use of these functions and caveats.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometric_mean_sd.R
\name{gmean}
\alias{gmean}
\alias{gvar}
\alias{gsd}
\title{Geometric Mean, Variance, and Standard Deviation}
\usage{
gmean(x, na_rm = FALSE)

gvar(x, na_rm = FALSE)

gsd(x, na_rm = FALSE)
}
\arguments{
\item{x}{a numeric vector}

\item{na_rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}
}
\value{
a numeric value
}
\description{
Return the geometric mean, variance, and standard deviation,
}
\examples{

gmean(mtcars$mpg)
identical(gmean(mtcars$mpg), exp(mean(log(mtcars$mpg))))

gvar(mtcars$mpg)
identical(gvar(mtcars$mpg),
          exp(var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars)))

gsd(mtcars$mpg)
identical(gsd(mtcars$mpg),
          exp(sqrt( var(log(mtcars$mpg)) * (nrow(mtcars) - 1) / nrow(mtcars))))

#############################################################################
set.seed(42)
x <- runif(14, min = 4, max = 70)

# geometric mean - four equivalent ways to get the same result
prod(x) ^ (1 / length(x))
exp(mean(log(x)))
1.2 ^ mean(log(x, base = 1.2))
gmean(x)

# geometric variance
gvar(x)

# geometric sd
exp(sd(log(x)))                                     ## Uses n - 1
exp(sqrt((length(x) - 1) / length(x)) * sd(log(x))) ## Correct calculation
gsd(x)

# Missing data will result in and NA being returned
x[c(2, 4, 7)] <- NA
gmean(x)
gmean(x, na_rm = TRUE)
gvar(x, na_rm = TRUE)
gsd(x, na_rm = TRUE)

}
\seealso{
\code{\link{gmean_sd}} for easy formatting of the geometric mean and
standard deviation.
\code{vignette("qwraps2-formatted-summary-statistics", package = "qwraps2")}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat-stepribbon.R
\name{StatStepribbon}
\alias{StatStepribbon}
\title{Stat Step Ribbon}
\description{
Provides stair step values for ribbon plots (Copied this from the
https://github.com/hrbrmstr/ggalt version 0.6.0, which is not yet on CRAN.
Some minor modifications to the file have been made).
}
\references{
\url{https://groups.google.com/forum/?fromgroups=#!topic/ggplot2/9cFWHaH1CPs}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mtcars2}
\alias{mtcars2}
\title{mtcars2}
\format{
a data.frame with 32 rows and 19 columns

\tabular{rlll}{
[,  1] \tab make          \tab Manufacturer name                        \tab parted out from \code{rownames(mtcars)} \cr
[,  2] \tab model         \tab                                          \tab parted out from \code{rownames(mtcars)} \cr
[,  3] \tab mpg           \tab miles per (US) gallon                    \tab identical to mtcars$mpg \cr
[,  4] \tab disp          \tab Displacement (cu.in.)                    \tab identical to mtcars$disp \cr
[,  5] \tab hp            \tab Gross horsepower                         \tab identical to mtcars$hp \cr
[,  6] \tab drat          \tab Rear axle ratio                          \tab identical to mtcars$drat \cr
[,  7] \tab wt            \tab weight (1000 lbs)                        \tab identical to mtcars$wt \cr
[,  8] \tab qsec          \tab 1/4 mile time                            \tab identical to mtcars$qsec \cr
[,  9] \tab cyl           \tab number of cylinders                      \tab identical to mtcars$cyl \cr
[, 10] \tab cyl_character \tab                                          \tab \cr
[, 11] \tab cyl_factor    \tab                                          \tab \cr
[, 12] \tab vs            \tab Engine (0 = V-shaped, 1 = straight)      \tab identical to mtcars$vs \cr
[, 13] \tab engine        \tab                                          \tab \cr
[, 14] \tab am            \tab Transmission (0 = automatic, 1 = manual) \tab identical to mtcars$am \cr
[, 15] \tab transmission  \tab                                          \tab \cr
[, 16] \tab gear          \tab Number of forward gears                  \tab identical to mtcars$gear \cr
[, 17] \tab gear_factor   \tab                                          \tab \cr
[, 18] \tab carb          \tab Number of carburetors                    \tab identical to mtcars$carb \cr
[, 19] \tab test_date     \tab fictitious testing date                  \tab \cr
}
}
\usage{
mtcars2
}
\description{
An extended version of \code{\link[datasets]{mtcars}} data set.
}
\seealso{
\code{vignette("qwraps2-data-sets", package = "qwraps2")} for
details on the construction of the data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qwraps2-package.R
\docType{package}
\name{qwraps2-package}
\alias{qwraps2}
\alias{qwraps2-package}
\title{Quick Wraps for Reproducible Reports}
\description{
\pkg{qwraps2} is a collection of helper functions for authoring reproducible
analysis reports. The package focuses on formatted summary statistics,
summary tables, regression-result formatting, and \pkg{ggplot2}-based plots.
}
\details{
Several wrappers for \pkg{ggplot2}-style graphics are provided, including
receiver operating characteristic curves, precision-recall curves,
Bland-Altman plots, autocorrelation plots, and Kaplan-Meier plots. These
functions include \code{\link{qroc}}, \code{\link{qprc}},
\code{\link{qacf}}, \code{\link{qblandaltman}}, and \code{\link{qkmplot}}.

Other functions quickly generate meaningful character strings for reporting
results in .Rnw, .Rmd, or similar report files.
}
\section{Vignettes}{

\itemize{
  \item \code{vignette("qwraps2-formatted-summary-statistics", package = "qwraps2")}
  introduces individual summary-statistic and formatting helpers.
  \item \code{vignette("qwraps2-summary-table", package = "qwraps2")}
  demonstrates full summary-table workflows.
  \item \code{vignette("qwraps2-graphics", package = "qwraps2")}
  demonstrates the plotting helpers.
  \item \code{vignette("qwraps2-data-sets", package = "qwraps2")}
  documents the bundled example data sets.
}
}

\section{Options}{

There are several options which can be set via \code{options} and will be
used via \code{getOption}.  The following lists, in alphabetical order the
different options which are available and what they control.

\itemize{
  \item \code{getOption("qwraps2_alpha", 0.05)} significance level, used for
  generating \code{(1 - getOption("qwraps2_alpha", 0.05)) * 100}\% confidence
  intervals, and determining significance for p-value <
  \code{getOption("qwraps2_alpha", 0.05)}.

  \item \code{getOption("qwraps2_frmt_digits", 2)}  Number of digits to the
  right of the decimal point for any value other than p-values.

  \item \code{getOption("qwraps2_frmtp_case", "upper")} set to either
  'upper' or 'lower' for the case of the 'P' for reporting p-values.

  \item \code{getOption("qwraps2_frmtp_digits", 4)}  Number of digits to the
    right of the decimal point to report p-values too.  If
    \code{log10(p-value) < getOption("qwraps2_frmtp_digits", 4)} then the
    output will be "P < 0.01", to however many digits are correct.  Other
    options control other parts of the output p-value format.

  \item \code{getOption("qwraps2_frmtp_leading0", TRUE)} to display or not
  to display the leading zero in p-values, i.e., if TRUE p-values are
  reported as 0.02 versus when FALSE p-values are reported as .02.

  \item \code{getOption("qwraps2_journal", "default")} if a journal has
  specific formatting for p-values or other statistics, this option will
  control the output.  Many other options are ignored if this is any other
  than default.  Check the GitHub wiki, or this file,  for current lists of
  implemented journal style methods.

  \item \code{getOption("qwraps2_markup", "latex")} value set to 'latex' or
  to 'markdown'.  Output is formatted to meet requirements of either markup
  language.
}
}

\section{Journals with predefined formatting}{


\itemize{
  \item Obstetrics & Gynecology
  \itemize{
     \item \url{https://www.editorialmanager.com/ong/default.aspx}
     \item \code{options(qwraps2_journal = "obstetrics_gynecology")}
     \item P-value formatting as of April 2015:

      Express P values to no more than three decimal places.

      Based on observations of published work, leading 0 will be omitted.
   }

  \item Pediatric Dentistry:
  \itemize{
    \item \url{https://www.aapd.org/publications/}
    \item \code{options(qwraps2_journal = "pediatric_dentistry")}
    \item P-value formatting  as of March 2015.

    If P > .01, the actual value for
    P should be expressed to 2 digits.  Non-significant values should not be
    expressed as "NS" whether or not P is significant, unless rounding a
    significant P-value expressed to 3 digits would make it non-significant
    (i.e., P=.049, not P=.05).  If P<.01, it should be expressed to 3 digits
    (e.g., P=.003, not P<.05).  Actual P-values should be expressed unless
    P<.001, in which case they should be so designated.
  }

}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/dewittpe/qwraps2/}
  \item \url{https://www.peteredewitt.com/qwraps2/}
  \item Report bugs at \url{https://github.com/dewittpe/qwraps2/issues}
}

}
\author{
\strong{Maintainer}: Peter DeWitt \email{dewittpe@gmail.com} (\href{https://orcid.org/0000-0002-6391-0795}{ORCID})

Authors:
\itemize{
  \item Peter DeWitt \email{dewittpe@gmail.com} (\href{https://orcid.org/0000-0002-6391-0795}{ORCID})
}

Other contributors:
\itemize{
  \item Tell Bennett \email{tell.bennett@cuanschutz.edu} (\href{https://orcid.org/0000-0003-1483-4236}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{traprule}
\alias{traprule}
\title{Trapezoid Rule Numeric Integration}
\usage{
traprule(x, y)
}
\arguments{
\item{x, y}{numeric vectors of equal length}
}
\value{
a numeric value, the estimated integral
}
\description{
Compute the integral of y with respect to x via trapezoid rule.
}
\examples{
xvec <- seq(-2 * pi, 3 * pi, length = 560)
foo  <- function(x) { sin(x) + x * cos(x) + 12 }
yvec <- foo(xvec)
plot(xvec, yvec, type = "l")

integrate(f = foo, lower = -2 * pi, upper = 3 * pi)
traprule(xvec, yvec)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_checks.R
\name{pkg_check}
\alias{pkg_check}
\title{Package Checks}
\usage{
pkg_check(pkgs, versions, stop = FALSE)
}
\arguments{
\item{pkgs}{a character vector of package names to check for}

\item{versions}{an optional character vector, of the same length of
\code{pkgs} for the minimum version of the packages.}

\item{stop}{if \code{TRUE} then an error is thrown if any of the checks fail.
If \code{FALSE} (default) a logical is returned.}
}
\description{
Check if a package is available on the local machine and optionally verify a
version.
}
\details{
When writing a script that will be shared it is very likely that the multiple
authors/users will need to have a certain set of packages available to load.
The \code{pkg_check} function will verify that the packages are available to
load, this includes an optional version test, and attach the package to the
search list if requested.

Testing for package versions will is done as \code{packageVersion(x) >=
version}.  If you need a specific version of a package you should explicitly
use \code{packageVersion(x) == version} in your script.  In general,
\code{pkg_check} is a handy tool in interactive sessions.  For a package you
should have package version documentation in the DESCRIPTION file.
For a script a base R solution of
\code{stopifnot(packageVersion("pkg") >= "x.y.z")}
}
\examples{
# verify that the packages qwraps2, and ggplot2 are available (this should be
# TRUE if you have qwraps2 installed since ggplot2 is imported by qwraps2)
pkg_check(c("qwraps2", "ggplot2"))

# show that the return is FALSE if a package is not available
pkg_check(c("qwraps2", "ggplot2", "NOT a PCKG"))

# verify the version for just ggplot2
pkg_check(c("qwraps2", "ggplot2"), c(NA, "2.2.0"))

# verify the version for qwraps2 (this is expected to fail as we are looking for
# version 42.3.14 which is far too advanced for the actual package development.
pkg_check(c("qwraps2", "ggplot2"), c("42.3.14", "2.2.0"))

\dontrun{
  # You can have the function throw an error is any of the checks fail
  pkg_check(c("qwraps2", "ggplot2"),
            c("42.3.14", "2.2.0"),
            stop = TRUE)
}

\dontrun{
  # If you have missing packages that can be installed from CRAN you may find
  # the following helpful.  If this code, with the needed edits, were placed at
  # the top of a script, then if a package is missing then the current version
  # from a target repository will be installed.  Use this set up with
  # discretion, others may not want the automatic install of packages.
  pkgs <- pkg_check("<packages to install>")
  if (!pkgs) {
    install.packages(attr(pkgs, "checks")[!attr(pkgs, "checks")$available][["package"]])
  }
}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{pefr}
\alias{pefr}
\title{pefr}
\format{
a data frame with four columns
\tabular{rll}{
[, 1] \tab subject     \tab id number \cr
[, 2] \tab measurement \tab first or second \cr
[, 3] \tab meter       \tab \dQuote{Wright peak flow meter} or \dQuote{Mini Write peak flow meter} \cr
[, 4] \tab pefr        \tab peak expiratory flow rate (liters / min) \cr
}
}
\usage{
pefr
}
\description{
Peak expiratory flow rate data
}
\details{
Peak expiratory flow rate (pefr) data is used for examples within the qwraps2
package.  The data has been transcribed from Bland (1986).

\dQuote{The sample comprised colleagues and family of J.M.B. chosen to give a
wide range of PEFR but in no way representative of any defined population.
Two measurements were made with a Wright peak flow meter and two with a mini
Wright meter, in random order. All measurements were taken by J.M.B., using
the same two instruments. (These data were collected to demonstrate the
statistical method and provide no evidence on the comparability of these two
instruments.) We did not repeat suspect readings and took a single reading as
our measurement of PEFR. Only the first measurement by each method is used to
illustrate the comparison of methods, the second measurements being used in
the study of repeatability.}
}
\references{
Bland, J. Martin, and Douglas G Altman. "Statistical methods for assessing
agreement between two methods of clinical measurement." The lancet 327, no. 8476
(1986): 307-310.
}
\seealso{
\code{vignette('qwraps2-data-sets', package = 'qwraps2')} for
details on the construction of the data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_diff.R
\name{set_diff}
\alias{set_diff}
\alias{print.qwraps2_set_diff}
\title{Set Differences}
\usage{
set_diff(x, y)

\method{print}{qwraps2_set_diff}(x, verbose = (length(x$x_only) < 10 && length(x$y_only) < 10), ...)
}
\arguments{
\item{x, y}{vectors (of the same mode)}

\item{verbose}{a logical value indicating whether to print the unique
elements in each set.}

\item{...}{ignored}
}
\value{
a qwraps2_set_diff object, a list of set comparisons

\itemize{
  \item \code{all_values} = \code{\link[base]{union}(x, y)}
  \item \code{x_only}     = \code{\link[base]{setdiff}(x, y)}
  \item \code{y_only}     = \code{\link[base]{setdiff}(y, x)}
  \item \code{both}       = \code{\link[base]{intersect}(x, y)}
  \item \code{equal}      = \code{\link[base]{setequal}(x, y)}
}
}
\description{
Function for testing for unique values between two vectors, specifically,
which values are in vector1, and not in vector2, which values are not in
vector1 and in vector2, which values are in both vector1 and vector2.
}
\examples{

# example with two sets which as a union are the upper and lower case vowels.
set_a <- c("A", "a", "E",      "I", "i", "O", "o", "U", "u", "E", "I")
set_b <- c("A", "a", "E", "e",      "i",      "o", "U", "u", "u", "a", "e")

set_diff(set_a, set_b)
str(set_diff(set_a, set_b))

set_diff(set_b, set_a)

# example
set_a <- 1:90
set_b <- set_a[-c(23, 48)]
set_diff(set_a, set_b)
set_diff(set_b, set_a)

# example
set_a <- c("A", "A", "B")
set_b <- c("B", "A")
set_diff(set_a, set_b)

# When there are 10 or more unique items in either set, the default print will
# omit printing them by default.  Use the verbose argument to change this

x <-
  set_diff(
    x = c(LETTERS, letters[1:10]),
    y = letters
  )
print(x)
print(x, verbose = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spin.R
\name{check_comments}
\alias{check_comments}
\title{Check Comments}
\usage{
check_comments(c1, c2)
}
\arguments{
\item{c1}{index (line numbers) for the start delimiter of comments}

\item{c2}{index (line numbers) for the end delimiter of comments}
}
\description{
A more robust check for open/close matching sets of comments in a spin file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{logit}
\alias{logit}
\alias{invlogit}
\title{Deprecated logit and invlogit}
\usage{
logit(x)

invlogit(x)
}
\arguments{
\item{x}{a numeric vector}
}
\description{
Deprecated logit and invlogit functions.  When originally written they were
faster than the base R \code{\link{qlogis}} and \code{\link{plogis}}.
However, that is no longer the case.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qacf.R
\name{qacf}
\alias{qacf}
\title{Autocorrelation plot}
\usage{
qacf(
  x,
  conf_level = 1 - getOption("qwraps2_alpha", 0.05),
  show_sig = FALSE,
  ...
)
}
\arguments{
\item{x}{object}

\item{conf_level}{confidence level for determining \sQuote{significant}
correlations}

\item{show_sig}{logical, highlight significant correlations}

\item{...}{Other arguments passed to \code{\link[stats]{acf}}}
}
\value{
a ggplot.
}
\description{
ggplot2 style autocorrelation plot
}
\details{
qacf calls \code{\link[stats]{acf}} to generate a data set which is
then plotted via ggplot2.

More details and examples for graphics within qwraps2 are in the
vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
}
\examples{
# Generate a random data set
set.seed(42)
n <- 250
x1 <- x2 <- x3 <- x4 <- vector('numeric', length = n)
x1[1] <- runif(1)
x2[1] <- runif(1)
x3[1] <- runif(1)
x4[1] <- runif(1)

# white noise
Z_1 <- rnorm(n, 0, 1)
Z_2 <- rnorm(n, 0, 2)
Z_3 <- rnorm(n, 0, 5)

for(i in 2:n)
{
  x1[i] <- x1[i-1] + Z_1[i] - Z_1[i-1] + x4[i-1] - x2[i-1]
  x2[i] <- x2[i-1] - 2 * Z_2[i] + Z_2[i-1] - x4[i-1]
  x3[i] <- x3[i-1] + x2[i-1] + 0.2 * Z_3[i] + Z_3[i-1]
  x4[i] <- x4[i-1] + runif(1, 0.5, 1.5) * x4[i-1]
}
testdf <- data.frame(x1, x2, x3, x4)

# qacf plot for one variable
qacf(testdf$x1)
qacf(testdf$x1, show_sig = TRUE)

# more than one variable
qacf(testdf)
qacf(testdf, show_sig = TRUE)

}
\seealso{
\code{\link[stats]{acf}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qblandaltman.R
\name{qblandaltman}
\alias{qblandaltman}
\alias{qblandaltman_build_data_frame}
\title{Bland Altman Plots}
\usage{
qblandaltman(x, alpha = getOption("qwraps2_alpha", 0.05), generate_data = TRUE)

qblandaltman_build_data_frame(x, alpha = getOption("qwraps2_alpha", 0.05))
}
\arguments{
\item{x}{a \code{data.frame} with two columns, or an object that can be
coerced to a data frame.  If a \code{data.frame} with more than two columns
is used only the first two columns will be used.}

\item{alpha}{(Defaults to 0.05) place (1 - alpha)*100% confidence levels to
place on the plot.}

\item{generate_data}{logical, defaults to TRUE.  If TRUE, then the call to
\code{qblandaltman_build_data_frame} is done automatically for you.  If
FALSE, then you should explicitly call \code{qblandaltman_build_data_frame}
before calling \code{qblandaltman}.}
}
\value{
a ggplot.  Minimal aesthetics have been used so that the user may
modify the graphic as desired with ease.
}
\description{
Construct and plot a Bland Altman plot in ggplot2.
}
\details{
Providing a \code{data.frame} with two columns, the function returns a ggplot
version of a Bland Altman plot with the specified confidence intervals.

Two ways to call the plotting function.  If you submit a \code{data.frame}
\code{qblandaltman} then the data needed to produce the Bland Altman plot is
automatically generated by a call to \code{qblandaltman_build_data_frame}.
Alternatively, you may call \code{qblandaltman_build_data_frame} directly and
then call \code{qblandaltman}.  This might be helpful if you are putting
multiple Bland Altman plots together into one ggplot object.  See Examples.

More details and examples for graphics within qwraps2 are in the
vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
}
\examples{

data(pefr)
pefr_m1 <-
  cbind("Large" = pefr[pefr$measurement == 1 & pefr$meter == "Wright peak flow meter", "pefr"],
        "Mini"  = pefr[pefr$measurement == 1 & pefr$meter == "Mini Wright peak flow meter", "pefr"])

# The Bland Altman plot plots the average value on the x-axis and the
# difference in the measurements on the y-axis:
qblandaltman(pefr_m1) +
  ggplot2::xlim(0, 800) +
  ggplot2::ylim(-100, 100) +
  ggplot2::xlab("Average of two meters") +
  ggplot2::ylab("Difference in the measurements")

}
\references{
Altman, Douglas G., and J. Martin Bland. "Measurement in medicine: the analysis
of method comparison studies." The statistician (1983): 307-317.

Bland, J. Martin, and Douglas G Altman. "Statistical methods for assessing
agreement between two methods of clinical measurement." The lancet 327, no. 8476
(1986): 307-310.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{qkmplot_bulid_data_frame}
\alias{qkmplot_bulid_data_frame}
\title{Deprecated qkmplot data-frame builder}
\usage{
qkmplot_bulid_data_frame(x)
}
\arguments{
\item{x}{object}
}
\description{
Deprecated misspelled wrapper for \code{\link{qkmplot_build_data_frame}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confusion_matrix.R
\name{confusion_matrix}
\alias{confusion_matrix}
\alias{confusion_matrix.default}
\alias{confusion_matrix.formula}
\alias{confusion_matrix.glm}
\alias{print.qwraps2_confusion_matrix}
\title{Confusion Matrices (Contingency Tables)}
\usage{
confusion_matrix(
  ...,
  thresholds = NULL,
  confint_method = "logit",
  alpha = getOption("qwraps2_alpha", 0.05)
)

\method{confusion_matrix}{default}(
  truth,
  predicted,
  ...,
  thresholds = NULL,
  confint_method = "logit",
  alpha = getOption("qwraps2_alpha", 0.05)
)

\method{confusion_matrix}{formula}(
  formula,
  data = parent.frame(),
  ...,
  thresholds = NULL,
  confint_method = "logit",
  alpha = getOption("qwraps2_alpha", 0.05)
)

\method{confusion_matrix}{glm}(
  x,
  ...,
  thresholds = NULL,
  confint_method = "logit",
  alpha = getOption("qwraps2_alpha", 0.05)
)

\method{print}{qwraps2_confusion_matrix}(x, ...)
}
\arguments{
\item{...}{pass through}

\item{thresholds}{a numeric vector of thresholds to be used to define the
confusion matrix (one threshold) or matrices (two or more thresholds).  If
\code{NULL} the unique values of \code{predicted} will be used.}

\item{confint_method}{character string denoting if the logit (default),
binomial, or Wilson Score method for deriving confidence intervals}

\item{alpha}{alpha level for 100 * (1 - alpha)\% confidence intervals}

\item{truth}{a integer vector with the values \code{0} and \code{1}, or a logical vector.
A value of \code{0} or \code{FALSE} is an indication of condition negative;
\code{1} or \code{TRUE} is an indication of condition positive.}

\item{predicted}{a numeric vector.  See Details.}

\item{formula}{column (known) ~ row (test) for building the confusion matrix}

\item{data}{environment containing the variables listed in the formula}

\item{x}{a \code{glm} object}
}
\value{
\code{confusion_matrix} returns a list with elements
\itemize{
  \item \code{cm_stats} a data.frame with columns:
  \item \code{auroc} numeric value for the area under the receiver operating
  curve
  \item \code{auroc_ci} a numeric vector of length two with the lower and
  upper bounds for a 100(1-alpha)\% confidence interval about the auroc
  \item \code{auprc} numeric value for the area under the precision recall
  curve
  \item \code{auprc_ci} a numeric vector of length two with the lower and
  upper limits for a 100(1-alpha)\% confidence interval about the auprc
  \item \code{confint_method} a character string reporting the method used to
  build the \code{auroc_ci} and \code{auprc_ci}
  \item \code{alpha} the alpha level of the confidence intervals
  \item \code{prevalence} the proportion of the input of positive cases, that
  is (TP + FN) / (TP + FN + FP + TN) = P / (P + N)
}
}
\description{
Construction of confusion matrices, accuracy, sensitivity,
specificity, confidence intervals (Wilson's method and (optional
bootstrapping)).
}
\details{
The confusion matrix:

\tabular{lccc}{
                    \tab      \tab True \tab Condition \cr
                    \tab      \tab +    \tab -         \cr
Predicted Condition \tab +    \tab TP   \tab FP        \cr
Predicted Condition \tab -    \tab FN   \tab TN        \cr
}
where
\itemize{
  \item FN: False Negative = truth = 1 & prediction < threshold,
  \item FP: False Positive = truth = 0 & prediction >= threshold,
  \item TN: True Negative  = truth = 0 & prediction < threshold, and
  \item TP: True Positive  = truth = 1 & prediction >= threshold.
}

The statistics returned in the \code{cm_stats} element are:
\itemize{
  \item accuracy    = (TP + TN) / (TP + TN + FP + FN)
  \item sensitivity, aka true positive rate or recall = TP / (TP + FN)
  \item specificity, aka true negative rate = TN / (TN + FP)
  \item positive predictive value (PPV), aka precision = TP / (TP + FP)
  \item negative predictive value (NPV) = TN / (TN + FN)
  \item false negative rate (FNR) = 1 - Sensitivity
  \item false positive rate (FPR) = 1 - Specificity
  \item false discovery rate (FDR) = 1 - PPV
  \item false omission rate (FOR) = 1 - NPV
  \item F1 score
  \item Matthews Correlation Coefficient (MCC) =
    ((TP * TN) - (FP * FN)) / sqrt((TP + FP) (TP+FN) (TN+FP) (TN+FN))
}

Synonyms for the statistics:
\itemize{
\item Sensitivity: true positive rate (TPR), recall, hit rate
\item Specificity: true negative rate (TNR), selectivity
\item PPV: precision
\item FNR: miss rate
}

Sensitivity and PPV could, in some cases, be indeterminate due to division by
zero.  To address this we use the GERBIL convention from the DICE group
\url{https://github.com/dice-group/gerbil/wiki/Precision,-Recall-and-F1-measure}:
If TP, FP, and FN are all 0, then PPV, sensitivity, and F1 will be defined to
be 1.  If TP are 0 and FP + FN > 0, then PPV, sensitivity, and F1 are all
defined to be 0.
}
\examples{

# Example 1: known truth and prediction status
df <-
  data.frame(
      truth = c(1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0)
    , pred  = c(1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0)
  )

confusion_matrix(df$truth, df$pred, thresholds = 1)

# Example 2: Use with a logistic regression model
mod <- glm(
  formula = spam ~ word_freq_our + word_freq_over + capital_run_length_total
, data = spambase
, family = binomial()
)

confusion_matrix(mod)
confusion_matrix(mod, thresholds = 0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean_ci.R
\name{mean_ci}
\alias{mean_ci}
\alias{print.qwraps2_mean_ci}
\title{Means and Confidence Intervals}
\usage{
mean_ci(
  x,
  na_rm = FALSE,
  alpha = getOption("qwraps2_alpha", 0.05),
  qdist = stats::qnorm,
  qdist.args = list(),
  ...
)

\method{print}{qwraps2_mean_ci}(x, ...)
}
\arguments{
\item{x}{a numeric vector}

\item{na_rm}{if true, omit NA values}

\item{alpha}{defaults to \code{getOption('qwraps2_alpha', 0.05)}.  The
symmetric 100(1-alpha)\% CI will be determined.}

\item{qdist}{defaults to \code{qnorm}.  use \code{qt} for a Student t
intervals.}

\item{qdist.args}{list of arguments passed to \code{qdist}}

\item{...}{arguments passed to \code{frmtci}.}
}
\value{
a vector with the mean, lower confidence limit (LCL), and the upper
confidence limit (UCL).
}
\description{
A function for calculating and formatting means and
confidence interval.
}
\details{
Given a numeric vector, \code{mean_ci} will return a vector with the mean,
LCL, and UCL.  Using \code{frmtci} will be helpful for reporting the results
in print.
}
\examples{
# using the standard normal for the CI
mean_ci(mtcars$mpg)

# print it nicely
qwraps2::frmtci(mean_ci(mtcars$mpg))
qwraps2::frmtci(mean_ci(mtcars$mpg), show_level = TRUE)
qwraps2::frmtci(mean_ci(mtcars$mpg, alpha = 0.01), show_level = TRUE)

# Compare to the ci that comes form t.test
t.test(mtcars$mpg)
t.test(mtcars$mpg)$conf.int
mean_ci(mtcars$mpg, qdist = stats::qt, qdist.args = list(df = 31))

}
\seealso{
\code{\link{frmtci}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file-check.R
\name{file_check}
\alias{file_check}
\title{File and Working Directory Check}
\usage{
file_check(
  paths,
  md5sums = NULL,
  absolute_paths = c("warn", "stop", "silent"),
  stop = FALSE
)
}
\arguments{
\item{paths}{a character path to the target file}

\item{md5sums}{a character string for the expected md5sum of the target file.
If \code{NULL} then only a \code{file.exists} check will be done.}

\item{absolute_paths}{a character string to set the behavior of warning
(default), stopping, or silent if/when absolute file paths are used.}

\item{stop}{if \code{TRUE} then an error is thrown if any of the checks fail,
else returns a logical. If \code{FALSE} (default) a logical is returned.}
}
\value{
The function will return a single TRUE/FALSE value with attributes
\code{attr(, "checks")}.
}
\description{
This check is three-fold: 1) verify the current working directory is as
expected, 2) verify the user can access the file, and 3) verify the file
contents are as expected (via md5sum).
}
\details{
The test for the file access is done to verify the file can be read by the
current user.

The return of the function is \code{TRUE} if all the files in \code{paths}
are accessible, are case matched (optional), and all of requested md5sum
checks pass.  Windows and macOS are generally case-insensitive systems, but
many Linux systems are case-sensitive.  As such
\code{\link[base]{file.exists}} and \code{\link[base]{file.access}} may
return different values depending the OS that is active.  \code{file_check}
looks for a case match as part of its checks to hopefully prevent issues
across operating systems.

By default, if the return is \code{TRUE} then only \code{TRUE} will
be printed to the console.  If the return is \code{FALSE} then the
\code{attr(, "checks")} is printed by default as well.

Good practice would be to use relative paths, a warning will be given if any
of the \code{paths} are determined to be absolute paths.  That said, there
are cases when an absolute path is needed, e.g., a common data file on a
server with multiple users accessing the file(s).  Set \code{absolute_paths =
c("silent")} to silence the warnings.
}
\examples{
# create example files

relative_example_file1 <-
  basename(
    tempfile(
      pattern = "QWRAPS2_EXAMPLE_1"
      , fileext = ".txt"
      , tmpdir = getwd()
    )
  )

relative_example_file2 <-
  basename(
    tempfile(
      pattern = "QWRAPS2_EXAMPLE_2"
      , fileext = ".txt"
      , tmpdir = getwd()
    )
  )

absolute_example_file <- tempfile()

cat("example file.", file = relative_example_file1)
cat("Another example file.", file = relative_example_file2)
cat("Another example file.", file = absolute_example_file)

# Check that you have access to the files in the working directory.
test1 <- file_check(c(relative_example_file1, relative_example_file2))
test1

# By default, when the checks return TRUE the details of the checks are not
# printed.  You can view the details of the checks as follows:
attr(test1, "checks")

# access to absolute_example_file will generate a warning about
# absolute_paths by default
test2 <- file_check(absolute_example_file)
test2 <- file_check(absolute_example_file, absolute_paths = "silent")
test2

# Case Match
test_case_match <-
  file_check(
    c(relative_example_file1, tolower(relative_example_file1))
  )
test_case_match

# If one or more files is not accessible then return is FALSE and the metadata
# is printed by default.
test_non_existent_file <-
  file_check(
    c("UNLIKELYFILENAME", relative_example_file1, relative_example_file2)
  )
test_non_existent_file

# Or have an error thrown:
\dontrun{
file_check(
  c("UNLIKELYFILENAME", relative_example_file1, relative_example_file2)
, stop = TRUE
)
}

# Verify the md5sums as well as file access:
md5_check1 <- file_check(relative_example_file1, "7a3409e17f9de067740e64448a86e708")
md5_check1

# If you only need to verify a subset of md5sums then use an NA in the md5sums
# argument:
md5_check2 <-
  file_check(c(relative_example_file1, relative_example_file2),
             c("7a3409e17f9de067740e64448a86e708", NA))
md5_check2

# Verify all the md5sums
md5_check3 <-
  file_check(c(relative_example_file1, relative_example_file2),
             c("7a3409e17f9de067740e64448a86e708", "798e52b92e0ae0e60f3f3db1273235d0"))
md5_check3

# clean up working directory
unlink(relative_example_file1)
unlink(relative_example_file2)
unlink(absolute_example_file)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt.R
\name{Rpkg}
\alias{Rpkg}
\alias{CRANpkg}
\alias{Githubpkg}
\alias{GitHubpkg}
\alias{Gitlabpkg}
\title{Formatting Style on URLs for packages on CRAN, Github, and Gitlab.}
\usage{
Rpkg(pkg)

CRANpkg(pkg)

Githubpkg(pkg, username)

GitHubpkg(pkg, username)

Gitlabpkg(pkg, username)
}
\arguments{
\item{pkg}{The name of the package, will work as a quoted or raw name.}

\item{username}{username for Github.com or Gitlab.com}
}
\description{
Functions for controlling the look of package names in markdown created
vignettes and easy curating of URLs for the packages.
}
\examples{

Rpkg(qwraps2)
Rpkg("qwraps2")

CRANpkg(qwraps2)
CRANpkg("qwraps2")

Githubpkg(qwraps2, "dewittpe")
Githubpkg("qwraps2", dewittpe)

Gitlabpkg(qwraps2, "dewittpe")
Gitlabpkg("qwraps2", dewittpe)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spin.R
\name{spin_comments}
\alias{spin_comments}
\title{Spin Comment Check}
\usage{
spin_comments(hair, comment = c("^[# ]*/[*]", "^.*[*]/ *$"), text = NULL, ...)
}
\arguments{
\item{hair}{Path to the R script. The script must be encoded in UTF-8 if it
contains multi-byte characters.}

\item{comment}{A pair of regular expressions for the start and end delimiters
of comments; the lines between a start and an end delimiter will be
ignored. By default, the delimiters are \verb{/*} at the beginning of a
line, and \verb{*/} at the end, following the convention of C-style
comments.}

\item{text}{A character vector of code, as an alternative way to provide the
R source. If \code{text} is not \code{NULL}, \code{hair} will be ignored.}

\item{...}{additional arguments (not currently used.)}
}
\description{
A tool to help identify the opening and closing of comments in a spin
document.  This function is designed to help the user resolve the error
"comments must be put in pairs of start and end delimiters."
}
\examples{

spin_comments(hair = system.file("examples/spinner1.R", package = "qwraps2"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qkmplot.R
\name{qkmplot}
\alias{qkmplot}
\alias{qkmplot_build_data_frame}
\alias{qkmplot_build_data_frame.survfit}
\alias{qrmst}
\alias{qrmst.survfit}
\alias{qrmst.qkmplot_data}
\title{Kaplan-Meier Plot}
\usage{
qkmplot(x, conf_int = FALSE, ...)

qkmplot_build_data_frame(x)

\method{qkmplot_build_data_frame}{survfit}(x)

qrmst(x, tau = Inf)

\method{qrmst}{survfit}(x, tau = Inf)

\method{qrmst}{qkmplot_data}(x, tau = Inf)
}
\arguments{
\item{x}{object}

\item{conf_int}{logical if TRUE show the CI}

\item{...}{Other arguments passed to survival::plot.survfit}

\item{tau}{upper bound on time for restricted mean survival time estimate}
}
\value{
a ggplot.
}
\description{
A ggplot2 version of a Kaplan-Meier Plot
}
\details{
Functions to build data frames explicitly or implicitly and then create a
ggplot2 Kaplan-Meier plot. The misspelled
\code{qkmplot_bulid_data_frame} is retained as a deprecated
backward-compatible wrapper for \code{qkmplot_build_data_frame}.

More details and examples for graphics within qwraps2 are in the
vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
}
\examples{
if (requireNamespace("survival", quietly = TRUE)) {

  leukemia.surv <- survival::survfit(survival::Surv(time, status) ~ x,
                                     data = survival::aml)

  qkmplot(leukemia.surv, conf_int = TRUE)

  qkmplot_build_data_frame(leukemia.surv)

  qrmst(leukemia.surv) # NaN for rmst.se in Nonmaintained strata as last observation is an event
  qrmst(leukemia.surv, 44)

  # pbc examples
  pbc_fit <-
    survival::survfit(
        formula = survival::Surv(time, status > 0) ~ trt
      , data = survival::pbc
      , subset = !is.na(trt)
    )

  qkmplot(pbc_fit)
  qkmplot(pbc_fit, conf_int = TRUE)

  qrmst(pbc_fit)
  qrmst(pbc_fit)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qable.R
\name{qable}
\alias{qable}
\title{qable: an extended version of knitr::kable}
\usage{
qable(
  x,
  rtitle = "",
  rgroup = numeric(0),
  rnames = rownames(x),
  cnames = colnames(x),
  markup = getOption("qwraps2_markup", "latex"),
  kable_args = list(),
  ...
)
}
\arguments{
\item{x}{\code{matrix} or \code{data.frame} to be turned into a qable}

\item{rtitle}{a row grouping title. See Details.}

\item{rgroup}{a named numeric vector with the name of the row group and the
number of rows within the group.  \code{sum(rowgroup) == nrow(x)}.}

\item{rnames}{a character vector of the row names}

\item{cnames}{column names}

\item{markup}{the markup language to use expected to be either "markdown" or
"latex"}

\item{kable_args}{a list of named arguments to send to
\code{\link[knitr]{kable}}.  See Details.}

\item{...}{pass through}
}
\value{
\code{qable} returns a \code{qwraps2_qable} object that is just a character matrix with
some additional attributes and the print method returns, invisibly, the
object passed to print.
}
\description{
Create a simple table via \code{\link[knitr]{kable}} with row
groups and rownames similar to those of \code{\link[Hmisc]{latex}} from the
Hmisc package or \code{\link[htmlTable]{htmlTable}} from the htmlTable
package.
}
\details{
\code{rtitle} can be used to add a title to the column constructed by the
\code{rgroup} and \code{rnames}.  The basic layout of a table generated by
\code{qable} is:

\tabular{lcc}{
 rtitle    \tab cnames[1] \tab cnames[2] \cr
 rgroup[1] \tab           \tab           \cr
 rnames[1] \tab x[1, 1]   \tab x[1, 2]   \cr
 rnames[2] \tab x[2, 1]   \tab x[2, 2]   \cr
 rnames[3] \tab x[3, 1]   \tab x[3, 2]   \cr
 rgroup[2] \tab           \tab           \cr
 rnames[4] \tab x[4, 1]   \tab x[4, 1]   \cr
 rnames[5] \tab x[5, 1]   \tab x[5, 1]   \cr
}

Passing arguments to \code{\link[knitr]{kable}} is done via the list
\code{kable_args}.  This is an improvement in 0.6.0 to address arguments with
different use between qable and kable but the same name, notably
\code{format}.  Within the print method for \code{qwraps2_qable} objects,
some default arguments for knitr::kable are created.

Defaults if the named element of \code{kable_args} is missing:
\code{kable_args$format} will be "latex" if \code{markup = "latex"} and will
be \code{"pipe"} if \code{markup = "markdown"}.

\code{kable_args$escape = !(markup = "latex")}

\code{kable_args$row.names} defaults to \code{FALSE}

\code{kable_args$col.names} defaults to \code{colnames(x)}
}
\examples{
data(mtcars)
x <- qable(mtcars)
x
qable(mtcars, markup = "markdown")

# by make
make <- sub("^(\\\\w+)\\\\s?(.*)$", "\\\\1", rownames(mtcars))
make <- c(table(make))

# A LaTeX table with a vertical bar between each column
qable(mtcars[sort(rownames(mtcars)), ], rgroup = make)

# A LaTeX table with no vertical bars between columns
qable(mtcars[sort(rownames(mtcars)), ], rgroup = make, kable_args = list(vline = ""))

# a markdown table
qable(mtcars[sort(rownames(mtcars)), ], rgroup = make, markup = "markdown")

# define your own column names
qable(mtcars[sort(rownames(mtcars)), ],
      rgroup = make,
      cnames = toupper(colnames(mtcars)),
      markup = "markdown")

# define your own column names and add a title
qable(mtcars[sort(rownames(mtcars)), ],
      rtitle = "Make & Model",
      rgroup = make,
      cnames = toupper(colnames(mtcars)),
      markup = "markdown")

}
\seealso{
\code{\link[knitr]{kable}}

\code{\link{summary_table}}, for an example of building a data summary table.

For more detail on arguments you can pass via \code{kable_args} look at the
non-exported functions from the knitr package \code{knitr:::kable_latex},
\code{knitr:::kable_markdown}, or others.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qroc.R
\name{qroc-qprc}
\alias{qroc-qprc}
\alias{qroc}
\alias{qroc.default}
\alias{qroc.qwraps2_confusion_matrix}
\alias{qroc.glm}
\alias{qprc}
\alias{qprc.default}
\alias{qprc.qwraps2_confusion_matrix}
\alias{qprc.glm}
\title{Receiver Operating Characteristic and Precision-Recall Curves}
\usage{
qroc(x, ...)

\method{qroc}{default}(x, ...)

\method{qroc}{qwraps2_confusion_matrix}(x, ...)

\method{qroc}{glm}(x, ...)

qprc(x, ...)

\method{qprc}{default}(x, ...)

\method{qprc}{qwraps2_confusion_matrix}(x, ...)

\method{qprc}{glm}(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{pass through}
}
\value{
a ggplot.  Minimal aesthetics have been used so that the user may
modify the graphic as desired with ease.
}
\description{
Construction of ROC and PRC data and plots.
}
\details{
The area under the curve (AUC) is determined by a trapezoid approximation for
both the AUROC and AUPRC.

More details and examples for graphics within qwraps2 are in the
vignette(\dQuote{qwraps2-graphics}, package = \dQuote{qwraps2})
}
\examples{
#########################################################
# Example 1

df <-
  data.frame(
      truth = c(1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0)
    , pred  = c(1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0)
  )

cm <- confusion_matrix(df$truth, df$pred)
qroc(cm)
qprc(cm)

#########################################################
# Getting a ROC or PRC plot from a glm object:

mod <- glm(
  formula = spam ~ word_freq_our + word_freq_over + capital_run_length_total
, data = spambase
, family = binomial()
)

qroc(mod)
qprc(mod)

#########################################################
# View the vignette for more examples
\dontrun{
vignette("qwraps2-graphics")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean_sd.R
\name{mean_sd}
\alias{mean_sd}
\title{Mean and Standard deviation}
\usage{
mean_sd(
  x,
  digits = getOption("qwraps2_frmt_digits", 2),
  na_rm = FALSE,
  show_n = "ifNA",
  denote_sd = "pm",
  markup = getOption("qwraps2_markup", "latex"),
  ...
)
}
\arguments{
\item{x}{a numeric vector}

\item{digits}{digits to the right of the decimal point to return in the
percentage estimate.}

\item{na_rm}{if true, omit NA values}

\item{show_n}{defaults to "ifNA".  Other options are "always" or "never".}

\item{denote_sd}{a character string set to either "pm" or "paren" for reporting 'mean
\eqn{\pm} sd' or 'mean (sd)'}

\item{markup}{character string with value "latex" or "markdown"}

\item{...}{pass through}
}
\value{
a character vector of the formatted values
}
\description{
A function for calculating and formatting means and
standard deviations.
}
\details{
Given a numeric vector, \code{mean_sd} will return a character string with
the mean and standard deviation.  Formatting of the output will be extended in
future versions.
}
\examples{
set.seed(42)
x <- rnorm(1000, 3, 4)
mean(x)
sd(x)
mean_sd(x)
mean_sd(x, show_n = "always")
mean_sd(x, show_n = "always", denote_sd = "paren")

x[187] <- NA
mean_sd(x, na_rm = TRUE)

}
\seealso{
\code{\link{gmean_sd}}, \code{\link{n_perc}},
\code{\link{median_iqr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{\%s\%}
\alias{\%s\%}
\title{Operators}
\usage{
e1 \%s\% e2
}
\arguments{
\item{e1}{a character string}

\item{e2}{a character string}
}
\description{
A set of helpful operators to make writing and basic data analysis easier.
}
\examples{

# base R
paste0("A longer string ", "can be ", "built")

# with the \%s\% operator
"A longer string " \%s\% "can be " \%s\% "built"

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt.R
\name{backtick}
\alias{backtick}
\title{Backtick}
\usage{
backtick(x, dequote = FALSE)
}
\arguments{
\item{x}{the thing to be deparsed and encapsulated in backticks}

\item{dequote}{remove the first and last double or signal quote form \code{x}}
}
\description{
Encapsulate a string in backticks. Very helpful for inline code in
\code{\link[knitr]{spin}} scripts.
}
\examples{
backtick("a quoted string")
backtick(no-quote)
backtick(noquote)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_perc.R
\name{n_perc}
\alias{n_perc}
\alias{perc_n}
\alias{n_perc0}
\title{Count and Percentage}
\usage{
n_perc(
  x,
  digits = getOption("qwraps2_frmt_digits", 2),
  na_rm = FALSE,
  show_denom = "ifNA",
  show_symbol = TRUE,
  markup = getOption("qwraps2_markup", "latex"),
  ...
)

perc_n(
  x,
  digits = getOption("qwraps2_frmt_digits", 2),
  na_rm = FALSE,
  show_denom = "ifNA",
  show_symbol = FALSE,
  markup = getOption("qwraps2_markup", "latex"),
  ...
)

n_perc0(
  x,
  digits = 0,
  na_rm = FALSE,
  show_denom = "never",
  show_symbol = FALSE,
  markup = getOption("qwraps2_markup", "latex"),
  ...
)
}
\arguments{
\item{x}{a 0:1 or boolean vector}

\item{digits}{digits to the right of the decimal point to return in the
percentage estimate.}

\item{na_rm}{if true, omit NA values}

\item{show_denom}{defaults to "ifNA".  Other options are "always" or "never".}

\item{show_symbol}{if TRUE (default) the percent symbol is shown, else it is suppressed.}

\item{markup}{latex or markdown}

\item{...}{pass through}
}
\value{
a character vector of the formatted values
}
\description{
A function for calculating and formatting counts and
percentages.
}
\details{
Default behavior will return the count of successes and the percentage as "N
(pp%).  If there are missing values in the input vector the omission of such
can be controlled by setting \code{na.rm = TRUE}.  In this case, the number
of non-missing values will be reported by default.  Omission of the
non-missing values can be controlled by setting \code{show_denom = "never"}.

The function \code{n_perc0} uses a set of default arguments which may be
advantageous for use in building tables.
}
\examples{

n_perc(c(0, 1,1, 1, 0, 0), show_denom = "always")
n_perc(c(0, 1,1, 1, 0, 0, NA), na_rm = TRUE)

n_perc(mtcars$cyl == 6)

set.seed(42)
x <- rbinom(4269, 1, 0.314)
n_perc(x)
n_perc(x, show_denom = "always")
n_perc(x, show_symbol = FALSE)

# n_perc0 examples
n_perc0(c(0, 1,1, 1, 0, 0))
n_perc0(mtcars$cyl == 6)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ll.R
\name{ll}
\alias{ll}
\title{List Object Aliases}
\usage{
ll(
  pos = 1,
  pattern,
  order_by = "size",
  decreasing = order_by \%in\% c("size", "rows", "columns")
)
}
\arguments{
\item{pos}{specifies the environment as a position in the search list}

\item{pattern}{an optional regular expression.  Only names matching
\code{pattern} are returned.  \code{\link[utils]{glob2rx}} can be used to
convert wildcard patterns to regular expressions.}

\item{order_by}{a character, order the results by \dQuote{object},
\dQuote{size} (default), \dQuote{class}, \dQuote{rows}, or \dQuote{columns}.}

\item{decreasing}{logical, defaults to \code{TRUE}, decreasing order? passed
to \code{\link[base]{order}}.}
}
\value{
a data.frame with columns
\itemize{
 \item object: name of the object
 \item class: class, or mode if class is not present, of the object
 \item size: approximate size, in bytes, of the object in memory
 \item rows: number of rows for data.frames or matrices, or the number of
 elements for a list like structure
 \item columns: number of columns for data.frames or matrices
}
}
\description{
Aliases for \code{\link[base]{ls}} providing additional details.
}
\examples{
# View your current workspace
\dontrun{
ls()
ll()
}

# View another environment
e <- new.env()
ll(e)

e$fit <- lm(mpg ~ wt, mtcars)
e$fit2 <- lm(mpg ~ wt + am + vs, data = mtcars)
e$x <- rnorm(1e5)
e$y <- runif(1e4)
e$z <- with(e, x * y)
e$w <- sum(e$z)

ls(e)
ll(e)

}
\references{
The basis for this work came from a Stack Overflow posting:
\url{https://stackoverflow.com/q/1358003/1104685}
}
\seealso{
\code{\link[base]{ls}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frmt.R
\name{frmt}
\alias{frmt}
\alias{frmtp}
\alias{frmtci}
\title{Format Wrappers}
\usage{
frmt(x, digits = getOption("qwraps2_frmt_digits", 2), append = NULL)

frmtp(
  x,
  style = getOption("qwraps2_journal", "default"),
  digits = getOption("qwraps2_frmtp_digits", 4),
  markup = getOption("qwraps2_markup", "latex"),
  case = getOption("qwraps2_frmtp_case", "upper"),
  leading0 = getOption("qwraps2_frmtp_leading0", TRUE)
)

frmtci(
  x,
  est = 1,
  lcl = 2,
  ucl = 3,
  format = "est (lcl, ucl)",
  show_level = FALSE,
  ...
)
}
\arguments{
\item{x}{a vector of numbers or a numeric matrix to format.}

\item{digits}{number of digits, including trailing zeros, to the right of the
decimal point.  This option is ignored if \code{is.integer(x) == TRUE)}.}

\item{append}{a character string to append to the formatted number.  This is
particularly useful for percentages or adding punctuation to the end of the
formatted number.  This should be a vector of length 1, or equal to the
length of \code{x}.}

\item{style}{a character string indicating a specific journal requirements
for p-value formatting.}

\item{markup}{a character string indicating if the output should be latex or
markup.}

\item{case}{a character string indicating if the output should be upper case
or lower case.}

\item{leading0}{boolean, whether or not the p-value should be reported as
0.0123 (TRUE, default), or .0123 (FALSE).}

\item{est}{the numeric index of the vector element or the matrix column
containing the point estimate.}

\item{lcl}{the numeric index of the vector element or the matrix column
containing the lower confidence limit.}

\item{ucl}{the numeric index of the vector element or the matrix column
containing the upper confidence limit.}

\item{format}{a string with "est" "lcl", and "ucl" to denote the location of
the estimate, lower confidence limit, and upper confidence limit for the
formatted string.  Defaults to "est (lcl, ucl)".}

\item{show_level}{defaults to FALSE.  If TRUE and \code{format} is the
default, then
"100*(1-options()$qwraps2_alpha)% CI:" will be placed between the left
parenthesis and the lcl.  If set to a string, then the given string will be
placed between the left parenthesis and the lcl.  If the \code{format} is not
the default, then this argument is ignored.}

\item{...}{args passed to frmt}
}
\value{
a character vector of the formatted numbers
}
\description{
Functions for formatting numeric values for consistent display
in reports.
}
\details{
`frmt` was originally really just a wrapper for the \code{formatC}.  It has
extended functionality now as I have found common uses cases.

`frmtp` formats P-values per journal
requirements.  As I work on papers aimed at different journals, the
formatting functions will be extended to match.

Default settings are controlled through the function arguments but should be
set via \code{options()}.

Default settings report the P-value exactly if P >
\code{getOption("qwraps2_frmtp_digits", 4)} and reports
P < \code{10^-(getOption("qwraps2_frmtp_digits", 2))} otherwise.  By the
leading zero is controlled via
\code{getOption("qwraps2_frmtp_leading0", TRUE)}
and a upper or lower case P is controlled by
\code{getOption("qwraps2_frmtp_case", "upper")}.  These options are ignored
if \code{style != "default"}.

Journals with predefined P-value formatting are noted in the
\pkg{\link{qwraps2}} documentation.

`frmtci` takes a \code{matrix}, or \code{data.frame}, with a point estimate
and the lcl and ucl and formats a string for reporting.  est (lcl, ucl) is
the default.  The confidence level can be added to the string, e.g., "est
(95% CI: lcl, ucl)" with \code{show_level = TRUE} or by setting your own
format.

`frmtcip` expects four values, est, lcl, ucl, and p-value.  The resulting
sting will be of the form "est (lcl, ucl; p-value)".

The `Rpkg`, `CRANpkg`, and `Githubpkg` functions are used to help make
documenting packages stylistically consistent and with valid urls.  These
functions were inspired by similar ones found in the BioConductor BiocStyle
package.
}
\examples{

### Formatting numbers
integers <- c(1234L, 9861230L)
numbers  <- c(1234,  9861230)
frmt(integers)  # no decimal point
frmt(numbers)   # decimal point and zeros to the right

numbers <- c(0.1234, 0.1, 1234.4321, 0.365, 0.375)
frmt(numbers)

# reporting a percentage
frmt(17/19 * 100, digits = 2, append = "\%")   # good for markdown
frmt(17/19 * 100, digits = 2, append = "\\\\\%") # good for LaTeX

# append one character
frmt(c(1, 2, 3)/19 * 100, digits = 2, append = "\%")

# append different characters
frmt(c(1, 2, 3)/19 * 100, digits = 2, append = c("\%;", "\%!", "\%."))

### Formatting p-values
ps <- c(0.2, 0.001, 0.00092, 0.047, 0.034781, 0.0000872, 0.787, 0.05, 0.043)
# LaTeX is the default markup language
cbind("raw"      = ps,
      "default"  = frmtp(ps),
      "3lower"   = frmtp(ps, digits = 3, case = "lower"),
      "PediDent" = frmtp(ps, style = "pediatric_dentistry"))

### Using markdown
cbind("raw"      = ps,
      "default"  = frmtp(ps, markup = "markdown"),
      "3lower"   = frmtp(ps, digits = 3, case = "lower", markup = "markdown"),
      "PediDent" = frmtp(ps, style = "pediatric_dentistry", markup = "markdown"))

# Formatting the point estimate and confidence interval
# for a set of three values
temp <- c(a = 1.23, b = .32, CC = 1.78)
frmtci(temp)

# show level uses getOption("qwraps2_alpha", 0.05)
frmtci(temp, show_level = TRUE)

# note that the show_level will be ignored in the following
frmtci(temp, format = "est ***lcl, ucl***", show_level = TRUE)

# show_level as a character
frmtci(temp, show_level = "confidence between: ")

# For a matrix: the numbers in this example don't mean anything, but the
# formatting should.
temp2 <- matrix(rnorm(12), nrow = 4,
                dimnames = list(c("A", "B", "C", "D"), c("EST", "LOW", "HIGH")))
temp2
frmtci(temp2)

# similar for a data.frame
df2 <- as.data.frame(temp2)
frmtci(df2)

}
\seealso{
\code{\link{formatC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/median_iqr.R
\name{median_iqr}
\alias{median_iqr}
\title{Median and Inner Quartile Range}
\usage{
median_iqr(
  x,
  digits = getOption("qwraps2_frmt_digits", 2),
  na_rm = FALSE,
  show_n = "ifNA",
  markup = getOption("qwraps2_markup", "latex"),
  ...
)
}
\arguments{
\item{x}{a numeric vector}

\item{digits}{digits to the right of the decimal point to return.}

\item{na_rm}{if true, omit NA values}

\item{show_n}{defaults to "ifNA".  Other options are "always" or "never".}

\item{markup}{latex or markdown}

\item{...}{pass through}
}
\value{
a character vector of the formatted values
}
\description{
A function for calculating and formatting the median and
inner quartile range of a data vector.
}
\details{
Given a numeric vector, \code{median_iqr} will return a character string with
the median and IQR.  Formatting of the output will be extended in
future versions.
}
\examples{
set.seed(42)
x <- rnorm(1000, 3, 4)
median(x)
quantile(x, probs = c(1, 3)/4)
median_iqr(x)
median_iqr(x, show_n = "always")

x[187] <- NA
# median_iqr(x) ## Will error
median_iqr(x, na_rm = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{spambase}
\alias{spambase}
\title{Spambase}
\format{
a data.frame with 4601 rows, 58 columns; 57 features and 0/1
indicator for spam

Used under CC BY 4.0 license.
}
\usage{
spambase
}
\description{
Classifying Email as Spam or Non-Spam
}
\references{
Hopkins,Mark, Reeber,Erik, Forman,George, and Suermondt,Jaap. (1999). Spambase. UCI Machine Learning Repository. https://doi.org/10.24432/C53G6X.
}
\seealso{
\code{vignette("qwraps2-data-sets", package = "qwraps2")} for
details on the construction of the data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazyload_cache.R
\name{lazyload_cache_dir}
\alias{lazyload_cache_dir}
\alias{lazyload_cache_labels}
\title{Lazyload Cache}
\usage{
lazyload_cache_dir(
  path = "./cache",
  envir = parent.frame(),
  ask = FALSE,
  verbose = TRUE,
  ...
)

lazyload_cache_labels(
  labels,
  path = "./cache/",
  envir = parent.frame(),
  verbose = TRUE,
  filter,
  ...
)
}
\arguments{
\item{path}{the path to the cache directory.}

\item{envir}{the environment to load the objects into}

\item{ask}{if TRUE ask the user to confirm loading each database found in
\code{path}}

\item{verbose}{if TRUE display the chunk labels being loaded}

\item{...}{additional arguments passed to \code{\link[base]{list.files}}.}

\item{labels}{a character vector of the chunk labels to load.}

\item{filter}{an optional function passed to \code{\link[base:lazyload]{lazyLoad}}.
when called on a character vector of object names returns a logical vector:
only objects for which this is true will be loaded.}
}
\description{
Lazyload Cached label(s) or a whole directory.
}
\details{
These functions are helpful for loading cached chunks into an interactive R
session.  Consider the following scenario: you use knitr and have cached
chunks for lazyloading.  You've created the document, close up your IDE and
move on to the next project.  Later, you revisit the initial project and need
to retrieve the objects created in the cached chunks.  One option is to
reevaluate all the code, but this could be time consuming.  The other option
is to use \code{lazyload_cache_labels} or \code{lazyload_cache_dir} to
quickly (lazy)load the chunks into an active R session.

Use \code{lazyload_cache_dir} to load a whole directory of cached objects.

Use \code{lazyload_cache_labels} to load an explicit set of cached chunks.
}
\examples{
# this example is based on \url{https://stackoverflow.com/a/41439691/1104685}

# create a temp directory for a and place a .Rmd file within
tmpdir <- normalizePath(paste0(tempdir(), "/llcache_eg"), mustWork = FALSE)
tmprmd <- tempfile(pattern = "report", tmpdir = tmpdir, fileext = "Rmd")
dir.create(tmpdir)
oldwd <- getwd()
setwd(tmpdir)

# build an example .Rmd file
# note that the variable x is created in the first chunk and then overwritten
# in the second chunk
cat("---",
    "title: \"A Report\"",
    "output: html_document",
    "---",
    "",
    "```{r first-chunk, cache = TRUE}",
    "mpg_by_wt_hp <- lm(mpg ~ wt + hp, data = mtcars)",
    "x_is_pi <- pi",
    "x <- pi",
    "```",
    "",
    "```{r second-chunk, cache = TRUE}",
    "mpg_by_wt_hp_am <- lm(mpg ~ wt + hp + am, data = mtcars)",
    "x_is_e <- exp(1)",
    "x <- exp(1)",
    "```",
    sep = "\n",
    file = tmprmd)

# knit the file.  evaluate the chunks in a new environment so we can compare
# the objects after loading the cache.
kenv <- new.env()
knitr::knit(input = tmprmd, envir = kenv)

# The objects defined in the .Rmd file are now in kenv
ls(envir = kenv)

# view the cache
list.files(path = tmpdir, recursive = TRUE)

# create three more environment, and load only the first chunk into the
# first, and the second chunk into the second, and then load all of the
# cache into the third
env1 <- new.env()
env2 <- new.env()
env3 <- new.env()

lazyload_cache_labels(labels = "first-chunk",
                      path = paste0(tmpdir, "/cache"),
                      envir = env1)

lazyload_cache_labels(labels = "second-chunk",
                      path = paste0(tmpdir, "/cache"),
                      envir = env2)

lazyload_cache_dir(path = paste0(tmpdir, "/cache"), envir = env3)

# Look at the contents of each of these environments
ls(envir = kenv)
ls(envir = env1)
ls(envir = env2)
ls(envir = env3)

# The regression models are only fitted once and should be the same in all the
# environments where they exist, as should the variables x_is_e and x_is_pi
all.equal(kenv$mpg_by_wt_hp, env1$mpg_by_wt_hp)
all.equal(env1$mpg_by_wt_hp, env3$mpg_by_wt_hp)

all.equal(kenv$mpg_by_wt_hp_am, env2$mpg_by_wt_hp_am)
all.equal(env2$mpg_by_wt_hp_am, env3$mpg_by_wt_hp_am)

# The value of x, however, should be different in the different
# environments.  For kenv, env2, and env3 the value should be exp(1) as that
# was the last assignment value.  In env1 the value should be pi as that is
# the only relevant assignment.

all.equal(kenv$x, exp(1))
all.equal(env1$x, pi)
all.equal(env2$x, exp(1))
all.equal(env3$x, exp(1))

# cleanup
setwd(oldwd)
unlink(tmpdir, recursive = TRUE)

}
